@@ -193,12 +193,13 @@ class Field:
       layoutTarget=field">:field.getPx(minimal, \
                                        hostLayout or layoutType, _ctx_)</x>''')
 
-    def doRender(self, layoutType, obj, name=None, minimal=False):
+    def doRender(self, layoutType, obj, name=None, minimal=False, tagCss=None):
         '''Allows to call pxRender from code, to display the content of this
            field on p_obj in some specific context, for example in a Computed
            field.'''
         context = {'layoutType': layoutType, 'field': self, 'minimal': minimal,
                    'name': name or self.name, 'obj': obj, 'zobj': obj.o}
+        if tagCss: context['tagCss'] = tagCss
         # Get the current PX context: we will need to copy some entries from it.
         # If no such context exists, create one.
         req = obj.request
