@@ -15,11 +15,13 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
-import time, os.path, mimetypes, shutil, base64
+from DateTime import DateTime
 from cStringIO import StringIO
+import time, os.path, mimetypes, shutil, base64
+
+from appy.px import Px
 from appy import Object
 from appy.fields import Field, Layouts
-from appy.px import Px
 from appy.shared import utils as sutils
 from appy.shared import UnmarshalledFile, mimeTypesExts
 
@@ -28,6 +30,7 @@ WRONG_FILE_TUPLE = 'This is not the way to set a file. You can specify a ' \
     '2-tuple (fileName, fileContent) or a 3-tuple (fileName, fileContent, ' \
     'mimeType).'
 CONVERSION_ERROR = 'An error occurred. %s'
+RESIZED = '%s resized to %spx.'
 
 def guessMimeType(fileName):
     '''Try to find the MIME type of file p_fileName'''
@@ -88,16 +91,20 @@ class FileInfo:
         self.size = 0 # Its size, in bytes
         self.mimeType = None # Its MIME type
         self.modified = None # The last modification date for this file
-        # Complete metadata if p_inDb is False
+        # Complete metadata if p_inDb is False. p_inDb is not stored as is:
+        # checking if self.fsName is the empty string is equivalent.
         if not inDb:
             self.fsName = '' # Already included in self.fsPath
-            # We will not store p_inDb. Checking if self.fsName is the empty
-            # string is equivalent.
-            fileInfo = os.stat(self.fsPath)
-            self.size = fileInfo.st_size
-            self.mimeType = guessMimeType(self.fsPath)
-            from DateTime import DateTime
-            self.modified = DateTime(fileInfo.st_mtime)
+            self.setAttributes(self.fsPath)
+
+    def setAttributes(self, path):
+        '''Compute file attributes (size, MIME type and last modification date)
+           for a file whose absolute p_path is passed and store these attributes
+           on p_self.'''
+        info = os.stat(path)
+        self.size = info.st_size
+        self.mimeType = guessMimeType(path)
+        self.modified = DateTime(info.st_mtime)
 
     def getFilePath(self, obj):
         '''Returns the absolute file name of the file on disk that corresponds
@@ -349,9 +356,26 @@ class FileInfo:
                 return
         return filePath
 
+    def resize(self, folder, width, obj):
+        '''Resize this image if it is a resizable image'''
+        if self.mimeType not in File.resizableImages: return
+        # Get the absolute path to the file on disk
+        path = self.getFilePath(obj)
+        # Get the width, in pixels, that will be used to resize it
+        width = sutils.keepDigits(str(width))
+        sutils.convert(path, '-resize %sx%s>' % (width, width))
+        obj.log(RESIZED % (path, width))
+        # (re)compute p_self's attributes, after resizing
+        self.setAttributes(path)
+
 # ------------------------------------------------------------------------------
 class File(Field):
 
+    # MIME types for images
+    imageMimeTypes = ('image/png', 'image/jpeg', 'image/gif')
+    # MIME types for resizable images
+    resizableImages = imageMimeTypes
+
     pxView = pxCell = Px('''
       <x>::field.getDownloadLink(name, zobj, layoutType)</x>''')
 
@@ -391,12 +415,12 @@ class File(Field):
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
       defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
       move=0, specificReadPermission=False, specificWritePermission=False,
-      width=None, height=None, maxChars=None, colspan=1, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, isImage=False, downloadAction=None,
-      sdefault='', scolspan=1, swidth=None, sheight=None, view=None, cell=None,
-      edit=None, xml=None, translations=None, render=None, icon='paperclip',
-      nameStorer=None):
+      width=None, height=None, inputWidth=None, maxChars=None, colspan=1,
+      master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, isImage=False,
+      downloadAction=None, sdefault='', scolspan=1, swidth=None, sheight=None,
+      view=None, cell=None, edit=None, xml=None, translations=None, render=None,
+      icon='paperclip', nameStorer=None, resize=False):
         # This boolean is True if the file is an image
         self.isImage = isImage
         # "downloadAction" can be a method called every time the file is
@@ -414,15 +438,17 @@ class File(Field):
         # its name will be copied into the nameStorer field, without the
         # extension, only if this latter is not empty.
         self.nameStorer = nameStorer
-        # Call the base constructor. Regarding attribute "width":
-        # ----------------------------------------------------------------------
-        #  if used for... | it represents...
-        # ----------------------------------------------------------------------
-        #    a file       | the width of the input field allowing to upload the
-        #                 | file;
-        #   an image      | the maximum width for rendering the image in the
-        #                 | web UI. It will not resize the uploaded image.
-        # ----------------------------------------------------------------------
+        # Attribute "width" is used to specify the width of the image or
+        # document. The width of the input field allowing to upload the file can
+        # be defined in attribute "inputWidth".
+        self.inputWidth = inputWidth
+        # If attribute "resize" is False (the default), the image will be
+        # previewed in the UI with dimensions as defined in attributes "width"
+        # and "height", but without being effectively resized. If "resize" is
+        # True, when uploading the file, it will be resized to self.width,
+        # keeping the width / height ratio.
+        self.resize = resize
+        # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
           show, page, group, layouts, move, False, True, None, False,
           specificReadPermission, specificWritePermission, width, height, None,
@@ -468,8 +494,8 @@ class File(Field):
     def getStyle(self):
         '''Get the content of the "style" attribute of the "input" tag on the
            "edit" layout for this field.'''
-        if self.width:
-            return 'width: %s' % self.width
+        if self.inputWidth:
+            return 'width: %s' % self.inputWidth
         return ''
 
     def isRenderable(self, layoutType):
@@ -592,6 +618,8 @@ class File(Field):
                 mimeType = mimeType or guessMimeType(fileName)
                 info.writeFile(self.name, (fileName, fileContent, mimeType),
                                dbFolder)
+            # Resize the image when relevant
+            if self.resize and self.width: info.resize(folder, self.width, obj)
             # Store the FileInfo instance in the database
             setattr(obj, self.name, info)
         else:
