@@ -215,7 +215,7 @@ class Pod(Field):
       cell=None, edit=None, xml=None, translations=None, downloadName=None,
       downloadDisposition='attachment', forceOoCall=False,
       optimalColumnWidths=False, distributeColumns=None, script=None,
-      pdfOptions='ExportNotes=True', tabbedCR=False, confirm=False,
+      pdfOptions='ExportNotes=True', tabbedCR=False, fonts=None, confirm=False,
       raiseOnError=False, action=None, beforeAction=None):
         # Param "template" stores the path to the pod template(s). If there is
         # a single template, a string is expected. Else, a list or tuple of
@@ -243,7 +243,7 @@ class Pod(Field):
         self.templateName = templateName
         # If you want to use "templateName" hereabove even if self.template
         # contains a single template, set "usetTemplateName" to True.
-        if useTemplateName == None:
+        if useTemplateName is None:
             self.useTemplateName = len(self.template) > 1
         else:
             self.useTemplateName = useTemplateName
@@ -402,6 +402,11 @@ class Pod(Field):
         # "tabbedCR" determines how the "carrage return" char is rendered on
         # POD results. See appy.pod.renderer.py for more information.
         self.tabbedCR = tabbedCR
+        # The name of the font passed in "fonts" will be injected in all styles
+        # from the POD template. It must return a string or be a function that
+        # will accept the current template (one among self.template) as unique
+        # arg and return the name of the font to inject in it.
+        self.fonts = fonts
         # If "confirm" is True, a popup will be shown before generating the
         # pod result. "confirm" may also hold a method, that must accept, as
         # single arg, the name of the current template (indeed, self.template
@@ -732,10 +737,11 @@ class Pod(Field):
         script = self.script
         if callable(script): script = script(obj, template, podContext)
         # Compute PDF options
-        if callable(self.pdfOptions):
-            pdfOptions = self.pdfOptions(obj, template)
-        else:
-            pdfOptions = self.pdfOptions
+        pdfOptions = self.pdfOptions
+        if callable(pdfOptions): pdfOptions = pdfOptions(obj, template)
+        # Compute fonts
+        fonts = self.fonts
+        if callable(fonts): fonts = fonts(obj, template)
         # Compute the renderer's parameters
         rendererParams = {'template': templatePath, 'context': podContext,
           'result': result, 'stylesMapping': stylesMapping,
@@ -743,7 +749,7 @@ class Pod(Field):
           'forceOoCall': self.forceOoCall, 'raiseOnError': self.raiseOnError,
           'optimalColumnWidths': self.optimalColumnWidths,
           'distributeColumns': self.distributeColumns, 'script': script,
-          'pdfOptions': pdfOptions, 'tabbedCR': self.tabbedCR}
+          'pdfOptions': pdfOptions, 'tabbedCR': self.tabbedCR, 'fonts': fonts}
         # Add parameters regarding the Python interpreter to use and LO port
         cfg = ztool.getProductConfig(True)
         if cfg.unoEnabledPython:
