@@ -378,6 +378,9 @@ class Layouts:
         # The "long" version of the previous layout (if the description is
         # long), with vertical alignment on top instead of middle.
         gdl = {'view': 'fl', 'edit': Table('f;dv=', width=None)}
+        # Description, renderes as *r*adio buttons
+        gdr = {'view': 'fl', 'edit': Table('d-fv=', width=None)}
+        rld = {'view': 'lf', 'edit': 'l-d-f'}
 
     class Calendar:
         b = {'edit': 'f', 'view': 'l-d-f'}
