@@ -183,13 +183,19 @@ function getNode(id, forceTop) {
   var container = window.document,
       startIndex = 0;
   if (id[0] == ':') {
-    container = window.top.document;
+    if (window.frameElement && (window.frameElement.id == 'subIFrame')) {
+      container = window.parent.document;
+    }
+    else { container = window.top.document }
     startIndex = 1;
   }
   var nodeId = id.slice(startIndex),
-      res = container.getElementById(nodeId);
-  if (!res && forceTop) res = window.top.document.getElementById(nodeId);
-  return res;
+      r = container.getElementById(nodeId);
+  if (!r && forceTop) {
+    r = window.parent.document.getElementById(nodeId) ||
+        window.top.document.getElementById(nodeId);
+  }
+  return r;
 }
 
 function getAjaxChunk(pos) {
@@ -801,9 +807,10 @@ function getMasterValues(master) {
     /* Get the selected value among all radio buttons of the group (p_master is
        the first one from this group) */
     var radios = document.getElementsByName(master.name);
+    res = [];
     for (var i=0; i < radios.length; i++) {
       if (radios[i].checked) {
-        res = [radios[i].value];
+        res.push(radios[i].value);
         break;
       }
     }
@@ -1340,11 +1347,13 @@ function openPopup(popupId, msg, width, height, back, commentLabel) {
   }
   // Open the popup
   var popup = document.getElementById(popupId),
-      frame = popupId == 'iframePopup'; // Is it the "iframe" popup ?
+      isSub = (popupId == 'iframeSub'),
+      frame = isSub || (popupId == 'iframePopup'), // Is it an "iframe" popup ?
+      delta = (isSub)? 100: 200;
   /* Define height and width. For non-iframe popups, do not set its height: it
      will depend on its content */
-  if (!width)  { width =  (frame)? window.innerWidth -200: 350 }
-  if (!height) { height = (frame)? window.innerHeight-200: null }
+  if (!width)  { width =  (frame)? window.innerWidth -delta: 350 }
+  if (!height) { height = (frame)? window.innerHeight-delta: null }
   // Take into account the visible part of the browser window
   var top = (height)?
             (window.innerHeight - height) / 2: window.innerHeight * 0.25;
@@ -1355,7 +1364,8 @@ function openPopup(popupId, msg, width, height, back, commentLabel) {
   popup.style.left = ((window.innerWidth - width) / 2).toFixed() + 'px';
   if (frame) {
     // Set the enclosed iframe dimensions
-    var iframe = document.getElementById('appyIFrame');
+    var iname = (popupId == 'iframeSub')? 'subIFrame': 'appyIFrame';
+    iframe = document.getElementById(iname);
     iframe.style.width = (width - 20).toFixed() + 'px';
     iframe.style.height = height.toFixed() + 'px';
     popup['back'] = back;
@@ -1365,9 +1375,16 @@ function openPopup(popupId, msg, width, height, back, commentLabel) {
 
 function closePopup(popupId, clean, tryCancel) {
   // Get the popup
-  var container = (popupId == 'iframePopup') ?
-                  window.parent.document: window.document,
-      popup = container.getElementById(popupId);
+  var container = null;
+  if (popupId == 'iframePopup') {
+    container = window.parent.document;
+  }
+  else if (popupId == ':iframeSub') {
+    container = window.parent.document;
+    popupId = 'iframeSub';
+  }
+  else { container = window.document; }
+  var popup = container.getElementById(popupId);
   // Close the popup
   popup.style.display = 'none';
   popup.style.width = null;
@@ -1708,7 +1725,10 @@ function onSelectObjects(popupId, initiatorId, objectUrl, mode, onav,
     return;
   }
   // Close the popup
-  closePopup('iframePopup');
+  var popupId = window.frameElement.parentNode.id,
+      isSub = popupId == 'iframeSub';
+  popupId = (isSub)? ':'+popupId: popupId;
+  closePopup(popupId);
   /* When refreshing the Ref field we will need to pass all those parameters,
      for replaying the popup query. */
   var params = {'selected': uids, 'semantics': semantics, 'sortKey': sortKey,
@@ -1726,7 +1746,7 @@ function onSelectObjects(popupId, initiatorId, objectUrl, mode, onav,
   }
 }
 
-function onSelectObject(checkboxId, initiatorId, objectUrl) {
+function onSelectObject(checkboxId, initiatorId, objectUrl, popupId) {
   /* In a Ref field with link="popup", a single object has been clicked. If
      multiple objects can be selected, simply update the corresponding checkbox
      status. Else, close the popup and return the selected object. */
@@ -1737,9 +1757,13 @@ function onSelectObject(checkboxId, initiatorId, objectUrl) {
   else {
     /* Close the popup and directly refresh the initiator field with the
        selected object. */
-    var uids = checkbox.value;
-    closePopup('iframePopup');
+    var uids = checkbox.value,
+        popupId = window.frameElement.parentNode.id;
+        isSub = popupId == 'iframeSub';
+    popupId = (isSub)? ':'+popupId: popupId;
+    closePopup(popupId);
     var params = {'selected': uids, 'semantics': 'checked'};
+    if (isSub) params['popup'] = '1';
     askField(':'+initiatorId, objectUrl, 'edit', params, false);
   }
 }
