@@ -168,13 +168,15 @@ class AbstractWrapper(object):
       </div>
 
       <!-- Popup containing the Appy iframe -->
-      <div id="iframePopup" class="popup" if="not inPopup"
+      <div var="popupDiv=inPopup and 'iframeSub' or 'iframePopup'"
+           id=":popupDiv" class="popup"
            onmousedown="dragStart(event)" onmouseup="dragStop(event)"
            onmousemove="dragIt(event)"
            onmouseover="dragPropose(event)" onmouseout="dragStop(event)">
        <img align=":dright" src=":url('close')" class="clickable"
-            onclick="closePopup('iframePopup',null,true)"/>
-       <iframe id="appyIFrame" name="appyIFrame" frameborder="0"></iframe>
+            onclick=":'closePopup(%s,null,true)' % q(popupDiv)"/>
+       <iframe var="popupFrame=inPopup and 'subIFrame' or 'appyIFrame'"
+               id=":popupFrame" name=":popupFrame" frameborder="0"></iframe>
       </div>
 
       <div class=":cfg.ui.getClass('main', wide, _px_, _ctx_)">
@@ -197,8 +199,7 @@ class AbstractWrapper(object):
          <!-- Custom links -->
          <x>:tool.pxLinks</x>
          <!-- Top-level pages -->
-         <a for="page in tool.pages" if="page.o.mayView()" class="pageLink"
-            href=":page.url">:page.title</a>
+         <x>:tool.pxPageSelector</x>
          <!-- Connect link if discreet login -->
          <a if="isAnon and cfg.discreetLogin" id="loginLink" name="loginLink"
             onclick="toggleLoginForm(true)" class="pageLink clickable">
@@ -1062,39 +1063,49 @@ class AbstractWrapper(object):
         req['nav'] = nav
         return appyObj
 
-    def createFrom(self, fieldNameOrClass, other, noSecurity=False,
-                   executeMethods=True, exclude=(), keepBase=False, fields={}):
-        '''Similar to m_create above, excepted that we will use another object
-           (p_other) as base for filling in data for the object to create.
-           p_exclude can list fields (by their names) that will not be copied on
-           p_other. If p_keepBase is True, basic attributes will be kept on the
-           new object: creator and dates "created" and "modified". Else, the
-           new object's creator will be the logged user.
-
-           Note that this method does not perform a deep copy: objects linked
-           via Ref fields from p_self will be referenced by the clone, but not
-           themselves copied.'''
-        # Get the field values to set from p_other and store it in a dict.
-        # p_other may not be of the same class as p_self.
-        params = {}
-        for field in other.fields:
-            # Skip non persistent fields, back references and p_excluded fields
+    def getCopyValues(self, exclude=(), fields={}):
+        '''Returns a dict ~{s_fieldName, fieldValue}~ representing p_self's
+           dataset ready-to-be-copied to another object.'''
+        # Check m_createFrom below to know more about method args
+        # ~~~
+        r = {}
+        # Browse field values from p_self and store "copyable" ones in "r"
+        for field in self.fields:
+            # Skip non persistent fields, back references & p_exclude(d) fields
             if not field.persist or (field.name in exclude) or \
                ((field.type == 'Ref') and field.isBack): continue
             # Use the value in p_fields if given
             if field.name in fields:
                 value = fields[field.name]
             else:
-                value = field.getCopyValue(other.o)
-            params[field.name] = value
-        res = self.create(fieldNameOrClass, noSecurity=noSecurity,
-                          raiseOnWrongAttribute=False,
-                          executeMethods=executeMethods, **params)
+                value = field.getCopyValue(self.o)
+            r[field.name] = value
+        return r
+
+    def createFrom(self, fieldNameOrClass, other, noSecurity=False,
+                   executeMethods=True, exclude=(), keepBase=False, fields={}):
+        '''Similar to m_create above, excepted that another object is used
+           (p_other) as base for filling in data for the object to create.
+           p_exclude can list fields (by their names) that will not be copied on
+           p_other. If p_keepBase is True, basic attributes will be kept on the
+           new object: creator and dates "created" and "modified". Else, the
+           new object's creator will be the logged user.'''
+        # Note that this method does not perform a deep copy: objects linked
+        # via Ref fields from p_self will be referenced by the clone, but not
+        # themselves copied.
+        # ~~~
+        # Get, as a dict, all field values to set from p_other. p_other may not
+        # be of the same class as p_self.
+        params = other.getCopyValues(exclude, fields)
+        # Create the new instance
+        r = self.create(fieldNameOrClass, noSecurity=noSecurity,
+                        raiseOnWrongAttribute=False,
+                        executeMethods=executeMethods, **params)
         # Propagate base attributes if required
         if keepBase:
             for name in ('creator', 'created', 'modified'):
-                setattr(res.o, name, getattr(other.o, name))
-        return res
+                setattr(r.o, name, getattr(other.o, name))
+        return r
 
     def freeze(self, name, template=None, format='pdf', noSecurity=True,
                freezeOdtOnError=True, value=None):
@@ -1651,13 +1662,16 @@ class AbstractWrapper(object):
             if userMethod and not getattr(user, userMethod)(self): continue
             # Get the mail recipient for this user
             recipient = user.getMailRecipient()
-            if not recipient: continue
+            if not recipient:
+                user.log('no mail recipient for "%s".' % user.login,
+                         type='warning')
+                continue
             recipients.append(recipient)
         if recipients:
             self.tool.sendMail(recipients, subject, body, attachments)
         else:
             self.log('no recipient for sending mail about %s with %s %s.' % \
-                     (self.id, pt, privilege))
+                     (self.id, pt, privilege), type='warning')
 
     def getLockers(self, names=False, sep=None):
         '''Returns the logins (or names if p_names is True) of the users having
