@@ -365,13 +365,13 @@ class Image:
        and size.'''
     jpgTypes = ('jpg', 'jpeg')
 
-    def __init__(self, path, format):
+    def __init__(self, path, format, unit='cm'):
         self.path = path # The image absolute path on disk
         self.format = format
         # Determine image size in pixels (again, by reading its first bytes)
-        self.width, self.height = self.getSizeInPx()
+        self.width, self.height = self.getSizeIn(unit)
 
-    def getSizeInPx(self):
+    def getSizeIn(self, unit='cm'):
         '''Reads the first bytes from the image on disk to get its size'''
         x = y = None
         # Get the file format from the file name of absent
@@ -407,9 +407,11 @@ class Image:
                 # else: invalid/corrupted GIF (bad header)
                 x, y, u = struct.unpack("<HHB", buf)
         f.close()
-        if x and y:
+        if (x and y) and (unit == 'cm'):
             return float(x)/px2cm, float(y)/px2cm
         else:
+            # Return the size, in pixels, or as a tuple (None, None) if
+            # uncomputable.
             return x, y
 
 # Compute size of images -------------------------------------------------------
@@ -640,8 +642,9 @@ class ImageImporter(DocImporter):
         if maxWidth and (width > maxWidth):
             ratio = maxWidth / width
             width = maxWidth
-            if keepRatio:
-                height = height * ratio
+            # Even if keepRatio is False, in that case, nevertheless apply the
+            # ratio on heigth, too.
+            height = height * ratio
         # Return the ODF attributes
         s = self.svgNs
         return ' %s:width="%fcm" %s:height="%fcm"' % (s, width, s, height)
