@@ -134,14 +134,14 @@ class PodEnvironment(OdfEnvironment):
                 res[elemName] = insert
         return res
 
-    def manageInserts(self):
+    def manageInserts(self, elem):
         '''We just dumped the start of an elem. Here we will insert any odt
            chunk if needed.'''
-        if self.inserts.has_key(self.currentElem.elem):
-            insert = self.inserts[self.currentElem.elem]
+        if self.inserts.has_key(elem):
+            insert = self.inserts[elem]
             self.currentBuffer.write(insert.odtChunk)
             # The insert is destroyed after single use
-            del self.inserts[self.currentElem.elem]
+            del self.inserts[elem]
 
     def onStartElement(self, elem, attrs):
         ns = self.namespaces
@@ -306,7 +306,7 @@ class PodParser(OdfParser):
                 if (elem == (e.tags['span'])) and not e.currentContent.strip():
                     e.currentBuffer.dumpStartElement(elem, attrs)
                     e.exprHasStyle = True
-        e.manageInserts()
+        e.manageInserts(elem)
 
     def endElement(self, elem):
         e = self.env
