@@ -13,23 +13,23 @@
 
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
+
 # ------------------------------------------------------------------------------
-import zipfile, shutil, xml.sax, os, os.path, re, mimetypes, time
 from UserDict import UserDict
+import zipfile, shutil, xml.sax, os, os.path, re, mimetypes, time
+
 import appy.pod
 from appy.pod import PodError
-from appy.shared import mimeTypes, mimeTypesExts
-from appy.shared.xml_parser import XmlElement
 from appy.shared.zip import unzip, zip
-from appy.shared.utils import FolderDeleter, executeCommand, FileWrapper
-from appy.pod.pod_parser import PodParser, PodEnvironment, OdInsert
-from appy.pod.converter import FILE_TYPES
 from appy.pod.buffers import FileBuffer
+from appy.pod import styles_manager as sm
+from appy.pod.converter import FILE_TYPES
+from appy.pod import doc_importers as imps
+from appy.shared.xml_parser import XmlElement
+from appy.shared import mimeTypes, mimeTypesExts
 from appy.pod.xhtml2odt import Xhtml2OdtConverter
-from appy.pod.doc_importers import getUuid, \
-     OdtImporter, ImageImporter, PdfImporter, ConvertImporter, PodImporter
-from appy.pod.styles_manager import \
-     StylesManager, TableProperties, NumberedProperties, BulletedProperties
+from appy.pod.pod_parser import PodParser, PodEnvironment, OdInsert
+from appy.shared.utils import FolderDeleter, executeCommand, FileWrapper
 
 # ------------------------------------------------------------------------------
 BAD_CONTEXT = 'Context must be either a dict, a UserDict or an instance.'
@@ -94,7 +94,7 @@ class Renderer:
       script=None, managePageStyles=None, resolveFields=False,
       expressionsHolders=defaultExpressionsHolders, metadata=True,
       pdfOptions='ExportNotes=True', deleteTempFolder=True, protection=False,
-      pageStart=1, tabbedCR=False):
+      pageStart=1, tabbedCR=False, fonts=None):
         '''Base on a document template (whose path is in p_template), which is
            an ODT or ODS file containing special expressions and statements
            written in Python, this renderer generates an ODT file (whose path is
@@ -260,6 +260,13 @@ class Renderer:
         # that way may cause problems when exporting the POD result to Microsoft
         # Word. This is why attribute "tabbedCR" exists.
 
+        # Attribute "fonts" may hold the name of a font that must exist on the
+        # machine running this renderer. If such a name is passed, all base
+        # styles defined in the POD template will be redefined with this font.
+        # On (most? all?) Linux machines, list the available fonts via a command
+        # like:
+        #                      fc-list :lang=en family
+
         self.template = template
         self.result = result
         self.resultType = os.path.splitext(result)[1].strip('.')
@@ -297,6 +304,7 @@ class Renderer:
         self.protection = protection
         self.pageStart = pageStart
         self.tabbedCR = tabbedCR
+        self.fonts = fonts
         # If sub-renderers are called, keep a trace of them
         self.children = {} # ~{s_templatePath: Renderer}~
         # Keep trace of the original context given to the renderer
@@ -314,7 +322,7 @@ class Renderer:
         self.contentXml = info['content.xml']
         self.stylesXml = info['styles.xml']
         # Manage the styles defined into the ODT template
-        self.stylesManager = StylesManager(self)
+        self.stylesManager = sm.StylesManager(self)
         # From LibreOffice 3.5, it is not possible anymore to dump errors into
         # the resulting ods as annotations. Indeed, annotations can't reside
         # anymore within paragraphs. ODS files generated with pod and containing
@@ -327,11 +335,6 @@ class Renderer:
         self.setStylesMapping(stylesMapping)
         # Store the p_html parameter
         self.html = html
-        # While working, POD may identify "dynamic styles" to insert either in
-        # the "automatic-styles" section of content.xml (ie, the column styles
-        # of tables generated from XHTML tables via xhtml2odt.py), or in the
-        # "styles" section of styles.xml (ie, bullet styles).
-        self.dynamicStyles = {'content': [], 'styles': []}
 
     def reinit(self, result, context):
         '''Re-initialise this renderer (p_self) for recycling him and produce
@@ -341,15 +344,15 @@ class Renderer:
         # Re-create POD parsers
         self.createParsers(context)
         # Reinitialise attributes being specific to a given result
-        self.dynamicStyles = {'content': [], 'styles': []}
-        sm = self.stylesManager.stylesMapping
-        self.stylesManager = StylesManager(self)
-        self.stylesManager.stylesMapping = sm
+        smap = self.stylesManager.stylesMapping
+        self.stylesManager = sm.StylesManager(self)
+        self.stylesManager.stylesMapping = smap
 
     # Attributes to clone to a sub-renderer when using m_clone hereafter
     cloneAttributes = ('pythonWithUnoPath', 'ooServer', 'ooPort', 'html',
       'raiseOnError', 'imageResolver', 'stylesTemplate', 'optimalColumnWidths',
-      'distributeColumns', 'expressionsHolders', 'protection', 'tabbedCR')
+      'distributeColumns', 'expressionsHolders', 'protection', 'tabbedCR',
+      'fonts')
 
     def clone(self, template, context, result, **params):
         '''Creates another Renderer instance, similar to p_self, but for
@@ -390,10 +393,11 @@ class Renderer:
         # Incorporate the default, unalterable, context
         r.update({'xhtml': self.renderXhtml, 'text': self.renderText,
           'document': self.importDocument, 'image': self.importImage,
-          'pod': self.importPod, 'test': self.evalIfExpression,
-          'TableProperties': TableProperties,
-          'BulletedProperties': BulletedProperties,
-          'NumberedProperties': NumberedProperties,
+          'pod': self.importPod, 'cell': self.importCell,
+          'shape': self.drawShape, 'test': self.evalIfExpression,
+          'TableProperties': sm.TableProperties,
+          'BulletedProperties': sm.BulletedProperties,
+          'NumberedProperties': sm.NumberedProperties,
           'pageBreak': self.insertPageBreak,
           'columnBreak': self.insertColumnBreak,
           # Variables to use for representing pod-reserved chars
@@ -437,7 +441,7 @@ class Renderer:
            If p_html is not None, it overrides renderer's homonym parameter.
         '''
         stylesMapping = self.stylesManager.checkStylesMapping(stylesMapping)
-        if html == None: html = self.html
+        if html is None: html = self.html
         return Xhtml2OdtConverter(s, encoding, self.stylesManager,
                                   stylesMapping, keepWithNext, keepImagesRatio,
                                   imagesMaxWidth, self, html, unwrap).run()
@@ -577,17 +581,17 @@ class Renderer:
         isImage = False
         isOdt = False
         if format in self.ooFormats:
-            importer = OdtImporter
+            importer = imps.OdtImporter
             self.forceOoCall = True
             isOdt = True
         elif (format in self.imageFormats) or not format:
             # If the format can't be guessed, we suppose it is an image
-            importer = ImageImporter
+            importer = imps.ImageImporter
             isImage = True
         elif format == 'pdf':
-            importer = PdfImporter
+            importer = imps.PdfImporter
         elif format in self.convertibleFormats:
-            importer = ConvertImporter
+            importer = imps.ConvertImporter
         else:
             raise PodError(DOC_WRONG_FORMAT % format)
         imp = importer(content, at, format, self)
@@ -636,7 +640,7 @@ class Renderer:
         # Renderer.
 
         # By default, if attribute "forceOoCall" is True for p_self, a sub-
-        # renderer ran by a PodImporter will inherit from this attribute,
+        # renderer ran by a imps.PodImporter will inherit from this attribute,
         # excepted if parameter p_forceOoCall is different from INHERIT.
 
         # Is there a pod template defined ?
@@ -646,7 +650,7 @@ class Renderer:
         # Appy FileWrapper.
         if content.__class__.__name__ == 'File':
             content = FileWrapper(content)
-        imp = PodImporter(content, at, format, self)
+        imp = imps.PodImporter(content, at, format, self)
         self.forceOoCall = True
         # Define the context to use: either the current context of the current
         # POD renderer, or p_context if given.
@@ -661,6 +665,30 @@ class Renderer:
                  managePageStyles, resolveFields, forceOoCall)
         return imp.run()
 
+    def importCell(self, content, style='Default'):
+        '''Creates a chunk of ODF code ready to be dumped as table cell'''
+        # The generated cell will contain this textual p_content and this
+        # p_style will be applied.
+        return '<table:table-cell table:style-name="%s">' \
+               '<text:p>%s</text:p></table:table-cell>' % (style, content)
+
+    def drawShape(self, name, type='rect', stroke='none', strokeWidth='0cm',
+                  strokeColor='#666666', fill='solid', fillColor='#666666',
+                  anchor='char', width='1.0cm', height='1.0cm',
+                  deltaX='0.0cm', deltaY='0.0cm', target='styles'):
+        '''Renders a shape within a POD result'''
+        # Generate a style for this shape and add it among dynamic styles
+        style = self.stylesManager.stylesGenerator.addGraphicalStyle(target,
+          stroke=stroke, strokeWidth=strokeWidth, strokeColor=strokeColor,
+          fill=fill, fillColor=fillColor)
+        # Return the ODF code representing the shape
+        return '<draw:%s text:anchor-type="%s" draw:z-index="1" ' \
+               'draw:name="%s" draw:style-name="%s" ' \
+               'draw:text-style-name="MP1" svg:width="%s" svg:height="%s"' \
+               ' svg:x="%s" svg:y="%s"><text:p/></draw:%s>' % \
+               (type, anchor, name, style.name, width, height, deltaX, deltaY,
+                type)
+
     def _insertBreak(self, type):
         '''Inserts a page or column break into the result'''
         name = (type == 'page') and 'podPageBreakAfter' or 'podColumnBreak'
@@ -775,7 +803,7 @@ class Renderer:
             ocw = self.optimalColumnWidths
             dis = self.distributeColumns
             if ocw or dis:
-                TableProperties.initStylesMapping(stylesMapping, ocw, dis)
+                sm.TableProperties.initStylesMapping(stylesMapping, ocw, dis)
             manager.stylesMapping = manager.checkStylesMapping(stylesMapping)
         except PodError, po:
             self.contentParser.env.currentBuffer.content.close()
@@ -892,26 +920,26 @@ class Renderer:
         if mps != None:
             pageStyles = self.stylesManager.pageStyles.init(mps)
         # Patch styles.xml and content.xml
+        dynamic = self.stylesManager.dynamicStyles
         for name in ('styles', 'content'):
             # Copy the [content|styles].xml file from the temp to the zip folder
             fn = '%s.xml' % name
             shutil.copy(j(self.tempFolder, fn), j(self.unzipFolder, fn))
-            # For styles.xml, complete dynamic styles with default styles for
-            # bulleted and numbered lists.
-            ds = self.dynamicStyles[name]
-            if name == 'styles':
-                env = self.stylesParser.env
-                n = {'text': env.ns(env.NS_TEXT), 'style': env.ns(env.NS_STYLE)}
-                ds.insert(0,NumberedProperties().dumpStyle('podNumberedList',n))
-                ds.insert(0,BulletedProperties().dumpStyle('podBulletedList',n))
-            # Get the file content and inject dynamic styles into it
+            # Get the file content
             fn = os.path.join(self.unzipFolder, fn)
             f = file(fn)
-            content = f.read().replace('<!DYNAMIC_STYLES!>', ''.join(ds))
+            content = f.read()
+            f.close()
+            # Inject self.fonts, when present, in styles.xml
+            isStylesXml = name == 'styles'
+            if isStylesXml and self.fonts:
+                content = sm.FontsInjector(self.fonts).injectIn(content)
+            # Inject dynamic styles
+            if isStylesXml: content = dynamic.injectIn('styles_base', content)
+            content = dynamic.injectIn(name, content)
             # Rename the page styles
             if pageStyles:
                 content = pageStyles.renameIn(name, content)
-            f.close()
             # Write the updated content to the file
             f = file(fn, 'w')
             f.write(content)
