@@ -233,11 +233,11 @@ class ListProperties(Properties):
         # The number of levels can > or < to the number of formats. In those
         # cases, formats will be applied partially or cyclically to levels.
 
-    def dumpStyle(self, name, ns):
+    def dumpStyle(self, name):
         '''Returns the OpenDocument style definition corresponding to this
            instance.'''
-        nsText = ns['text']
-        nsStyle = ns['style']
+        nsText = 'text'
+        nsStyle = 'style'
         res = []
         spaceBefore = 0
         space = formatNumber(self.space, sep='.', removeTrailingZeros=True)
@@ -377,6 +377,42 @@ class Style:
             return self.getOdfAttributes(withName=False, withDefaults=True,
                                          exclude=childAttrs)
 
+# ------------------------------------------------------------------------------
+class GraphicalStyle:
+    '''Represents an ODF style of type "graphic"'''
+
+    defaultAttributes = 'draw:textarea-horizontal-align="center" ' \
+      'draw:textarea-vertical-align="top" fo:margin-left="0.318cm" ' \
+      'fo:margin-right="0.318cm" fo:margin-top="0cm" fo:margin-bottom="0cm" ' \
+      'style:run-through="background" style:wrap="run-through" ' \
+      'style:number-wrapped-paragraphs="no-limit" ' \
+      'style:vertical-pos="from-top" style:vertical-rel="paragraph" ' \
+      'style:horizontal-pos="from-left" style:horizontal-rel="paragraph" ' \
+      'draw:wrap-influence-on-position="once-concurrent" ' \
+      'loext:allow-overlap="true" style:flow-with-text="false"'
+
+    def __init__(self, name, stroke='none', strokeWidth='0cm',
+                 strokeColor='#666666', fill='solid', fillColor='#666666'):
+        self.name = name
+        self.stroke = stroke
+        self.strokeWidth = strokeWidth
+        self.strokeColor = strokeColor
+        self.fill = fill
+        self.fillColor = fillColor
+
+    def getOdfAttributes(self):
+        '''Returns p_self's attributes expressed as ODF'''
+        return 'draw:stroke="%s" svg:stroke-width="%s" svg:stroke-color="%s" ' \
+               'draw:fill="%s" draw:fill-color="%s" %s' % \
+               (self.stroke, self.strokeWidth, self.strokeColor,
+                self.fill, self.fillColor, self.defaultAttributes)
+
+    def asOdf(self):
+        '''Returns p_self's representation as ODF code'''
+        return '<style:style style:name="%s" style:family="graphic">' \
+               '<style:graphic-properties %s/></style:style>' % \
+               (self.name, self.getOdfAttributes())
+
 # ------------------------------------------------------------------------------
 class PageStyle:
     '''Represents a page style'''
@@ -601,6 +637,36 @@ class Styles(UserDict):
                     res.append(style)
         return res
 
+# ------------------------------------------------------------------------------
+class FontsInjector:
+    '''Allows to inject a font within all styles in a ODF document'''
+
+    # Regular expressions allowing to identify, in styles.xml, fonts and
+    # font-family definitions.
+    any = '.*?'
+    fontAttr = 'style:font-name="%s"'
+    fontRex = re.compile(fontAttr % any)
+    familyAttr = 'fo:font-family="%s"'
+    familyRex = re.compile(familyAttr % any)
+
+    def __init__(self, font):
+        # The font name may also include the font family
+        parts = font.split(',', 1)
+        if len(parts) == 1:
+            self.font = self.family = parts[0]
+        else:
+            # In that case, the "family" must also be used for the font name
+            self.font = self.family = parts[1]
+
+    def injectIn(self, stylesXml):
+        '''Injects, within every style found in a "styles.xml" file whose string
+           content is in p_stylesXml, the font as defined in p_self.font.'''
+        # Set the correct font name
+        r = self.fontRex.sub(lambda match: self.fontAttr % self.font, stylesXml)
+        # Set the correct family name
+        return self.familyRex.sub(
+                 lambda m: self.familyAttr % ("'%s'" % self.family), r)
+
 # ------------------------------------------------------------------------------
 class StylesEnvironment(OdfEnvironment):
     def __init__(self):
@@ -860,6 +926,52 @@ class Css2odf:
         parser = self.xhtmlParser
         return parser and parser.env.currentTables
 
+# ------------------------------------------------------------------------------
+class DynamicStyles:
+    '''Styles generated on-the-fly by POD'''
+
+    # While working, POD may identify and create on-the-fly "dynamic styles", to
+    # insert either in the "automatic-styles" section of content.xml (ie, the
+    # column styles of tables generated from XHTML tables via xhtml2odt.py), in
+    # the "automatic-styles" section of styles.xml (ie, styles used in the
+    # header and the footer) or in its "styles" section (ie, bullet styles).
+
+    # The styles must be injected just before these "hooks"
+    hooks = {'content':     '</office:automatic-styles>',
+             'styles_base': '</office:styles>',
+             'styles':      '</office:automatic-styles>'}
+
+    def __init__(self):
+        # Styles to be injected in the "automatic-styles" tag within content.xml
+        self.content = []
+        # Styles to be injected in the "automatic-styles" tag within styles.xml
+        self.styles = []
+        # Styles to be injected in the "styles" tag within styles.xml
+        self.styles_base = []
+
+    def addListStyles(self):
+        '''Add bullet-related styles in attr "styles_base"'''
+        styles = self.styles_base
+        styles.append(NumberedProperties().dumpStyle('podNumberedList'))
+        styles.append(BulletedProperties().dumpStyle('podBulletedList'))
+
+    def add(self, attr, style):
+        '''Adds this p_style definition p_self's attribute named p_attr'''
+        getattr(self, attr).append(style.encode('utf-8'))
+
+    def injectIn(self, attr, content):
+        '''Inject the styles generated and collected in p_self's attribute named
+           p_attr and inject them in p_content.'''
+        # For "styles_base", add list-related styles now
+        if attr == 'styles_base': self.addListStyles()
+        # p_content is the string content of either content.xml or styles.xml
+        styles = getattr(self, attr)
+        if not styles: return content
+        # Perform the injection
+        hook = self.hooks[attr]
+        styles = '%s%s' % (''.join(styles), hook)
+        return content.replace(hook, styles)
+
 # ------------------------------------------------------------------------------
 class StylesGenerator:
     '''Analyse, for a given XHTML tag, its attributes (including CSS attributes
@@ -908,10 +1020,19 @@ class StylesGenerator:
         self.css2odf = Css2odf()
 
     def addStyle(self, style, target):
-        '''Adds the style definition to the renderer's dynamic styles.
-           Target is "styles" (styles.xml) or "content" (content.xml).'''
-        dynamicStyles = self.stylesManager.renderer.dynamicStyles[target]
-        dynamicStyles.append(style.encode('utf-8'))
+        '''Adds the style definition dynamic styles'''
+        self.stylesManager.dynamicStyles.add(target, style)
+
+    def addGraphicalStyle(self, target, **kwargs):
+        '''Creates and add a style of type "graphic" within the dynamically
+           generated styles.'''
+        # Generate a name for this new style
+        name = self.generateStyleName()
+        # Create a instance of GraphicalStyle
+        style = GraphicalStyle(name, **kwargs)
+        # Add it among dynamically created styles
+        self.addStyle(style.asOdf(), target)
+        return style
 
     def getStyleHash(self, xhtmlElem, odfAttrs, baseStyle):
         '''Returns a string uniquely representing the given set of ODF
@@ -931,6 +1052,11 @@ class StylesGenerator:
         '''Returns the ODF family style corresponding to p_xhtmlElem'''
         return self.styleFamilies.get(xhtmlElem.elem, 'text')
 
+    def generateStyleName(self):
+        '''Generates a new style name'''
+        self.last += 1
+        return '%s%d%s' % (self.prefix, self.last, self.suffix)
+
     def getStyleName(self, xhtmlElem, odfAttrs, baseStyle):
         '''Return the ODF style name for the style that will be applied to ODF
            element corresponding to the given XHTML element (p_xhtmlElem). We
@@ -938,22 +1064,21 @@ class StylesGenerator:
            a style has already been generated by this styles generator, for this
            tag and with this combination of attributes, we simply return its
            name. Else, we note that we must generate a new style; we compute a
-           new (incremental) name for it and we return this name.
-
-           More precisely, this method returns a tuple (createNew, styleName):
-            - "createNew" is a boolean indicating if a new style must be
-                          generated;
-            - "styleName" is the style name.'''
+           new (incremental) name for it and we return this name.'''
+        # More precisely, this method returns a tuple (createNew, styleName):
+        # - "createNew" is a boolean indicating if a new style must be
+        #               generated;
+        # - "styleName" is the style name.
+        # ~~~
         # If the style hash corresponds to an existing style, simply return
         # its name.
         hash = self.getStyleHash(xhtmlElem, odfAttrs, baseStyle)
         if hash in self.generated:
             return False, self.generated[hash]
-        # We must generate a new style
-        self.last += 1
-        res = '%s%d%s' % (self.prefix, self.last, self.suffix)
-        self.generated[hash] = res
-        return True, res
+        # A new style must be generated
+        r = self.generateStyleName()
+        self.generated[hash] = r
+        return True, r
 
     def flattenOdfAttributes(self, odfAttrs, forHash=False):
         '''Produce a string from the list of (name, value) pairs in
@@ -1006,7 +1131,7 @@ class StylesGenerator:
         # I could have added this style in content.xml. But in some cases it
         # does not work properly. For example, a percentage value for attribute
         # "fo-font-size" will be ignored if the style is dumped in content.xml.
-        self.addStyle(style, 'styles')
+        self.addStyle(style, 'styles_base')
         # If p_add is True, also add the style to styleManager's defined styles
         # (as a Style instance). Else, it won't be found when it will need to be
         # applied on an inner paragraph.
@@ -1094,6 +1219,7 @@ class StylesGenerator:
 class StylesManager:
     '''Reads the paragraph styles from styles.xml within an ODT file, and
        updates styles.xml with some predefined POD styles.'''
+
     podSpecificStyles = {
       'ParaKWN': Style('ParaKWN', 'paragraph'),
       # This style is common to bullet and number items. Behind the scenes,
@@ -1106,6 +1232,7 @@ class StylesManager:
       'podBulletItem': Style('podBulletItem', 'paragraph'),
       'podNumberItem': Style('podNumberItem', 'paragraph'),
     }
+
     # Valid value types for some keys within style mappings
     mappingValueTypes = {'h*': int, 'table': TableProperties,
                          'ol': NumberedProperties, 'ul': BulletedProperties}
@@ -1127,6 +1254,8 @@ class StylesManager:
         self.paragraphStyles = self.styles.getStyles('paragraph')
         # The custom styles generator
         self.stylesGenerator = StylesGenerator(self)
+        # Dynamically-generated styles
+        self.dynamicStyles = DynamicStyles()
 
     def checkStylesAdequation(self, htmlStyle, odtStyle):
         '''Checks that p_odtStyle may be used for style p_htmlStyle'''
