@@ -17,7 +17,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # Contributors: Gauthier Bastien, Fabio Marcuzzi, IMIO
 # ------------------------------------------------------------------------------
-import re, xml.sax
+import re, xml.sax, math
 
 from appy.pod import *
 from appy.pod.doc_importers import getUuid
@@ -516,7 +516,7 @@ class HtmlTable(Element):
                (s, s, self.name, s, s, s, s, formatNumber(tableWidth, sep='.'),
                 s, percentage, self.tableNs, align, self.tableNs, align,
                 margins, kwn, unbreak, s)
-        self.env.renderer.dynamicStyles['content'].append(decl.encode('utf-8'))
+        self.env.stylesManager.dynamicStyles.add('content', decl)
         return self.name, tableWidthPx, originalTableWidthPx
 
     def setColumnWidth(self, width):
@@ -529,11 +529,30 @@ class HtmlTable(Element):
         if widths[self.cellIndex] == None:
             widths[self.cellIndex] = width
 
-    def computeColumnStyles(self, renderer):
+    def softenWidths(self):
+        '''Reduce columns widths being proportionnaly too large'''
+        sizes = self.columnContentSizes
+        i = len(sizes) - 1
+        while i >= 0:
+            # Calibrate the value in order to soften differences between small
+            # and large column widths.
+            value = sizes[i]
+            if not value:
+                i -= 1
+                continue
+            elif value < 5: value = 10
+            elif value < 50: value = value * 2.5
+            else:
+                value = min(value, 1000)
+            sizes[i] = int(math.pow(math.log(value), 3))
+            i -= 1
+        return sizes
+
+    def computeColumnStyles(self):
         '''Once the table has been completely parsed, self.columnContentSizes
            should be correctly filled. Based on this, we can deduce the width
            of every column and create the corresponding style declarations, in
-           p_renderer.dynamicStyles.'''
+           p_self.env.stylesManager.dynamicStyles.'''
         # The objective is to compute, in "widths", relative column widths,
         # as percentages, from 0 to 1.0.
         widths = []
@@ -586,11 +605,11 @@ class HtmlTable(Element):
         # content from all columns.
         contentTotal = 0
         i = 0
-        contentSizes = self.columnContentSizes
+        contentSizes = self.softenWidths()
         longestWords = self.columnLongestWords
         while i < self.nbOfColumns:
             # Ignore columns for which a width has already been computed
-            if widths[i] == None:
+            if widths[i] is None:
                 if (i < len(contentSizes)) and contentSizes[i]:
                     contentTotal += contentSizes[i]
             i += 1
@@ -603,7 +622,7 @@ class HtmlTable(Element):
         surplus = 0.0 # The total surplus granted to too narrow cells
         remainingCount = 0 # The number of cells with no surplus
         while i < self.nbOfColumns:
-            if widths[i] == None:
+            if widths[i] is None:
                 # Get the content size and longest word for this column
                 if (i < len(contentSizes)) and contentSizes[i]:
                     contentSize = contentSizes[i]
@@ -611,13 +630,16 @@ class HtmlTable(Element):
                 else:
                     contentSize = longest = 0
                 # Compute the column width
-                width = (float(contentSize) / contentTotal) * remainingPc
-                if width < minCellWidth:
-                    surplus += minCellWidth - width
-                    val = (minCellWidth, True)
+                if contentSize == 0:
+                    val = 0
                 else:
-                    remainingCount += 1
-                    val = (width, False)
+                    width = (float(contentSize) / contentTotal) * remainingPc
+                    if width < minCellWidth:
+                        surplus += minCellWidth - width
+                        val = (minCellWidth, True)
+                    else:
+                        remainingCount += 1
+                        val = (width, False)
                 widths[i] = val
             i += 1
         # "Convert" stored tuples into final values
@@ -640,18 +662,19 @@ class HtmlTable(Element):
         i = 0
         total = 65534.0
         while i < self.nbOfColumns:
+            # Compute the width of this column, relative to "total"
             widths[i] = int(widths[i] * total)
             i += 1
         # Compute style declaration corresponding to every column
         s = self.styleNs
         i = 0
+        dynamic = self.env.stylesManager.dynamicStyles
         for width in widths:
             i += 1
-            # Compute the width of this column, relative to "total"
             decl = '<%s:style %s:name="%s.%d" %s:family="table-column">' \
                    '<%s:table-column-properties %s:rel-column-width="%d*"' \
                    '/></%s:style>' % (s, s, self.name, i, s, s, s, width, s)
-            renderer.dynamicStyles['content'].append(decl.encode('utf-8'))
+            dynamic.add('content', decl)
 
 # ------------------------------------------------------------------------------
 class XhtmlEnvironment(XmlEnvironment):
@@ -678,6 +701,7 @@ class XhtmlEnvironment(XmlEnvironment):
     def __init__(self, renderer):
         XmlEnvironment.__init__(self)
         self.renderer = renderer
+        self.stylesManager = renderer.stylesManager
         self.ns = renderer.currentParser.env.namespaces
         self.res = u''
         self.currentContent = u''
@@ -995,7 +1019,7 @@ class XhtmlEnvironment(XmlEnvironment):
             table = self.currentTables.pop()
             if table.nbOfColumns:
                 # Computes the column styles required by the table
-                table.computeColumnStyles(self.parser.caller.renderer)
+                table.computeColumnStyles()
             # Dumps the content of the last parsed table into the parent buffer
             self.dumpString(table.res)
         elif elem in TABLE_ROW_TAGS:
@@ -1030,8 +1054,7 @@ class XhtmlEnvironment(XmlEnvironment):
                     wordSizes.append(None)
                 longest = max(sizes[i], cellContentSize, 5)
                 wordLongest = max(wordSizes[i], cellLongestWord, 5)
-                # Put a maximum
-                sizes[i] = min(longest, 100)
+                sizes[i] = longest
                 wordSizes[i] = min(wordLongest, 25)
             table.inCell = table.cellContentSize = table.cellLongestWord = 0
         if current.tagsToClose:
@@ -1241,11 +1264,9 @@ class XhtmlParser(XmlParser):
 
     def endDocument(self):
         '''Dump all collected list styles'''
-        ds = self.caller.renderer.dynamicStyles['styles']
-        env = self.env
-        ns = {'text': env.textNs, 'style': env.styleNs}
-        for name, props in env.listProperties.iteritems():
-            ds.append(props.dumpStyle(name, ns))
+        styles = self.caller.stylesManager.dynamicStyles.styles_base
+        for name, props in self.env.listProperties.iteritems():
+            styles.append(props.dumpStyle(name))
 
 # ------------------------------------------------------------------------------
 class XhtmlPreprocessor:
