@@ -230,6 +230,10 @@ class CssStyles:
     directions = ('top', 'right', 'bottom', 'left')
     # Values, on combined attributes, that prevent splitting
     unsplittable = ('auto',)
+    # Values that must be ignored
+    ignore = {'width': ['auto']}
+    # The '!important' CSS rule
+    importantRule = '!important'
 
     def __init__(self, elem=None, attrs=None, other=None, **kwargs):
         '''Analyses styles as found in p_attrs and/or deduced from XHTML element
@@ -333,6 +337,11 @@ class CssStyles:
            also return it. p_name can be the original name of the CSS attribute
            (possibly containing dashes).'''
         if name == 'classes': raise Exception(ADD_CLASS_ERROR)
+        # Ignore potential "!important" suffix
+        if isinstance(value, basestring) and value.endswith(self.importantRule):
+            value = value[:-len(self.importantRule)].strip()
+        # Do not add anything if the value must be ignored
+        if (name in self.ignore) and (value in self.ignore[name]): return
         attrName = name.replace('-', '')
         if not override and hasattr(self, attrName): return
         # Define the new value or combine it with the existing value when
