@@ -20,11 +20,19 @@ try:
 except ImportError:
     # For people that do not care about ldap
     ldap = None
+
+from appy import Object as O
 from appy.shared.utils import sequenceTypes
 
 # ------------------------------------------------------------------------------
-LDAP_DISABLED = 'LDAP config not enabled.'
-LDAP_CONNECT_KO = 'Could not connect to %s'
+LDAP_DISABLED   = 'LDAP config not enabled.'
+LDAP_CONNECT_KO = 'Could not connect to %s (%s).'
+LDAP_COUNTS     = '%d local active SSO users // %d LDAP users'
+LDAP_ACT        = '%d user(s) %sactivated: %s.'
+LDAP_NO_DEACT   = 'No user was deactivated.'
+SYNC_SUMMARY    = 'users synchronization: %d local user(s) created, %d ' \
+                  'updated, %d untouched and %d invalid entries. %s local ' \
+                  'user(s) deactivated and %d reactivated.'
 
 # ------------------------------------------------------------------------------
 class LdapConfig:
@@ -98,6 +106,12 @@ class LdapConfig:
         # one, it will be converted to a distant copy, excepted if it is a
         # special user.
         self.convertLocalUsers = True
+        # While synchronizing distant users with their local copies, the
+        # following attribute determines if locally deactivated users being
+        # found in the LDAP are reactivated or not. By default, they are
+        # reactivated, the distant source being considered as the "authentic
+        # source of data" for its users.
+        self.reactivateUsers = True
 
     def init(self, tool):
         '''Lazy initialization'''
@@ -273,10 +287,46 @@ class LdapConfig:
         user, status = self.setLocalUser(tool, userParams, login, password)
         return user
 
+    def updateUserStates(self, tool, ldapUsers, counts, sso=None):
+        '''Browse local users and activate or deactivate some of them, based on
+           freshly queried LDAP users from p_ldapUsers. Update p_counts of users
+           (attributes "deactivated" and "reactivated").'''
+        # Remember logins of (de)(re)activated users, for logging them
+        deactivated = []
+        reactivated = []
+        source = sso and 'sso' or 'ldap'
+        local = 0 # Count the active local copies of external users
+        # Must deactivated users be reactivated if found among p_ldapUsers ?
+        mustReactivate = self.reactivateUsers
+        # Browse local users
+        for user in tool.users:
+            # Ignore those not being external
+            if user.source != source: continue
+            # Deactivate appropriate users
+            if user.state == 'active':
+                local += 1
+                if user.login not in ldapUsers:
+                    deactivated.append('%s (%s)' % (user.login,user.getTitle()))
+                    user.do('deactivate')
+                    counts.deactivated += 1
+            # Reactivate appropriate users
+            elif mustReactivate and (user.state == 'inactive'):
+                if user.login in ldapUsers:
+                    reactivated.append('%s (%s)' % (user.login,user.getTitle()))
+                    user.do('reactivate')
+                    counts.reactivated += 1
+        log = tool.log
+        log(LDAP_COUNTS % (local, len(ldapUsers)))
+        if deactivated:
+            log(LDAP_ACT % (counts.deactivated, 'de', ', '.join(deactivated)))
+        elif mustReactivate:
+            log(LDAP_NO_DEACT)
+        if reactivated:
+            log(LDAP_ACT % (counts.reactivated, 're', ', '.join(reactivated)))
+
     def synchronizeUsers(self, tool, sso=None):
         '''Synchronizes the local User copies with this LDAP user base. Returns
-           a 2-tuple containing the number of created, updated and untouched
-           local copies.'''
+           a message with details about the operation.'''
         # If this LDAP config is part of a SSO config, this latter is available
         # in p_sso ; in that case, users need to be flagged as having source
         # "sso" and not "ldap".
@@ -286,12 +336,16 @@ class LdapConfig:
         tool.log('reading users from %s...' % serverUri)
         connector = LdapConnector(serverUri, tool=tool)
         success, msg = connector.connect(self.adminLogin, self.adminPassword)
-        if not success: raise Exception(LDAP_CONNECT_KO % serverUri)
+        if not success: raise Exception(LDAP_CONNECT_KO % (serverUri, msg))
         # Query the LDAP for users. Perform several queries to avoid having
         # error ldap.SIZELIMIT_EXCEEDED.
         params = self.getUserAttributes()
         # Count the number of created, updated and untouched users
-        created = updated = untouched = wrong = 0
+        counts = O(created=0, updated=0, untouched=0, wrong=0,
+                   deactivated=0, reactivated=0)
+        # Remember the logins of encountered LDAP users: it is required to
+        # (de)activate local users.
+        ldapUsers = {} # ~{s_login: None}~
         for letter in string.ascii_lowercase:
             # Get all the users whose login starts with "letter"
             filter = connector.getFilter(self.getUserFilterValues('%s*'%letter),
@@ -306,20 +360,25 @@ class LdapConfig:
                         login = self.applyLoginTransform(login)
                     except TypeError:
                         # The retrieved data structure is not valid
-                        wrong += 1
+                        counts.wrong += 1
                         continue
                     # Get the other user parameters, as Appy wants it
                     userParams = self.getUserParams(userData[1])
                     # Create or update the user
                     user, status = self.setLocalUser(tool, userParams, login,
                                                      sso=sso)
-                    if status == 'created': created += 1
-                    elif status == 'updated': updated += 1
-                    else: untouched += 1
-        tool.log('users synchronization: %d local user(s) created, ' \
-                 '%d updated, %d untouched and %d invalid entries.' % \
-                 (created, updated, untouched, wrong))
-        return created, updated, untouched, wrong
+                    if status == 'created': counts.created += 1
+                    elif status == 'updated': counts.updated += 1
+                    else: counts.untouched += 1
+                    if user: ldapUsers[login] = None
+        # Activate or deactivate users
+        self.updateUserStates(tool, ldapUsers, counts, sso=sso)
+        # Log the operation and return a message
+        message = SYNC_SUMMARY % (counts.created, counts.updated,
+                                  counts.untouched, counts.wrong,
+                                  counts.deactivated, counts.reactivated) 
+        tool.log(message)
+        return message
 
 # ------------------------------------------------------------------------------
 class LdapConnector:
