@@ -285,6 +285,22 @@ def executeCommand(cmd, stdin=None):
                             stderr=subprocess.PIPE)
     return proc.communicate()
 
+# ------------------------------------------------------------------------------
+CONVERT_ERROR = 'Program "convert" (imagemagick) must be installed and ' \
+                'in the path for performing this operation (%s).'
+
+def convert(path, options):
+    '''Calls the "convert" executable to apply some transform to an image whose
+       p_path is passed.'''
+    # p_options can be a list, or a string containing blank-separated options
+    if isinstance(options, basestring):
+        options = options.split()
+    cmd = ['convert', path] + options
+    cmd.append(path)
+    out, err = executeCommand(cmd)
+    if err: raise Exception(CONVERT_ERROR % str(err))
+    return out
+
 # ------------------------------------------------------------------------------
 charsIgnore = u'.,:;*+=~?%^\'’"<>{}[]|\t\\°-'
 fileNameIgnore = charsIgnore + u' $£€/\r\n'
