@@ -314,7 +314,7 @@ class XmlUnmarshaller(XmlParser):
        one corresponding to the value of an attribute for this object.
        if "tuple" is specified, it will be converted to a list.'''
     def __init__(self, classes={}, tagTypes={}, conversionFunctions={},
-                 utf8=True):
+                 contentAttributes={}, utf8=True):
         XmlParser.__init__(self)
         # self.classes below is a dict whose keys are tag names and values are
         # Python classes. During the unmarshalling process, when an object is
@@ -351,6 +351,17 @@ class XmlUnmarshaller(XmlParser):
         # for example convert strings that have specific values (in this case,
         # knowing that the value is a 'string' is not sufficient).        
         self.conversionFunctions = conversionFunctions
+        # If you have the special case of a tag whose type is "object" and that
+        # has direct text content, you can specify, in attribute
+        # "contentAttributes", the name of the attribute onto which this content
+        # will be stored. For example, suppose you have:
+        #   <parent><child attr1="a1" attr2="a2">c1</child></parent>
+        # If you specify:
+        #   tagTypes={'child': 'object'}
+        #   contentAttributes={'child': 'content'}
+        # you will get, for "child", an object
+        #   O(attr1='a1', attr2='a2', content='c1')
+        self.contentAttributes = contentAttributes
         self.utf8 = utf8
         # Remember the name of the root tag
         self.rootTag = None
@@ -484,6 +495,15 @@ class XmlUnmarshaller(XmlParser):
         e = XmlParser.characters(self, content)
         if e.currentBasicType:
             e.currentContent += content
+        elif e.currentElem.name in self.contentAttributes:
+            # Store p_content in attribute named according to
+            # p_self.contentAttributes on the current object.
+            name = self.contentAttributes[e.currentElem.name]
+            current = e.containerStack[-1]
+            if hasattr(current, name):
+                setattr(current, name, getattr(current, name) + content)
+            else:
+                setattr(current, name, content)
 
     def endElement(self, elem):
         e = XmlParser.endElement(self, elem)
@@ -559,8 +579,8 @@ class XmlMarshaller:
     atFiles = ('image', 'file') # Types of archetypes fields that contain files
     typesMap = {'list': 'list', 'PersistentList': 'list', 'LazyMap': 'list',
                 'UserList': 'list', 'dict': 'dict', 'PersistentMapping': 'dict',
-                'UserDict': 'dict', 'FileInfo': 'file', 'bool': 'bool',
-                'int': 'int', 'float': 'float', 'long': 'long',
+                'UserDict': 'dict', 'IOBTree': 'dict', 'FileInfo': 'file',
+                'bool': 'bool', 'int': 'int', 'float': 'float', 'long': 'long',
                 'tuple': 'tuple', 'DateTime': 'DateTime'}
 
     def __init__(self, cdata=False, dumpUnicode=False, conversionFunctions={},
@@ -875,8 +895,11 @@ class XmlMarshaller:
                                        fieldType=fieldType)
             elif objectType == 'appy':
                 # Dump base attributes
-                for name in ('created', 'creator', 'modified'):
+                for name in ('created', 'creator'):
                     self.dumpField(res, name, getattr(instance, name))
+                # On very old objects, attribute "modified" may not exist
+                modified = getattr(instance, 'modified', instance.created)
+                self.dumpField(res, 'modified', modified)
                 appyi = instance.appy()
                 for field in instance.getAppyTypes('xml', None):
                     # Dump only needed fields
@@ -898,6 +921,8 @@ class XmlMarshaller:
                             self.dumpField(res, k, v)
                         res.write('</%s>' % eventTag)
                     res.write('</%s>' % histTag)
+                # Dump local roles
+                self.dumpField(res, 'localRoles', instance.__ac_local_roles__)
             self.marshallSpecificElements(instance, res)
             res.write('</'); res.write(rootTagName); res.write('>')
         else:
