@@ -20,13 +20,13 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import os, time, pathlib
 
-import ZODB, transaction
 from DateTime import DateTime
 from zc.lockfile import LockError
 from BTrees.IOBTree import IOBTree
 from persistent.mapping import PersistentMapping
-from ZODB.POSException import ConnectionStateError
+import ZODB, ZODB.POSException, transaction, transaction.interfaces
 
+from appy.px import Px
 from appy.utils import multicall
 from appy.database.lock import Lock
 from appy.database.lazy import Lazy
@@ -34,31 +34,31 @@ from appy.utils import path as putils
 from appy.database.catalog import Catalog
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-DB_CREATED = 'Database created @%s.'
+DB_CREATED   = 'Database created @%s.'
 DB_NOT_FOUND = 'Database does not exist @%s.'
-DB_LOCKED = 'The database is currently locked (%s)'
+DB_LOCKED    = 'The database is currently locked (%s)'
 DB_CORRUPTED = 'The database @%s is corrupted and probably empty. Please ' \
-  'remove it and restart your site.'
-DB_PACKING = 'Packing %s (may take a while)...'
-DB_PACKED = 'Done. Went from %s to %s.'
-TEMP_STORE_NOT_FOUND = 'Temp store does not exist in %s.'
-OBJECT_STORE_NOT_FOUND = 'Object store does not exist in %s.'
-TOOL_NOT_FOUND = 'Tool does not exist in %s.'
-METHOD_NOT_FOUND = 'Method "%s" does not exist on the tool.'
-TEMP_STORE_EMPTY = 'No temp object has been deleted in %s.'
-TEMP_OBJECTS_DELETED = '%s temp object(s) removed from %s.'
-OBJECT_EXISTS = 'An object with id "%s" already exists.'
-CLASS_NOT_FOUND = 'Class "%s" does not exist.'
-CUSTOM_ID_NO_IKEY = 'It is not possible to get an ikey for object having a ' \
-  'custom ID (%s).'
-NEW_TEMP_WITH_ID = 'An ID cannot be specified when creating a temp object.'
-CUSTOM_ID_NOT_STR = 'Custom ID must be a string.'
-SEARCH_NO_CATALOG = 'Invalid operation: there is no catalog for instances ' \
-  'of class "%s".'
-MISSING_OBJECT_CLASS = 'Catalog for class "%s": object not found for ID "%s".'
-MISSING_OBJECT = 'Object not found for ID "%s".'
-MISSING_IDS_DELETED = 'Catalog "%s": ID(s) removed because no corresponding ' \
-  'object(s) found: %s.'
+               'remove it and restart your site.'
+DB_PACKING   = 'Packing %s (may take a while)...'
+DB_PACKED    = 'Done. Went from %s to %s.'
+TMP_STORE_NF = 'Temp store does not exist in %s.'
+O_STORE_NF   = 'Object store does not exist in %s.'
+TOOL_NF      = 'Tool does not exist in %s.'
+METH_NF      = 'Method "%s" does not exist on the tool.'
+TMP_STORE_E  = 'No temp object has been deleted in %s.'
+TMP_OBJS_DEL = '%s temp object(s) removed from %s.'
+O_EXISTS     = 'An object with id "%s" already exists.'
+CLASS_NF     = 'Class "%s" does not exist.'
+CUS_ID_NO_IK = 'It is not possible to get an ikey for object having a ' \
+               'custom ID (%s).'
+NEW_TMP_W_ID = 'An ID cannot be specified when creating a temp object.'
+CUS_ID_N_STR = 'Custom ID must be a string.'
+SRCH_NO_CAT  = 'Invalid operation: there is no catalog for instances ' \
+               'of class "%s".'
+MISS_O_CLASS = 'Catalog for class "%s": object not found for ID "%s".'
+MISS_O       = 'Object not found for ID "%s".'
+MISS_IDS_DEL = 'Catalog "%s": ID(s) removed because no corresponding ' \
+               'object(s) found: %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -81,6 +81,9 @@ class Config:
         # do it. You can place here roles expressed as strings or Role
         # instances, global or local.
         self.unlockers = ['Manager']
+        # Number of times a transaction is retried after a conflict error has
+        # occurred.
+        self.conflictRetries = 3
 
     def set(self, folder, filePath=None):
         '''Sets site-specific configuration elements. If filePath is None,
@@ -132,6 +135,11 @@ class Config:
             # Execute method named p_method on the tool
             database.run(method, handler, logger)
 
+    def getDatabaseSize(self, formatted=False):
+        '''Returns the database size'''
+        r = os.stat(self.filePath).st_size
+        return putils.getShownSize(r) if formatted else r
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Database:
     '''Represents the app's database'''
@@ -139,6 +147,11 @@ class Database:
     # Database-specific exception class
     class Error(Exception): pass
 
+    # ZODB exceptions that identify a conflict error
+    ConflictErrors = (transaction.interfaces.TransactionFailedError,
+                      transaction.interfaces.TransientError,
+                      ZODB.POSException.ConflictError)
+
     # Modulo for computing "ikeys" (see m_init's doc) from object's integer IDs
     MOD_IKEY = 10000
 
@@ -156,12 +169,16 @@ class Database:
         '''Closes the p_connection to this database'''
         try:
             connection.close()
-        except ConnectionStateError:
+        except ZODB.POSException.ConnectionStateError:
             # Some updates are probably uncommitted. Abort the current
             # transaction and then try again to close the connection.
             transaction.abort()
             connection.close()
 
+    def listConnections(self):
+        '''Returns the list of active connections to the database'''
+        return self.db.connectionDebugInfo()
+
     def commit(self, handler):
         '''Commits the current transaction related to p_handler'''
         transaction.commit()
@@ -181,7 +198,7 @@ class Database:
         if abort: transaction.abort()
         try:
             self.db.close()
-        except ConnectionStateError:
+        except ZODB.POSException.ConnectionStateError:
             # Force aborting any ongoing transaction
             transaction.abort()
             self.db.close()
@@ -193,7 +210,7 @@ class Database:
         # server startup.
         connection = self.db.open()
         # Make this connection available to the initialisation p_handler
-        handler.connection = connection
+        handler.dbConnection = connection
         # Get the root object
         root = connection.root
         # If the database is being p_created, add the following attributes to
@@ -330,9 +347,9 @@ class Database:
         # number of objects without needing to get information about them.
         # ~~~
         # Ensure there is a catalog for p_className
-        catalog = handler.connection.root.catalogs.get(className)
+        catalog = handler.dbConnection.root.catalogs.get(className)
         if not catalog:
-            raise self.Error(SEARCH_NO_CATALOG % className)
+            raise self.Error(SRCH_NO_CAT % className)
         # Get the IIDs of matching objects
         iids = catalog.search(handler, **kwargs)
         # Return IIDs if requested
@@ -344,9 +361,9 @@ class Database:
         '''(re)indexes this object in the catalog corresponding to its class'''
         # Ensure p_o is "indexable"
         className = o.class_.name
-        catalog = handler.connection.root.catalogs.get(className)
+        catalog = handler.dbConnection.root.catalogs.get(className)
         if not catalog:
-            raise self.Error(SEARCH_NO_CATALOG % className)
+            raise self.Error(SRCH_NO_CAT % className)
         return catalog.reindexObject(o, **kwargs)
         
     def count(self, className): pass
@@ -375,16 +392,16 @@ class Database:
     def clean(self, handler, logger):
         '''Cleaning the database means removing any temp object from it'''
         # Create a specific connection
-        connection = handler.connection = self.db.open()
+        connection = handler.dbConnection = self.db.open()
         # Get the temp store
         temp = getattr(connection.root, 'temp', None)
         dbFile = handler.server.config.database.filePath
         dbPath = str(dbFile)
         if temp is None:
-            return self.abort(connection, TEMP_STORE_NOT_FOUND % dbPath, logger)
+            return self.abort(connection, TMP_STORE_NF % dbPath, logger)
         count = len(temp)
         if count == 0:
-            logger.info(TEMP_STORE_EMPTY % dbPath)
+            logger.info(TMP_STORE_E % dbPath)
         else:
             # For removing temp objects, it is useless to call m_delete, because
             # the object is not yet linked to any other object, does not have
@@ -392,7 +409,7 @@ class Database:
             connection.root.temp = PersistentMapping()
             # Reset the ID counter
             connection.root.lastTempId = 0
-            logger.info(TEMP_OBJECTS_DELETED % (count, dbPath))
+            logger.info(TMP_OBJS_DEL % (count, dbPath))
             transaction.commit()
         connection.close()
         # Pack the database
@@ -401,7 +418,7 @@ class Database:
     def run(self, method, handler, logger):
         '''Executes method named m_method on the tool'''
         # Create a specific connection
-        connection = handler.connection = self.db.open()
+        connection = handler.dbConnection = self.db.open()
         # Get the store containing the tool
         root = connection.root
         store = getattr(root, 'objects', None)
@@ -410,14 +427,14 @@ class Database:
         # Ensure the "objects" store exists
         abort = self.abort
         if store is None:
-            return abort(connection, OBJECT_STORE_NOT_FOUND % dbPath, logger)
+            return abort(connection, O_STORE_NF % dbPath, logger)
         # Ensure the tool exists
-        tool = root.objects.get('tool')
+        tool = handler.tool = root.objects.get('tool')
         if tool is None:
-            return abort(connection, TOOL_NOT_FOUND % dbPath, logger)
+            return abort(connection, TOOL_NF % dbPath, logger)
         # Ensure p_method is defined on the tool
         if not hasattr(tool, method):
-            return abort(connection, METHOD_NOT_FOUND % method, logger)
+            return abort(connection, METH_NF % method, logger)
         # Execute the method
         try:
             getattr(tool, method)()
@@ -434,7 +451,7 @@ class Database:
            given integer p_id or from p_o's ID.'''
         id = id if o is None else o.id
         if not isinstance(id, int):
-            raise self.Error(CUSTOM_ID_NO_IKEY % id)
+            raise self.Error(CUS_ID_NO_IK % id)
         return abs(id) % Database.MOD_IKEY
 
     def newId(self, root, temp=False):
@@ -454,8 +471,8 @@ class Database:
            does not exist, it is created. If the p_root database object is None,
            it will be retrieved from p_o.'''
         # Get the root database object
-        root = root or o.H().connection.root
-        id = id or o.id
+        root = root or o.H().dbConnection.root
+        id = o.id if id is None else id
         # It may be the store of objects with custom IDs...
         if isinstance(id, str):
             r = root.objects
@@ -500,7 +517,7 @@ class Database:
         r = id in store if store else None
         # Return the result or raise an error when appropriate
         if r and raiseError:
-            raise self.Error(OBJECT_EXISTS % id)
+            raise self.Error(O_EXISTS % id)
         return r
 
     def new(self, handler, className, id=None, temp=False, secure=True,
@@ -537,18 +554,18 @@ class Database:
         # Ensure p_id's validity
         custom = id is not None
         if custom and not isinstance(id, str):
-            raise self.Error(CUSTOM_ID_NOT_STR)
+            raise self.Error(CUS_ID_N_STR)
         # Find the class corresponding to p_className
         class_ = handler.server.model.classes.get(className)
-        if not class_: raise self.Error(CLASS_NOT_FOUND % className)
+        if not class_: raise self.Error(CLASS_NF % className)
         # Security check
         guard = handler.guard
         if secure:
             guard.mayInstantiate(class_, checkInitiator=True, raiseOnError=True)
         # The root database object
-        root = handler.connection.root
+        root = handler.dbConnection.root
         # Define the object IDs: "id" and "iid"
-        if custom and temp: raise self.Error(NEW_TEMP_WITH_ID)
+        if custom and temp: raise self.Error(NEW_TMP_W_ID)
         iid = self.newId(root, temp=temp)
         # Determine the place to store the object
         store = self.getStore(id=iid, root=root, create=True)
@@ -568,7 +585,7 @@ class Database:
         '''Move the temp object p_o from the "temp" store to one of the final
            stores, "objects" or "iobjects".'''
         # The root database object
-        root = o.H().connection.root
+        root = o.H().dbConnection.root
         # A method named "generateId" may exist on p_o's class, for producing a
         # database ID for the object. If such method is found, it must return a
         # string ID and the object will be added to store "objects" in addition
@@ -595,10 +612,10 @@ class Database:
            The possible p_initiator object may be given. Returns a message to
            return to the UI, or None if p_o has already been deleted.'''
         o.H().commit = True
-        # Additionally, if p_o is a temp object, the operation has the effect of
-        # "converting" it to a "final" object, by changing its ID and moving it
-        # from the "temp" store to a final store: "objects" or "iobjects".
         isTemp = o.isTemp()
+        # If p_o is temp, convert it to a "final" object, by changing its ID and
+        # moving it from dict "temp" to "objects" or "iobjects".
+        if isTemp: self.move(o)
         # If p_o is not temp, as a preamble, remember the previous values of
         # fields, for potential historization.
         currentValues = None if isTemp \
@@ -620,20 +637,13 @@ class Database:
             if not self.exists(o=o): return
         # Manage the relationship between the initiator and the new object
         if isTemp and initiator:
-            initiator.manage(o)
-            if not self.exists(o=o): return
-        # Store the "cid" if the object is not temp
-        if not isTemp: o.cid = o.computeCid()
-        # Prepare the method result: a potential translated message
-        r = None
-        # Call the custom "onEdit" if available
-        r = multicall(o, 'onEdit', False, isTemp)
-        if not self.exists(o=o): return
+            r = initiator.manage(o)
+            if not self.exists(o=o): return r
         # Update last modification date
         if not isTemp: o.history.modified = DateTime()
-        # If p_o is temp, convert it to a "final" object by moving it from dict
-        # "temp" to "objects" or "iobjects".
-        if isTemp: self.move(o)
+        # Call the custom "onEdit" if available. It may return a translated msg.
+        r = multicall(o, 'onEdit', False, isTemp)
+        if not self.exists(o=o): return
         # Unlock the currently saved page on the object
         Lock.remove(o, o.req.page)
         # Reindex the object when appropriate
@@ -641,21 +651,50 @@ class Database:
             o.reindex()
         return r or o.translate('object_saved')
 
-    def delete(self, o, historize=False, executeMethods=True, root=None):
+    def delete(self, o, historize=False, executeMethods=True, root=None,
+               ignore=None):
         '''Delete object p_o from the database. When unlinking it from other
            objects, if the concerned Ref fields are historized and p_historize
-           is True, this deletion is noted in tied object's histories.'''        
-        handler = o.H()
-        handler.commit = True
+           is True, this deletion is noted in tied object's histories.'''
+        # ~~~
+        # If p_executeMethods is False, the custom "onDelete" method, even if
+        # present on p_o, will not be executed.
+        # ~~~
+        # If this object is cascade-deleted from another one, p_ignore contains
+        # the back ref pointing to the deleting object, that must not be walked
+        # again.
+        # ~~~
+        # For performance, when available, the p_root database object is passed
+        # ~~~
+        # If the deletion is aborted by custom method "onDelete", a translated
+        # error message is returned.
+        # ~~~
         isTemp = o.isTemp()
         # Call a custom "onDelete" if it exists
         if not isTemp and executeMethods and hasattr(o, 'onDelete'):
-            o.onDelete()
-        # Remove any link to any other object
+            message = o.onDelete()
+            # If a return message is present, the deletion is aborted
+            if message: return message
+        handler = o.H()
+        handler.commit = True
+        # Remove any link to any object and cascade-delete contained objects
         title = o.getShownValue()
         for field in o.class_.fields.values():
+            # Browse only Ref fields
             if field.type != 'Ref': continue
+            # Do not browse the field mentioned in p_ignore
+            if ignore and (field.name == ignore.name): continue
+            # Browse tied objects
+            contained = field.composite
             for tied in field.getValue(o, single=False):
+                # Delete the tied object if contained within this one
+                if contained:
+                    self.delete(tied, historize=historize, ignore=field.back,
+                                executeMethods=executeMethods, root=root)
+                    continue
+                # Unlink the tied object else
+                back = field.back
+                if back is None: continue
                 field.back.unlinkObject(tied, o, back=True)
                 # Historize this unlinking when relevant
                 if historize and field.back.getAttribute(tied, 'historized'):
@@ -670,13 +709,19 @@ class Database:
             # Try to move it to the OS temp folder; if it fails, delete it
             putils.FolderDeleter.delete(folder, move=True)
         # Get the store containing p_o
-        root = root or o.H().connection.root
+        root = root or o.H().dbConnection.root
         iid = o.iid
         store = self.getStore(id=iid, root=root)
         # Remove p_o from this store, and potentially from root.objects, too
-        del(store[iid])
+        try:
+            del(store[iid])
+        except KeyError:
+            pass # The object may already have been deleted
         if o.id != iid:
-            del(root.objects[o.id])
+            try:
+                del(root.objects[o.id])
+            except KeyError:
+                pass # The object may already have been deleted
 
     def getObject(self, handler, id, logMissing=False, className=None):
         '''Gets an object given its p_id'''
@@ -687,15 +732,15 @@ class Database:
         # Convert p_id to an int if it is an integer coded in a string
         if isinstance(id, str) and (id.isdigit() or id.startswith('-')):
             id = int(id)
-        store = self.getStore(id=id, root=handler.connection.root)
+        store = self.getStore(id=id, root=handler.dbConnection.root)
         r = store.get(id) if store else None
         if (r is None) and logMissing:
             # Log the fact that no object has been found
             sid = str(id)
             if className:
-                message = MISSING_OBJECT_CLASS % (className, sid)
+                message = MISS_O_CLASS % (className, sid)
             else:
-                message = MISSING_OBJECT % sid
+                message = MISS_O % sid
             handler.log('app', 'error', message)
         return r
 
@@ -718,6 +763,9 @@ class Database:
                 if o is not None:
                     # The object has been found; add it to the result
                     r.append(o)
+                    # Stop now if p_size is reached
+                    if (size is not None) and (len(r) >= size):
+                        break
                 else:
                     # No object was found: add the id among missing IDs
                     if missing is None: missing = []
@@ -741,27 +789,60 @@ class Database:
                 i += 1
         # If missing IDs were found, clean it from the corresponding catalog
         if missing and className:
-            catalog = handler.connection.root.catalogs.get(className)
+            catalog = handler.dbConnection.root.catalogs.get(className)
             if catalog:
                 for id in missing:
                     catalog.unindexObject(id)
-                message = MISSING_IDS_DELETED % (className, ', '.join(missing))
+                missing = [str(iid) for iid in missing]
+                message = MISS_IDS_DEL % (className, ', '.join(missing))
                 handler.log('app', 'info', message)
                 handler.commit = True # Force a commit in that case
         return r
 
-    def getFolder(self, o, create=True):
+    def getFolder(self, o, create=True, withRelative=False):
         '''Gets, as a pathlib.Path instance, the folder where binary files
            related to p_o are (or will be) stored on the database-controlled
            filesystem. If p_create is True and the folder does not exist, it is
            created (together with potentially missing parent folders).'''
-        # Start with the root folder storing site binaries
+        # If p_withRelative is True, the methods returns a tuple
+        #                         (path, relative),
+        # where "path" is the Path instance as described hereabove, and
+        # "relative" is a string containing the part of the path that does not
+        # contain its start (= the root folder containing binaries).
+        # ~~~
+        # Start with getting the root folder storing site binaries
         r = o.config.database.binariesFolder
         # Add the object-specific path, depending on its ID
         id = o.iid
-        r = r / str(self.getIkey(id=id)) / str(abs(id))
+        ikey = str(self.getIkey(id=id))
+        sid = str(abs(id))
+        r = r / ikey / sid
         # Create this folder if p_create is True and it does not exist yet
         if create and not r.exists():
             r.mkdir(parents=True)
-        return r
+        # Return the path, together with its relative part if requested
+        return r if not withRelative else (r, ('%s/%s' % (ikey, sid)))
+
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    #                                 PXs
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    view = Px('''
+     <h2>Main configuration</h2>
+     <table class="small" var="cfg=config.database">
+      <tr><th>Database (ZODB)</th><td>:cfg.filePath</td></tr>
+      <tr><th>Database size</th><td>:cfg.getDatabaseSize(True)</td></tr>
+      <tr><th>Binaries</th><td>:cfg.binariesFolder</td></tr>
+     </table>
+
+     <x var="connections=handler.server.database.listConnections()">
+      <h2>Active connections <x>:'(%d)' % len(connections)</x></h2>
+      <table class="small">
+       <tr for="info in connections">
+        <th>:info['info']</th>
+        <td>
+         <div for="k,v in info.items()" if="k!='info'">
+          <b>:k</b> : <x>:v or '-'</x></div>
+        </td></tr>
+      </table></x>''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
