@@ -37,12 +37,12 @@ from appy.model.utils import Object as O
 from appy.database.operators import Operator
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CATALOG_CREATED = 'Catalog created for class "%s".'
-CATALOG_REMOVED = 'Catalog removed for class "%s".'
-INDEXES_POPULATED = '%d/%d object(s) reindexed during population of ' \
-  'index(es) %s.'
-INDEXES_CLEANED = '%d catalog entries removed (inexistent objects).'
-INDEX_NOT_FOUND = 'There is no indexed field named "%s" on class "%s".'
+CTL_CREATED = 'Catalog created for class "%s".'
+CTL_REMOVED = 'Catalog removed for class "%s".'
+IDXS_POP    = '%d/%d object(s) reindexed during population of index(es) %s.'
+IDXS_CLEAN  = '%d catalog entries removed (inexistent objects).'
+IDX_NF      = 'There is no indexed field named "%s" on class "%s".'
+OBJECT_NF   = 'Object with IID %d does not exist in catalog "%s".'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Changes:
@@ -138,10 +138,10 @@ class Catalog(PersistentMapping):
             for className, indexes in populate.items():
                 for index in indexes:
                     names.append('%s::%s' % (className, index.name))
-            message = INDEXES_POPULATED % (counts.updated, counts.total, names)
+            message = IDXS_POP % (counts.updated, counts.total, names)
             handler.log('app', 'info', message)
         if counts.lost:
-            message = INDEXES_CLEANED % len(counts.lost)
+            message = IDXS_CLEAN % len(counts.lost)
             handler.log('app', 'warning', message)
 
     @classmethod
@@ -170,7 +170,7 @@ class Catalog(PersistentMapping):
             indexable[name] = None
             if name not in catalogs:
                 catalog = catalogs[name] = Catalog(handler, modelClass)
-                handler.log('app', 'info', CATALOG_CREATED % name)
+                handler.log('app', 'info', CTL_CREATED % name)
             else:
                 catalog = catalogs[name]
             # Potentially update indexes in this catalog
@@ -181,7 +181,7 @@ class Catalog(PersistentMapping):
         for name in names:
             if name not in indexable:
                 del(catalogs[name])
-                handler.log('app', 'info', CATALOG_REMOVED % name)
+                handler.log('app', 'info', CTL_REMOVED % name)
         # Populate indexes requiring it
         if populate:
             class_.populate(root, handler, populate)
@@ -235,7 +235,7 @@ class Catalog(PersistentMapping):
         '''Get the index named p_name or raise an exception if no such index
            exists in this catalog.'''
         r = self.get(name)
-        if not r: raise self.Error(INDEX_NOT_FOUND % (name, self.name))
+        if not r: raise self.Error(IDX_NF % (name, self.name))
         return r
 
     def getField(self, name, handler):
@@ -323,7 +323,7 @@ class Catalog(PersistentMapping):
         # Convert the IITreeSet to a sorted list of IIDs and return it
         return self.getSorted(r, sortBy, sortOrder)
 
-    def unindexObject(self, iid):
+    def unindexObject(self, iid, log=None):
         '''Unindexes object having this p_iid from this catalog. Returns True if
            at least one entry from one index has actually been removed within
            this catalog.'''
@@ -332,7 +332,11 @@ class Catalog(PersistentMapping):
             changed = index.unindexObject(iid)
             if changed: r = True
         # Remove it from p_self.all
-        self.all.remove(iid)
+        try:
+            self.all.remove(iid)
+        except KeyError:
+            # The object is not in the catalog, issue a warning
+            if log: log(OBJECT_NF % (iid, self.name), type='warning')
         return r
         
     def reindexObject(self, o, fields=None, indexes=None, unindex=False,
@@ -371,7 +375,7 @@ class Catalog(PersistentMapping):
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         r = False
         # Manage unindexing
-        if unindex: return self.unindexObject(o.iid)
+        if unindex: return self.unindexObject(o.iid, o.log)
         # Manage (re)indexing
         if indexes:
             # The list of indexes to update is given
@@ -385,7 +389,7 @@ class Catalog(PersistentMapping):
                 for name in fields:
                     # Get the corresponding index
                     if name not in self:
-                        raise self.Error(INDEX_NOT_FOUND % (name, self.name))
+                        raise self.Error(IDX_NF % (name, self.name))
                     changed = self[name].indexObject(o)
                     if changed: r = True
             else:
