@@ -20,19 +20,27 @@ import persistent
 from appy.px import Px
 from appy.ui import Iframe
 from appy.utils import multicall
+from appy.model.fields import Show
 from appy.database.lock import Lock
 from appy.ui.template import Template
 from appy.ui.criteria import Criteria
+from appy.model.fields.ref import Ref
 from appy.ui.validate import Validator
+from appy.model.searches import Search
+from appy.model.fields.text import Text
+from appy.model.fields.date import Date
 from appy.model.utils import Object as O
 from appy.database.catalog import Catalog
 from appy.ui.layout import Layout, Layouts
 from appy.xml.marshaller import Marshaller
-from appy.model.fields.phase import UiPhases
+from appy.model.fields.string import String
+from appy.model.fields.action import Action
+from appy.model.fields.computed import Computed
+from appy.model.workflow.history import History
+from appy.model.fields.phase import Page, UiPhases
 from appy.model.workflow import standard as workflows
+from appy.model.fields.select import Select, Selection
 from appy.utils.string import Normalize, produceNiceMessage
-from appy.all import String, Text, Date, Select, Selection, Computed, \
-                     Ref, Search, Show, Action
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 READ_ONLY_EXPR = 'This is a read-only property: you cannot set a value for it.'
@@ -85,9 +93,9 @@ class Base:
     # for displaying base information about an object.
     titleAttributes = {'multiplicity': (1,1), 'show': Show.EX, 'indexed': True,
       'searchable': True, 'filterField': searchable,
-      # This method allows to produce an index value suitable for sorting,
+      # This method allows to produce an index value suitable for sorting
       'indexValue': lambda o, v: Normalize.text(o.getShownValue(),
-                                                keepBlank=False)}
+                                                keepBlank=False) if v else ''}
 
     title = String(**titleAttributes)
 
@@ -95,7 +103,12 @@ class Base:
     p = {'multiplicity': (1,1), 'show': 'xml', 'indexed': True, 'label': 'Base'}
     creator = Computed(method=lambda o: o.history[-1].login, **p)
 
+    def setCreator(self, login):
+        '''Sets p_login as p_self's creator'''
+        self.history[-1].login = login
+
     # Object's creation and last modification dates
+    p['show'] = Show.VE_
     created = Date(default=lambda o: o.history[-1].date, persist=False, **p)
     modified = Date(default=lambda o: o.history.modified, persist=False, **p)
 
@@ -131,6 +144,17 @@ class Base:
     allowed = Select(multiplicity=(0, None), show=False, persist=False,
                      indexed=True, default=getAllowed, label='Base')
 
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # History
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    recordAttributes = {'method': History.view, 'label':'Base', 'layouts':'f',
+      'page':Page('history', label='Base_page_history',
+                  sticky=True, icon='history.svg',
+                  show=lambda o: 'view' if o.user.hasRole('Manager') else None)}
+
+    record = Computed(**recordAttributes)
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Properties
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -213,12 +237,17 @@ class Base:
         return self.getClass().workflow
 
     def do(self, name, comment=None, doAction=True, doHistory=True,
-           secure=False, data=None):
+           secure=False, doSay=False, data=None):
         '''Programmatically triggers on p_self a transition named p_name'''
-        # p_data can be a dict that will be included into the history event
+        # p_data can be a dict that will be included into the history event.
+        # ~~~
+        # If p_doSay is True, it will not lead to the UI message being
+        # automatically included in the response cookie, but will allow the
+        # caller to retrieve it (via the return value) and do what he wants with
+        # it.
         transition = self.getWorkflow().transitions.get(name)
         return transition.trigger(self, comment=comment, doAction=doAction,
-                     doHistory=doHistory, doSay=False, secure=secure, data=data)
+                     doHistory=doHistory, doSay=doSay, secure=secure, data=data)
 
     def getPageLayout(self, layout):
         '''Returns the page layout corresponding to p_layout for p_self'''
@@ -258,7 +287,7 @@ class Base:
 
     def isTemp(self):
         '''Is this object temporary ?'''
-        return self.id in self.H().connection.root.temp
+        return self.id in self.H().dbConnection.root.temp
 
     def log(self, message, type='info', noUser=False):
         '''Logs a p_message of some p_type'''
@@ -313,16 +342,20 @@ class Base:
             raise AttributeError(FIELD_NOT_FOUND % (name, class_.name))
         return r
 
-    def getFields(self, layout, page=None, type=None, fields=None):
-        '''Returns the list of fields from a given p_page on a given p_layout.
-           If p_page is None, fields of all pages are returned. If p_type is
-           defined, only fields of this p_type are returned. If p_fields are
-           given, they are used instead of using self's fields.'''
+    def getFields(self, layout, page=None, phase=None, type=None, fields=None):
+        '''Returns the list of fields to render on p_layout'''
         r = []
+        # Browse p_self's fields, or p_fields if passed
         for field in (fields or self.class_.fields.values()):
+            # Ignore fields not being on p_page
             if page and (field.pageName != page): continue
+            # Ignore fields not being on any page in p_phase
+            if phase and (field.page and (field.page.phase != phase)): continue
+            # Ignore fields not being of p_type
             if type and (field.type != type): continue
+            # Ignore fields that can't be rendered on p_layout
             if not field.isRenderable(layout): continue
+            # Ignore unshowable fields 
             if not field.isShowable(self, layout): continue
             r.append(field)
         return r
@@ -631,7 +664,47 @@ class Base:
         # Use this method instead of writing len(self.search(...))
         return len(self.search(className=className, ids=True, **kwargs))
 
-    def compute(self, *args, **kwargs): pass
+    def compute(self, className, sortBy=None, context=None, expression=None,
+                secure=False, max=None, **fields):
+        '''Computes a result based on a search on objects of p_className'''
+        # Like m_search and m_count above, this method performs a search on
+        # objects of p_className; but in this case, instead of returning a list
+        # of matching objects (like m_search) or counting elements (like
+        # p_count), it evaluates, on every matching object (possibly restricted
+        # to at most p_max objects), a Python p_expression (which may be an
+        # expression or a statement), and returns, if needed, a result. The
+        # result may be initialized through parameter p_context. p_expression is
+        # evaluated with 2 variables in its context: "o" which is the currently
+        # walked object, instance of p_className, and "ctx", which is the
+        # context as initialized (or not) by p_context. p_context may be used as
+        # (1) a variable or instance that is updated on every call to produce a
+        #     result;
+        # (2) an input variable or instance;
+        # (3) both.
+
+        # The method returns p_context, modified or not by evaluation of
+        # p_expression on every matching object.
+
+        # When you need to perform an action or computation on a lot of objects,
+        # use this method instead of doing things like
+           
+        #              "for o in self.search(MyClass,...)"
+        
+        # Initialize the context variable "ctx"
+        ctx = context
+        i = 0
+        for iid in self.search(className, ids=True, sortBy=sortBy, \
+                               secure=secure, **fields):
+            # Stop if the maximum number of objects has been reached
+            if (max is not None) and (i >= max): break
+            # Get the object from its ID
+            o = self.getObject(iid, logMissing=True)
+            # Ignore dead IDs
+            if o is None: continue
+            i += 1
+            # Evaluate the expression
+            if expression: exec(expression)
+        return ctx
 
     def reindex(self, **kwargs):
         '''(re-/un-)indexes this object in the catalog corresponding to its
@@ -644,7 +717,7 @@ class Base:
 
     def getCatalog(self, raiseError=False):
         '''Returns the catalog storing p_self (if indexed)'''
-        r = self.H().connection.root.catalogs.get(self.class_.name)
+        r = self.H().dbConnection.root.catalogs.get(self.class_.name)
         if not r and raiseError:
             raise Exception(CATALOG_NOT_FOUND % self.class_.name)
         return r
@@ -653,140 +726,120 @@ class Base:
     # Sub-PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    # The header displays, for a given object, its title, breadcrumb, siblings
-    # navigation, phases and pages.
+    # The header displays, for a given object, its title and breadcrumb
     pxHeader = Px('''
-     <table width="100%">
-      <tr valign="top">
-       <!-- Breadcrumb -->
-       <td var="breadcrumb=ui.Breadcrumb(o, popup)" if="breadcrumb.parts"
-                class="breadcrumb">
-        <x if="breadcrumb.sup">::breadcrumb.sup</x>
-        <x for="part in breadcrumb.parts"
-           var2="link=not loop.part.last and part.view">
-         <img if="not loop.part.first" src=":url('to')"/>
-         <!-- Display the title or a link to the object -->
-         <span if="not link">::part.title</span>
-         <!-- Display a link for parent objects -->
-         <a if="link" href=":part.url">::part.title</a>
-        </x>
-        <x if="breadcrumb.sub">::breadcrumb.sub</x>
-       </td>
-       <!-- Object navigation -->
-       <td var="nav=req.nav" if="nav and (nav != 'no')" align=":dright"
-           var2="self=ui.Siblings.get(nav, tool, _ctx_)">:self.pxNavigate</td>
-      </tr>
-     </table>
-     <!-- Object phases and pages -->
-     <x if="o.class_.mayNavigate(o)">:phases.view</x>''',
-
-     css='.breadcrumb { font-size: 130%; padding-bottom: 6px }')
+     <div var="breadcrumb=ui.Breadcrumb(o, popup)"
+          if="breadcrumb.parts" class=":breadcrumb.css">
+      <x if="breadcrumb.sup">::breadcrumb.sup</x>
+      <x for="part in breadcrumb.parts"
+         var2="link=not loop.part.last and part.view">
+       <img if="not loop.part.first" src=":url('arrowsA.svg')"
+            class="iconS" style="transform: rotate(270deg)"/>
+       <!-- Display the title or a link to the object -->
+       <span if="not link">::part.title</span>
+       <!-- Display a link for parent objects -->
+       <a if="link" href=":part.url">::part.title</a>
+      </x>
+      <x if="breadcrumb.sub">::breadcrumb.sub</x>
+     </div>''')
 
     # The grouped fields to show on a given page
     pxFields = Px('''
      <table width=":table.width | '100%'">
       <tr for="field in grouped"><td>:field.pxRender</td></tr></table>''')
 
-    # The range of buttons (next, save,...) and the workflow transitions
+    # PX displaying the content of a single field
+    pxField = Px('''
+     <div style="padding: 30px"
+          var="field=o.getField(req.name);
+               layout='view'">:field.pxRender</div>''',
+     template=Template.px, hook='content')
+
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # Object controls
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    pxControls = Px('''
+     <!-- Object phases and pages. pxButtons will be called and inserted into
+          the main phase. -->
+
+     <x var="isEdit=layout == 'edit';
+             nav=req.nav or 'no';
+             current=phases.currentPage;
+             compact=popup or config.ui.compact;
+             picto='pictoC' if compact else 'picto';
+             mayAct=not isEdit and guard.mayAct(o)">:phases.view</x>''')
+
+    # Standard, custom and workflow-related actions
     pxButtons = Px('''
-     <div class="objectButtons"
-          var="isEdit=layout == 'edit';
-               current=phases.currentPage;
-               nav=req.nav or 'no';
-               mayAct=not isEdit and guard.mayAct(o)">
-      <!-- Previous -->
-      <x if="current.previous and current.showPrevious"
-         var2="previous=current.previous;
-               label=_('page_previous');
-               css=ui.Button.getCss(label, small=False)">
-       <!-- Button on the edit page -->
-       <x if="isEdit">
-        <input type="button" class=":css" value=":label" name="previous"
-           onclick=":'submitAppyForm(%s,%s,%s)' % \
-                     (q('save'), q(previous.name), q('edit'))"
-           style=":url('previous', bg=True)"/>
-       </x>
-       <!-- Button on the view page -->
-       <input if="not isEdit" type="button" class=":css" value=":label"
-          style=":url('previous', bg=True)" name="previous"
-          onclick=":'goto(%s)' % \
-           q(o.getUrl(sub='view',page=previous.name,popup=popup,nav=nav))"/>
-      </x>
+     <x if="phase.name == 'main'">
+      <!-- Standard actions -->
+
       <!-- Save -->
-      <input if="isEdit and current.showSave" type="button" name="save"
-         var2="label=_('object_save');
-               css=ui.Button.getCss(label, small=False)"
-          onclick=":'submitAppyForm(%s,%s,%s)' % \
-                    (q('save'), q(current.name), q('view'))"
-          class=":css" value=":label" style=":url('save', bg=True)" />
-      <!-- Cancel -->
-      <input if="isEdit and current.showCancel" type="button" name="cancel"
-         var2="label=_('object_cancel');
-               css=ui.Button.getCss(label, small=False)"
-         onclick=":'submitAppyForm(%s,%s,%s)' % \
-                    (q('cancel'), q(current.name), q('view'))"
-         class=":css" value=":label" style=":url('cancel', bg=True)"/>
-      <!-- Edit -->
-      <x if="not isEdit"
-         var2="locked=o.Lock.isSet(o, user, page);
-               editable=current.showOnEdit and current.showEdit and \
-                        mayAct and guard.mayEdit(o)">
-       <input if="editable and not locked" type="button" name="edit"
-          var="label=_('object_edit');
-               css=ui.Button.getCss(label, small=False)"
-          value=":label" class=":css" style=":url('edit', bg=True)"
-          onclick=":'goto(%s)' % \
-                    q(o.getUrl(sub='edit',page=page,popup=popup,nav=nav))"/>
-       <x if="locked" var2="lockStyle='Big'">::o.Lock.px</x>
-       <!-- Delete -->
-       <input if="not locked and not popup and (page == 'main') and \
-                  guard.mayDelete(o)" type="button"
-          var2="label=_('object_delete');
-                css=ui.Button.getCss(label, small=False)"
-          value=":label" class=":css" style=":url('delete', bg=True)"
-          onclick=":'onDeleteObject(%s)' % q(o.url)"/>
-      </x>
-      <!-- Next -->
-      <x if="current.next and current.showNext"
-         var2="next=current.next;
-               label=_('page_next');
-               css=ui.Button.getCss(label, small=False)">
-       <!-- Button on the edit page -->
-       <x if="isEdit">
-        <input type="button" class=":css"
+      <div if="isEdit and current.showSave" var2="label=_('object_save')">
+       <div>
+        <a class="clickable"
            onclick=":'submitAppyForm(%s,%s,%s)' % \
-                     (q('save'), q(next.name), q('edit'))"
-           name="next" style=":url('next', bg=True)" value=":label"/>
-       </x>
-       <!-- Button on the view page -->
-       <input if="not isEdit" type="button" class=":css" value=":label"
-          style=":url('next', bg=True)" name="next"
-          onclick=":'goto(%s)' % \
-           q(o.getUrl(sub='view',page=next.name,popup=popup,nav=nav))"/>
-      </x>
-      <!-- Fields (actions) defined with layout "buttons" -->
-      <x if="not isEdit"
-         var2="fields=o.getFields('buttons', page, type='Action');
-               layout='view'">
-       <!-- Call view and not pxRender to avoid having a table -->
-       <x for="field in fields" var2="name=field.name">:field.view</x>
-      </x>
+                      (q('save'), q(current.name), q('view'))">
+         <img src=":url('save.svg')" class=":picto"/><br/>
+         <x>::label</x>
+        </a>
+       </div>
+      </div>
+
+      <!-- Cancel -->
+      <div if="isEdit and current.showCancel" var2="label=_('object_cancel')">
+       <div>
+        <a class="clickable" name="cancel"
+           onclick=":'submitAppyForm(%s,%s,%s)' % \
+                      (q('cancel'), q(current.name), q('view'))">
+         <img src=":url('cancel.svg')" class=":picto"/><br/>
+         <x>::label</x>
+        </a>
+       </div>
+      </div>
+
+      <!-- Delete -->
+      <div if="not isEdit and not popup and guard.mayDelete(o) and \
+               not o.Lock.isSet(o, user, page)" var2="label=_('object_delete')">
+       <div>
+        <a class="clickable" onclick=":'onDeleteObject(%s)' % q(o.url)">
+         <img src=":url('delete.svg')" class=":picto"/><br/>
+         <x>::label</x>
+        </a>
+       </div>
+      </div>
+
       <!-- Workflow transitions -->
-      <x if="mayAct and (page == 'main') and o.class_.showTransitions(o,layout)"
-         var2="workflow=o.getWorkflow()">:workflow.pxTransitions</x>
-     </div>''')
+      <x if="mayAct and o.class_.showTransitions(o, layout)"
+         var2="workflow=o.getWorkflow()">
+       <x var="transitions=workflow.getTransitions(o)" if="transitions">
+        <x>:workflow.pxTransitions</x></x>
+      </x>
+     </x>
+
+     <!-- Fields on layout "buttons". Layout is "defined as view" and not
+          "cell", because "cell" would render a too minimalistic widget. -->
+     <x if="not isEdit"
+        var2="fields=o.getFields('buttons', phase=phase.name);
+              layout='view'">
+      <!-- Call view and not pxRender to avoid having a table -->
+      <div for="field in fields"
+           var2="name=field.name; value=field.getValueIf(o, name, layout)">
+       <div>:field.cell</div></div>
+     </x>''')
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Objet creation method
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def create(self, name, secure=False, raiseOnWrongAttribute=True,
+    def create(self, _name, secure=False, raiseOnWrongAttribute=True,
                executeMethods=True, initialComment=None, initialState=None,
                **kwargs):
         '''Create a new instance of a Appy class'''
-        # If p_name is the name of a field, the created object will be linked
-        # to p_self via this field. Else, p_name must correspond to a root class
-        # and no link will exist between the created instance and p_self.
+        # If p__name is the name of a field, the created object will be linked
+        # to p_self via this field. Else, p__name must correspond to a root
+        # class and no link will exist between the created instance and p_self.
 
         # p_kwargs allow to specify values for object fields. If p_secure is
         # False, security checks will not be performed.
@@ -808,15 +861,15 @@ class Base:
         id = kwargs.pop('id') if 'id' in kwargs else None
 
         # Must we create an instance of a root class or one via a Ref field ?
-        if name[0].islower():
-            # p_name is the name of a Ref field
-            field = self.getField(name)
+        if _name[0].islower():
+            # p__name is the name of a Ref field
+            field = self.getField(_name)
             className = field.class_.meta.name
             # Check that the user can edit this field
             if secure: field.checkAdd(self)
         else:
             # p_name is the name of a root class
-            className = name
+            className = _name
             field = None
         # Create the object
         handler = self.H()
@@ -834,8 +887,6 @@ class Base:
         if executeMethods: multicall(o, 'onEditEarly', False, None)
         # Link the created object to its initiator when relevant
         if field: field.linkObject(self, o, executeMethods=executeMethods)
-        # Define the object CID
-        o.cid = o.computeCid()
         # Call custom initialization: a hook allowing the app developer to
         # initialise its object after it's been linked to its initiator object.
         if executeMethods: multicall(o, 'onEdit', False, True)
@@ -844,7 +895,27 @@ class Base:
             o.reindex()
         return o
 
-    def createFrom(self, name, other, secure=False, executeMethods=True,
+    def getCopyValues(self, exclude=(), fields={}):
+        '''Returns a dict ~{s_fieldName, fieldValue}~ representing p_self's
+           dataset ready-to-be-copied to another object.'''
+        # Check m_createFrom below to know more about method args
+        # ~~~
+        r = {}
+        # Browse field values from p_self and store "copyable" ones in "r"
+        for field in self.class_.fields.values():
+            # Skip non persistent fields, back references & p_exclude(d) fields
+            if not field.persist or (field.name in exclude) or \
+               ((field.type == 'Ref') and field.isBack): continue
+            # Use the value in p_fields if given
+            if field.name in fields:
+                value = fields[field.name]
+            else:
+                value = field.getCopyValue(self)
+            if value is not None:
+                r[field.name] = value
+        return r
+
+    def createFrom(self, _name, other, secure=False, executeMethods=True,
                    exclude=(), keepBase=False, fields={}):
         '''Similar to m_create above, excepted that another object (p_other) is
            used as base for filling in data for the object to create.'''
@@ -857,21 +928,11 @@ class Base:
         # Ref fields from p_self will be referenced by the clone, but not
         # themselves copied.
 
-        # Get the field values to set from p_other and store it in a dict.
-        # p_other may not be of the same class as p_self.
-        params = {}
-        for field in other.class_.fields.values():
-            # Skip non persistent fields, back references and p_excluded fields
-            if not field.persist or (field.name in exclude) or \
-               ((field.type == 'Ref') and field.isBack): continue
-            # Use the value in p_fields if given
-            if field.name in fields:
-                value = fields[field.name]
-            else:
-                value = field.getCopyValue(other)
-            if value is not None:
-                params[field.name] = value
-        r = self.create(name, secure=secure, raiseOnWrongAttribute=False,
+        # Get, as a dict, all field values to set from p_other. p_other may not
+        # be of the same class as p_self.
+        params = other.getCopyValues(exclude, fields)
+        # Create the new instance
+        r = self.create(_name, secure=secure, raiseOnWrongAttribute=False,
                         executeMethods=executeMethods, **params)
         # Propagate base attributes if required
         if keepBase:
@@ -886,7 +947,7 @@ class Base:
 
     def delete(self, historize=False, executeMethods=True):
         '''Deletes p_self (see homonym method on the database object'''
-        self.H().server.database.delete(self, historize, executeMethods)
+        return self.H().server.database.delete(self, historize, executeMethods)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Translation method
@@ -894,13 +955,11 @@ class Base:
 
     def translate(self, label, mapping=None, language=None, asText=False,
                   field=None, blankOnError=False):
-        '''Translates a given p_label with p_mapping.
-
-           If p_field is given, p_label does not correspond to a full label
-           name, but to a label type linked to p_field: "label", "descr"
-           or "help". Indeed, in this case, a specific i18n mapping may be
-           available on the field, so we must merge this mapping into
-           p_mapping.'''
+        '''Translates a given p_label with p_mapping'''
+        # If p_field is given, p_label does not correspond to a full label name,
+        # but to a label type linked to p_field: "label", "descr" or "help".
+        # Indeed, in this case, a specific i18n mapping may be available on the
+        # field, so we must merge this mapping into p_mapping.
         handler = self.H()
         # In what language must we get the translation ?
         language = language or handler.guard.userLanguage
@@ -920,7 +979,7 @@ class Base:
                     return field.translations.get(label, language, mapping)
             label = getattr(field, '%sId' % label)
         # Get the appropriate Translation object, or "en" if not found
-        store = handler.connection.root.objects
+        store = handler.dbConnection.root.objects
         translation = store.get(language) or store.get('en')
         # Get the label from this translation object, or from "en" if not found
         if translation:
@@ -934,8 +993,8 @@ class Base:
         # a translated message.
         if not r and not blankOnError:
             return produceNiceMessage(label.rsplit('_', 1)[-1])
-        # Convert to text when required
-        return r if not asText else r.replace('<br/>', '\n')
+        # Manage carriage returns, depending on the desired format
+        return r.replace('<br/>', '\n') if asText else r
 
     def callOnView(self):
         '''Call "onView" method if present'''
@@ -958,26 +1017,30 @@ class Base:
 
     # The object, as shown in a list of referred (tied) objects
     pxTied = Px('''
-     <tr valign="top" class=":rowCss"
-         var="ifield=initiator.field;
+     <tr var="ifield=initiator.field;
               *=ifield.initPx(initiator.o, req, _ctx_) if _ctx_.ajaxSingle \
                                                        else None;
               id=o.iid;
               objectIndex=ifield.getIndexOf(initiator.o, o, raiseError=False);
               mayView=guard.mayView(o);
               cbId='%s_%s' % (hook, currentNumber)"
-         id=":id">
+         id=":id" valign=":ifield.rowAlign" class=":rowCss">
       <td for="column in columns" width=":column.width" align=":column.align"
-          var2="field=column.field">
+          var2="field=column.field; isCB=field=='_checkboxes'"
+          class=":selector.cbClass if isCB and selector else ''">
+
        <!-- Special column with the tied element's number -->
-       <x if="(field=='_number') and (objectIndex!=None)">:ifield.pxNumber</x>
+       <x if="(objectIndex!=None) and (field=='_number')">:ifield.pxNumber</x>
+
        <!-- Special column containing the tied element's checkbox -->
-       <input if="(field == '_checkboxes') and mayView" type="checkbox"
-              name=":hook" id=":cbId"
+       <input if="isCB and mayView" type="checkbox" name=":hook" id=":cbId"
               var2="checked=cbChecked|False" checked=":checked"
               value=":id" onclick="toggleCb(this)"/>
+
+       <!-- Any other column -->
        <x if="not column.special">:field.pxTied</x>
       </td>
+
       <!-- Store data in this tr node allowing to ajax-refresh it -->
       <script>:ifield.getAjaxDataRow(o, hook, rowCss=rowCss, \
                currentNumber=currentNumber, cbChecked=cbId, ajaxSingle=True, \
@@ -986,22 +1049,25 @@ class Base:
 
     # The object, as shown in a list of search results
     pxResult = Px('''
-     <tr valign="top" class=":rowCss"
+     <tr valign=":uiSearch.search.rowAlign" class=":rowCss"
          var="id=o.iid; mayView=guard.mayView(o); backHook=str(id);
               cbId='%s_%s' % (mode.checkboxesId, currentNumber)"
          id=":id">
       <td for="column in mode.columns"
-          var2="field=column.field" id=":'field_%s' % field.name|field"
+          var2="field=column.field; isCB=field=='_checkboxes'"
+          id=":'field_%s' % field.name|field"
           width=":column.width" align=":column.align"
-          class=":mode.cbClass if field == '_checkboxes' else ''">
+          class=":mode.cbClass if isCB else ''">
+
        <!-- Special column containing the checkbox -->
-       <input if="field == '_checkboxes'" type="checkbox"
-              name=":mode.checkboxesId" id=":cbId"
+       <input if="isCB" type="checkbox" name=":mode.checkboxesId" id=":cbId"
               var2="checked=mode.cbChecked|True" value=":id"
               onclick="toggleCb(this)" checked=":checked"/>
+
        <!-- Any other column -->
        <x if="not column.special">:field.pxResult</x>
       </td>
+
       <!-- Store data in this tr node allowing to ajax-refresh it -->
       <script>:mode.getAjaxDataRow(o, rowCss=rowCss, \
                currentNumber=currentNumber, cbChecked=cbId)</script>
@@ -1118,14 +1184,14 @@ class Base:
     def getInitiator(self, search=False):
         '''Gets, from the request, information about a potential initiator'''
         # An initiator is an object from which an action is triggered on another
-        # object (being p_self). The most obvious example is target object
+        # object (being p_self). The most obvious example is a target object
         # (p_self) created from a Ref field on a source object (the initiator).
         # The method returns None if there is no initiator.
         req = self.req
-        nav = req.nav
+        nav = req.nav or req.onav
         if not nav or (nav == 'no'): return
         if not search:
-            # Try to get a "classic" initiator, from the "nav" key
+            # Try to get a "classic" initiator, from the "[o]nav" key
             fieldType, info = nav.split('.', 1)
             field = eval(fieldType.capitalize())
             if field.initiator:
@@ -1142,10 +1208,10 @@ class Base:
             if ref:
                 return self.getObject(ref.split(':')[0])
 
-    def getFolder(self, create=False):
+    def getFolder(self, create=False, withRelative=False):
         '''Gets, as a pathlib.Path instance, the folder where binary files
            related to this object are stored on the filesystem.'''
-        return self.H().server.database.getFolder(self, create)
+        return self.H().server.database.getFolder(self, create, withRelative)
 
     def cancel(self, initiator=None, popup=False, isTemp=False):
         '''Redirect the user after an object edition has been canceled'''
@@ -1223,7 +1289,7 @@ class Base:
             # Let the initiator choose the back URL
             url = initiator.getBackUrl(self)
         else:
-            # Go the the page as defined by request varuables "gotoPage" and
+            # Go the the page as defined by request variables "gotoPage" and
             # "gotoLayout".
             url = self.getUrl(sub=req.gotoLayout, page=req.gotoPage,
                               nav=req.nav or 'no')
@@ -1272,16 +1338,28 @@ class Base:
     traverse['remove'] = 'perm:delete'
     def remove(self):
         '''Deletes p_self and return a message to the UI'''
-        # Delete the object from the database
-        self.delete(historize=True)
+        # Delete the object from the database. If a text is returned, an error
+        # occurred and the deletion was aborted.
+        text = self.delete(historize=True)
+        # Prepare the back URL. None = the default = the referer page.
+        back = None
         # Return a message to the UI
-        text = self.translate('action_done')
-        # If we are called from an Ajax request, simply return the message
         handler = self.H()
+        if not text:
+            # The deletion has been successfully performed
+            text = self.translate('action_done')
+            # If we were consulting the object itself, return to the home page.
+            # Else, return to the referer page (it is the default).
+            if handler.getPublishedObject() == self:
+                back = self.tool.computeHomePage()
+        # If we are called from an Ajax request, simply return the message
         if handler.isAjax(): return text
-        # If we were consulting the object itself, return to the home page.
-        # Else, return to the referer page (it is the default).
-        url = self.tool.computeHomePage() \
-              if handler.getPublishedObject() == self else None
-        return self.goto(url=url, message=text)
+        # Go to the "back" URL
+        return self.goto(url=back, message=text)
+
+    traverse['unlock'] = 'perm:write'
+    def unlock(self):
+        '''Unlocks the page whose name is in the request'''
+        Lock.remove(self, page=self.req.page, force=True)
+        self.goto(message=self.translate('action_done'))
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
