@@ -18,6 +18,7 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import copy, types, re
 from DateTime import DateTime
+from persistent.list import PersistentList
 
 from appy.px import Px
 from appy import utils
@@ -26,8 +27,8 @@ from appy.tr import FieldTranslations
 from appy.utils import string as sutils
 from appy.model.utils import Object as O
 from appy.model.fields.phase import Page
-from appy.model.fields.group import Group
 from appy.ui.layout import Layout, Layouts
+from appy.model.fields.group import Group, UiGroup
 
 # In this file, names "list" and "dict" refer to sub-modules. To use Python
 # builtin types, use __builtins__['list'] and __builtins__['dict']
@@ -36,7 +37,7 @@ from appy.ui.layout import Layout, Layouts
 XSS_DETECTED = 'Detected Javascript in user input.'
 XSS_WARNING = 'Your behaviour is considered a security attack. System ' \
               'administrator has been warned.'
-SEARCH_TRIGGER = '%s field %s:%s: executing search %s...'
+SEARCH_TRIGGER = '%s field "%s:%s": executing search "%s"...'
 SEARCH_DONE = 'Fetched %d object(s).'
 UNINDEXED = 'Field "%s": cannot retrieve catalog version of unindexed field.'
 
@@ -175,6 +176,7 @@ class Field:
     validatorTypes = (types.FunctionType, type(re.compile('')))
     labelTypes = ('label', 'descr', 'help')
     viewLayouts = ('view', 'cell')
+    cellLayouts = ('cell', 'query')
 
     # Those attributes can be overridden by subclasses for defining,
     # respectively, names of CSS and Javascript files that are required by this
@@ -189,6 +191,8 @@ class Field:
 
     # Some fields are "outer": they are composed or inner fields
     outer = False
+    # In most cases, outer fields store persistent lists
+    outerType = PersistentList
 
     # Parameters needed to re-trigger a search. Some fields applying on sets of
     # objects may need to re-trigger a search from these request parameters.
@@ -233,12 +237,13 @@ class Field:
       layoutTarget=field">:field.getPx(minimal, \
                                        hostLayout or layout, _ctx_)</x>''')
 
-    def doRender(self, layout, o, name=None, minimal=False):
+    def doRender(self, layout, o, name=None, minimal=False, tagCss=None):
         '''Allows to call pxRender from code, to display the content of this
            field on p_o in some specific context, for example in a Computed
            field.'''
         context = O(layout=layout, field=self, minimal=minimal,
                     name=name or self.name, o=o)
+        if tagCss: context.tagCss = tagCss
         # Get the current PX context, to copy some entries from it. If no such
         # context exists, create one.
         ctx = o.traversal.context or o.traversal.createContext(layout=layout)
@@ -259,9 +264,10 @@ class Field:
        <x if="not selector and guard.mayAct(o)">:ifield.pxObjectActions</x>
       </x>
       <div if="not mayView">
-       <img src=":url('fake')" style="margin-right: 5px"/>
+       <img src=":url('password.svg')" class="iconS"/>
        <x>:_('unauthorized')</x></div>
      </x>
+
      <!-- Any other field -->
      <x if="mayView and (field.name != 'title') and \
             field.isShowable(o, 'result')"
@@ -282,16 +288,31 @@ class Field:
        <x>::ui.Title.get(o, mode=mode.titleMode, nav=navInfo, target=target, \
           page=pageName, popup=popup, selectJs=selectJs, highlight=True, \
           backHook=backHook)</x>
-       <span if="hasattr(o,'getSubTitle')" class=":class_.getCssFor('subTitle')"
+       <span if="hasattr(o,'getSubTitle')"
+             class=":class_.getCssFor(o,'subTitle')"
              style=":mode.showSubTitles and 'display:inline' or 'display:none'"
              name="subTitle">::uiSearch.highlight(o.getSubTitle())</span>
 
        <!-- Actions -->
        <div if="not popup and uiSearch.showActions and guard.mayAct(o)"
-            class="objectActions" style=":'display:%s'% uiSearch.actionsDisplay"
+            class=":ui.getActionsCss(uiSearch.actionsDisplay)"
             var2="layout='buttons';
                   editable=guard.mayEdit(o);
                   locked=o.Lock.isSet(o, user, 'main')">
+
+        <!-- Fields on layout "buttons" -->
+        <x if="not popup and (uiSearch.showActions == 'all')"
+           var2="fields=o.getFields('buttons', 'main'); layout='cell'">
+         <!-- Call cell and not pxRender to avoid having a table -->
+         <x for="field in fields"
+            var2="name=field.name;
+                  value=field.getValueIf(o, name, layout)">:field.cell</x>
+        </x>
+
+        <!-- Transitions -->
+        <x if="class_.showTransitions(o, 'result')"
+           var2="workflow=o.getWorkflow()">:workflow.pxTransitions</x>
+
         <!-- Edit -->
         <x if="editable">
          <a if="not locked"
@@ -300,29 +321,19 @@ class Field:
             onclick=":target.getOnClick(backHook or str(o.iid))"
             href=":o.getUrl(sub='edit', page=o.getDefaultPage('edit'), \
                             nav=navInfo, popup=linkInPopup)">
-          <img src=":url('edit')" title=":_('object_edit')"/>
+          <img src=":url('edit.svg')" class="iconS" title=":_('object_edit')"/>
          </a>
-         <x if="locked" var2="lockStyle=''; page='main'">::o.Lock.px</x>
+         <x if="locked" var2="lockStyle='iconS'; page='main'">::o.Lock.px</x>
         </x>
+
         <!-- Delete -->
-        <img if="not locked and guard.mayDelete(o)" class="clickable"
-             src=":url('delete')" title=":_('object_delete')"
+        <img if="not locked and guard.mayDelete(o)" class="clickable iconS"
+             src=":url('deleteS.svg')" title=":_('object_delete')"
              onClick=":'onDeleteObject(%s)' % q(o.url)"/>
-        <!-- Fields (actions) defined with layout "buttons" -->
-        <x if="not popup and (uiSearch.showActions == 'all')"
-           var2="fields=o.getFields('buttons', 'main');
-                 layout='cell'">
-         <!-- Call cell and not pxRender to avoid having a table -->
-         <x for="field in fields"
-            var2="name=field.name; smallButtons=True">:field.cell</x>
-        </x>
-        <!-- Workflow transitions -->
-        <x if="class_.showTransitions(o, 'result')"
-           var2="workflow=o.getWorkflow()">:workflow.pxTransitions</x>
        </div>
       </x>
       <x if="not mayView">
-       <img src=":url('fake')" style="margin-right: 5px"/>
+       <img src=":url('password.svg')" class="iconS"/>
        <x>:_('unauthorized')</x>
       </x>
      </x>
@@ -341,17 +352,24 @@ class Field:
     pxDescription = Px('''<span if="field.hasDescr"
      class="discreet">::_('descr', field=field)</span>''')
 
-    # Displays a field help
-    pxHelp = Px('''<abbr title=":_('help', field=field)"><img
-     src=":url('help')"/></abbr>''')
+    # Displays a field help (UiGroup has exaclty the same PX)
+    pxHelp = UiGroup.pxHelp
 
     # Displays validation-error-related info about a field
-    pxValidation = Px('''<x><abbr if="error" title=":error"><img
-     src=":url('warning')"/></abbr><img if="not error"
-     src=":url('warning_no.gif')"/></x>''')
+    def validationPx(c):
+        '''Python implementation of validation PX'''
+        url = c.url('warning.svg')
+        if c.error:
+            r = '<abbr title="%s"><img src="%s" class="iconS"/></abbr>' % \
+                (c.error, url)
+        else:
+            r = '<img src="%s" class="iconS" style="visibility:hidden"/>' % url
+        return r
+    
+    pxValidation = Px(validationPx)
 
     # Displays the fact that a field is required
-    pxRequired = Px('''<img src=":url('required.gif')"/>''')
+    pxRequired = Px(lambda c: '<span class="required">*</span>')
 
     # Button for showing changes to the field
     pxChanges = Px('''
@@ -382,7 +400,7 @@ class Field:
              value=":mode.filters.get(name, '')"
              onkeydown=":'if (event.keyCode==13) document.getElementById ' \
                          '(%s).click()' % q(filterIdIcon)"/>
-      <img id=":filterIdIcon" class="clickable" src=":url('funnel')"
+      <img id=":filterIdIcon" class="clickable iconS" src=":url('funnel.svg')"
            onclick=":'askBunchFiltered(%s,%s)' % (q(mode.hook), q(name))"/>
      </x>''')
 
@@ -751,7 +769,7 @@ class Field:
             r = getattr(self, layout)
         else:
             # Get the Layout instance related to p_layout and render its PX
-            if layout == 'cell':
+            if layout in Field.cellLayouts:
                 table = self.Layouts.cell
             else:
                 table = self.layouts[layout]
@@ -823,13 +841,15 @@ class Field:
                 r = o.req[outerName] if fromRequest else o.values.get(outerName)
                 # Access the inner value
                 if r:
-                    if rowId.isdigit():
+                    if o.getField(outerName).outerType == PersistentList:
+                        # The row ID is the index if an element from a list
                         rowId = int(rowId)
                         if rowId < len(r):
                             r = getattr(r[rowId], name, None)
                         else:
                             r = None
                     else:
+                        # The row ID is a key from a dict
                         r = r.get(rowId, None)
                         if r: r = r.get(name, None)
             # Return an empty string if fromRequest is True
@@ -1028,8 +1048,9 @@ class Field:
         onClick = 'onclick="askField(\'%s\',\'%s\',\'edit:%s\')"' % \
                   (hook, o.url, layout)
         if inlineEdit == 'icon':
-            r = '<img src="%s" title="%s" class="inlineIcon" %s/>%s' % \
-                (o.buildUrl('edit'), o.translate('object_edit'), onClick, value)
+            r = '<img src="%s" title="%s" class="inlineIcon iconS" %s/>%s' % \
+                (o.buildUrl('edit.svg'), o.translate('object_edit'), onClick,
+                 value)
         else:
             r = '<span class="editable" %s>%s</span>' % (onClick, value)
         return r
@@ -1217,26 +1238,37 @@ class Field:
     def getSearchParams(self, req, layout):
         '''Encodes in a string all the params in the request that are required
            for re-triggering a search.'''
+        # If key "search" is present in the request, but empty, it represents
+        # the special search named "allSearch".
+        if ('search' in req) and not req.search:
+            req.search = 'allSearch'
         if not req.search or (layout == 'cell'): return ''
         return ':'.join([(req[key] or '') for key in Field.searchParams])
 
     def getSearchResults(self, tool, searchParams):
         '''Execute the search whose parameters (p_searchParams) are returned by
            m_getSearchParams and return its results. Log the operation.'''
-        # Get search parameters as local variables
-        cmd = ', '.join(Field.searchParams)
-        cmd += " = searchParams.split(':',%d)" % (len(Field.searchParams) - 1)
-        exec(cmd)
+        # Get search parameters as an object
+        params = O()
+        i = 0
+        for part in searchParams.split(':', len(Field.searchParams) - 1):
+            setattr(params, Field.searchParams[i], part)
+            i += 1
         # Retrieve a Ref object when relevant
-        refObject, refField = tool.getRefInfo()
+        Search = tool.Search
+        refObject, refField = Search.getRefInfo(tool)
+        # Get the Search instance
+        class_ = tool.model.classes.get(params.className)
+        search = Search.get(params.search, tool, class_, None, ui=False)
         # Executing the search may take some time and potentially slow down
         # the system: log this action.
-        tool.log(SEARCH_TRIGGER % (self.type, className, self.name, search))
+        tool.log(SEARCH_TRIGGER % (self.type, class_.name, self.name,
+                                   search.name))
         # (Re-)execute the search, but without restricting the number of results
-        r = tool.search(className, searchName=search, sortBy=sortKey,
-                        sortOrder=sortOrder, maxResults='NO_LIMIT',
-                        filters=sutils.getDictFrom(filters),
-                        refObject=refObject, refField=refField)
+        r = search.run(tool.H(), batch=False,
+          sortBy=params.sortKey, sortOrder=params.sortOrder,
+          filters=sutils.getDictFrom(params.filters),
+          refObject=refObject, refField=refField)
         tool.log(SEARCH_DONE % len(r))
         return r
 
@@ -1395,7 +1427,6 @@ class Field:
                 # Raise the initial error
                 raise te
         except Exception as e:
-            o.log(utils.Traceback.get(), type='error')
             raise e
 
     def getAttribute(self, o, name, cache=True):
