@@ -21,7 +21,7 @@ from appy.px import Px
 from appy import utils
 from appy.ui import LinkTarget
 from appy.ui.layout import Layouts
-from appy.model.fields import Field, Initiator
+from appy.model.fields import Field, Initiator, Show
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CONFIRM_ERROR = 'When using options, a popup will already be shown, with the ' \
@@ -49,6 +49,10 @@ class ActionInitiator(Initiator):
         # If we are back from a popup, must we force the back URL ?
         if field.result == 'redirect':
             self.backFromPopupUrl = msg
+            r = None
+        else:
+            r = msg
+        return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Action(Field):
@@ -76,7 +80,7 @@ class Action(Field):
             return class_.b
 
     # PX for viewing the Action button
-    view = cell = Px('''
+    view = cell = query = Px('''
      <form var="isFake=field.isFake(o, _);
                 formId='%d_%s_form' % (o.iid, name);
                 label=_(field.labelId);
@@ -84,11 +88,13 @@ class Action(Field):
                 className=className|o.class_.name;
                 inputTitle=field.getInputTitle(o, label);
                 inputLabel=field.getInputLabel(label, layout);
-                smallButtons=smallButtons|False;
-                css=ui.Button.getCss(label, smallButtons, field.render)"
-           id=":formId" action=":field.getFormAction(o, tool)" method="post"
-           target=":field.options and 'appyIFrame' or '_self'"
-           style="display:inline">
+                onCell=layout in field.cellLayouts;
+                picto=picto|'pictoB';
+                asPicto=not onCell and (picto != 'pictoB');
+                css=ui.Button.getCss(label, onCell, field.render)"
+        id=":formId" action=":field.getFormAction(o, tool, layout)"
+        target=":field.options and 'appyIFrame' or '_self'"
+        method="post" class="inline">
 
       <!-- Form fields for direct action execution -->
       <x if="not field.options and not isFake">
@@ -117,26 +123,40 @@ class Action(Field):
                 (formId, className)"/>
       </x>
 
-      <!-- The button for executing the action -->
-      <input if="not isFake" type="button" class=":css" title=":inputTitle"
-             value=":inputLabel" style=":url(field.icon, bg=True)"
-             onclick=":field.getOnClick(o, name, req, layout, q, multi)"/>
+      <!-- As picto -->
+      <a if="asPicto" class=":'help fake' if isFake else 'clickable'"
+         title=":isFake if isFake else ''"
+         onclick=":field.getOnClick(o, name, req, layout, q, multi) \
+                   if not isFake else ''">
+       <img src=":url(field.icon, base=field.iconBase)" class=":picto"/><br/>
+       <x>::inputTitle</x>
+      </a>
 
-      <!-- ... or the fake button -->
-      <input if="isFake" type="button" class=":'fake %s' % css" title=":isFake"
-             value=":inputLabel" style=":url('fake', bg=True)"/>
+      <!-- As button -->
+      <x if="not asPicto">
+       <!-- The button for executing the action -->
+       <input if="not isFake" type="button" class=":css" title=":inputTitle"
+              value=":inputLabel"
+              style=":url(field.sicon, base=field.siconBase, bg='18px 18px')"
+              onclick=":field.getOnClick(o, name, req, layout, q, multi)"/>
+
+       <!-- ... or the fake button -->
+       <input if="isFake" type="button" class=":'%s fake' % css"
+              title=":isFake" value=":inputLabel"
+              style=":url(field.sicon, base=field.siconBase, bg='18px 18px')"/>
+      </x>
      </form>''')
 
     # It is not possible to edit an action, not to search it
     edit = search = ''
 
     def __init__(self, validator=None, multiplicity=(1,1), default=None,
-      defaultOnEdit=None, show=('view', 'result'), page='main', group=None,
-      layouts=None, move=0, readPermission='read', writePermission='write',
-      width=None, height=None, maxChars=None, colspan=1, action=None,
-      result='computation', downloadDisposition='attachment', confirm=False,
-      master=None, masterValue=None, focus=False, historized=False,
-      mapping=None, generateLabel=None, label=None, icon=None, view=None,
+      defaultOnEdit=None, show=Show.TR, page='main', group=None, layouts=None,
+      move=0, readPermission='read', writePermission='write', width=None,
+      height=None, maxChars=None, colspan=1, action=None, result='computation',
+      downloadDisposition='attachment', confirm=False, master=None,
+      masterValue=None, focus=False, historized=False, mapping=None,
+      generateLabel=None, label=None, icon=None, sicon=None, view=None,
       cell=None, edit=None, xml=None, translations=None, render='button',
       options=None, fake=False):
         # Attribute "action" must hold a method or a list/tuple of methods.
@@ -196,8 +216,15 @@ class Action(Field):
         # to enter a comment; this comment will be available to self.action's
         # method(s), via a parameter named "comment".
         self.confirm = confirm
-        # If no p_icon is specified, "action.png" will be used
-        self.icon = icon or 'action'
+        # If you want to specify an icon being in folder <yourApp>/static,
+        # specify "<yourApp>/<yourImage>". Else, default icon "appy/action.svg"
+        # will be used.
+        self.icon, self.iconBase = utils.iconParts(icon or 'action.svg')
+        # You may specify, in attribute "sicon", an alternate icon suitable when
+        # rendered as a small icon.
+        sicon = sicon or icon or 'action.svg'
+        self.sicon, self.siconBase = utils.iconParts(sicon)
+        # Call the base constructor
         Field.__init__(self, None, (0,1), default, defaultOnEdit, show, page,
           group, layouts, move, False, True, None, None, False, None,
           readPermission, writePermission, width, height, None, colspan, master,
@@ -292,7 +319,8 @@ class Action(Field):
         # corresponding button at all in the UI, you prefer to show a "fake"
         # button with an explanation about why the action can't be currently
         # performed, place a method in attribute "fake", accepting no arg and
-        # returning True when such fake button must be shown.
+        # returning the explanation as a string, or True if you simply want a
+        # standard message to appear.
         self.fake = fake
         # Ensure validity of parameter values
         self.checkParameters()
@@ -305,11 +333,10 @@ class Action(Field):
         if self.options:
             if self.confirm: raise Exception(CONFIRM_ERROR)
 
-
     def renderLabel(self, layoutType):
         return # Label is rendered directly within the button
 
-    def getFormAction(self, o, tool):
+    def getFormAction(self, o, tool, layout):
         '''Get the value of the "action" parameter to the "form" tag
            representing the action.'''
         if self.options:
@@ -317,8 +344,10 @@ class Action(Field):
             # retrieve action's options.
             return '%s/new' % tool.url
         else:
-            # Submitting the form will really trigger the action
-            return '%s/%s/perform' % (o.url, self.name)
+            # Submitting the form will really trigger the action. Add the name
+            # of the class in the path if p_self is a multi-action.
+            part = '@%s/' % self.container.name if layout == 'query' else ''
+            return '%s/%s%s/perform' % (o.url, part, self.name)
     
     def getOnClick(self, o, name, req, layout, q, multi):
         '''Gets the JS code to execute when the action button is clicked'''
@@ -400,7 +429,7 @@ class Action(Field):
             historized = self.getAttribute(o, 'historized')
             if historized:
                 o.history.add('Action', action=self.name,
-                              comments=args.get('comment'))
+                              comment=args.get('comment'))
         return r
 
     def getValue(self, o, name=None, layout=None):
@@ -419,7 +448,7 @@ class Action(Field):
            the message to show with the fake button.'''
         if not self.fake: return
         msg = self.getAttribute(o, 'fake')
-        if msg and not isinstance(msg, basestring):
+        if msg and not isinstance(msg, str):
             msg = _('action_unexecutable')
         return msg
 
@@ -472,11 +501,11 @@ class Action(Field):
             if o.req.popup: resp.setCookie('closePopup', 'yes')
             header = resp.setHeader
             header('Content-Type', utils.getMimeType(msg.name))
-            header('Content-Length', len(r))
             header('Content-Disposition', '%s;filename="%s"' % \
                    (self.downloadDisposition, os.path.basename(msg.name)))
-            header('Accept-Ranges', 'none')
             msg.close()
+        else:
+            r = msg
         # Stop here if p_minimal is True
         if minimal: return r
         if (result == 'computation') or not success:
