@@ -89,7 +89,7 @@ class Date(Field):
       <!-- The icon for displaying the calendar popup -->
       <x if="field.calendar">
        <input type="hidden" id=":name" name=":name"/>
-       <img id=":'%s_img' % name" src=":url('calendar.gif')"/>
+       <img id=":'%s_img' % name" src=":url('calendar.svg')" class="iconS"/>
        <script>::field.getJsInit(name, years)</script>
       </x>
 
@@ -125,7 +125,8 @@ class Date(Field):
          <!-- The icon for displaying the calendar popup -->
          <x if="field.calendar">
           <input type="hidden" id=":fromName" name=":fromName"/>
-          <img id=":'%s_img' % fromName" src=":url('calendar.gif')"/>
+          <img id=":'%s_img' % fromName" src=":url('calendar.svg')"
+               class="iconS"/>
           <script>::field.getJsInit(fromName, years)</script>
          </x>
          <!-- Hour and minutes when relevant -->
@@ -161,7 +162,8 @@ class Date(Field):
          <!-- The icon for displaying the calendar popup -->
          <x if="field.calendar">
           <input type="hidden" id=":toName" name=":toName"/>
-          <img id=":'%s_img' % toName" src=":url('calendar.gif')"/>
+          <img id=":'%s_img' % toName" src=":url('calendar.svg')"
+               class="iconS"/>
           <script>::field.getJsInit(toName, years)</script>
          </x>
          <!-- Hour and minutes when relevant -->
@@ -288,8 +290,8 @@ class Date(Field):
 
     def getRequestSuffix(self): return '_year'
 
-    def getStorableValue(self, obj, value, complete=False):
-        if not self.isEmptyValue(obj, value):
+    def getStorableValue(self, o, value, complete=False):
+        if not self.isEmptyValue(o, value):
             return DateTime(value)
 
     def getDateFromSearchValue(self, year, month, day, hour, setMin):
