@@ -16,6 +16,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from persistent.mapping import PersistentMapping
+
 from appy.px import Px
 from appy.ui.layout import Layout
 from appy.model.utils import Object
@@ -48,6 +50,9 @@ class Dict(List):
        field is build on top of the List field.'''
     SepRow = SepRow
 
+    # As an outer type, a Dict stores persistent mappings
+    outerType = PersistentMapping
+
     # PX for rendering a single row
     pxRow = Px('''
      <!-- Render a separatation row -->
@@ -92,8 +97,8 @@ class Dict(List):
         # database, while "title" will get the text that will be shown in the ui
         # while encoding/viewing this entry.
 
-        # WARNING: a key must be a string, cannot contain digits only and
-        # cannot contain char "*". A key is typically an object ID.
+        # WARNING: a key must be a string and cannot contain char "*". A key is
+        # typically an object ID, CONVERTED to a string.
 
         # For a nice rendering of your dict, some of the tuples returned by
         # method "keys" can be "separator rows". The tuple representing such a
@@ -108,7 +113,7 @@ class Dict(List):
 
     def getStorableValue(self, o, value, complete=False):
         '''Gets p_value in a form that can be stored in the database'''
-        r = {}
+        r = PersistentMapping()
         for k, v in value.items():
             r[k] = self.getStorableRowValue(o, v)
         return r
@@ -118,7 +123,7 @@ class Dict(List):
         if self.name not in o.values: return
         val = o.values[self.name]
         if key not in val: return
-        del val[key]
+        del(val[key])
         o.values[self.name] = val
 
     def store(self, o, value, overwrite=False):
@@ -137,7 +142,6 @@ class Dict(List):
                     dbValue[key] = data
                 else:
                     dbValue[key].update(data)
-            o.values[self.name] = dbValue
 
     def subValidate(self, o, value, errors):
         '''Validates inner fields'''
