@@ -30,13 +30,15 @@ from appy.model.utils import Object
 from appy.server.static import Static
 from appy.utils import path as putils
 from appy.utils import string as sutils
+from appy.xml.unmarshaller import UnmarshalledFile
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-WRONG_FILE_TUPLE = 'This is not the way to set a file. You can specify a ' \
-  '2-tuple (fileName, fileContent) or a 3-tuple (fileName, fileContent, ' \
-  'mimeType).'
-CONVERSION_ERROR = 'An error occurred. %s'
-MISSING_PATH = 'Missing absolute disk path for %s.'
+BAD_FTUPLE = 'This is not the way to set a file. You can specify a 2-tuple ' \
+             '(fileName, fileContent) or a 3-tuple (fileName, fileContent, ' \
+             'mimeType).'
+CONV_ERR   = 'An error occurred. %s'
+PATH_KO    = 'Missing absolute disk path for %s.'
+RESIZED    = '%s resized to %spx.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class FileInfo:
@@ -79,15 +81,20 @@ class FileInfo:
         self.size = 0 # Its size, in bytes
         self.mimeType = None # Its MIME type
         self.modified = None # The last modification date for this file
-        # Complete metadata if p_inDb is False
+        # Complete metadata if p_inDb is False. p_inDb is not stored as is:
+        # checking if self.fsName is the empty string is equivalent.
         if not inDb:
             self.fsName = '' # Already included in self.fsPath
-            # We will not store p_inDb. Checking if self.fsName is the empty
-            # string is equivalent.
-            fileInfo = os.stat(self.fsPath)
-            self.size = fileInfo.st_size
-            self.mimeType = putils.guessMimeType(self.fsPath)
-            self.modified = DateTime(fileInfo.st_mtime)
+            self.setAttributes(self.fsPath)
+
+    def setAttributes(self, path):
+        '''Compute file attributes (size, MIME type and last modification date)
+           for a file whose absolute p_path is passed and store these attributes
+           on p_self.'''
+        info = os.stat(path)
+        self.size = info.st_size
+        self.mimeType = putils.guessMimeType(path)
+        self.modified = DateTime(info.st_mtime)
 
     def inDb(self):
         '''Does this FileInfo represent a file from the DB-controlled
@@ -162,7 +169,7 @@ class FileInfo:
             path = self.fsPath
         elif not path:
             # An in-database file for which we do not have the full path
-            raise Exception(MISSING_PATH % self.fsName)
+            raise Exception(PATH_KO % self.fsName)
         # Produce the file via the Static class
         Static.write(handler, path, None, response=handler.resp, fileInfo=self,
                      disposition=disposition, enableCache=cache)
@@ -173,7 +180,8 @@ class FileInfo:
            - another ("not-in-DB") FileInfo instance;
            - a tuple (fileName, content, mimeType): see method File.store.'''
         # Determine the file's MIME type
-        if isinstance(value, Object):
+        isO = isinstance(value, Object) or isinstance(value, UnmarshalledFile)
+        if isO:
             mimeType = value.type
             niceName = value.name
         elif isinstance(value, FileInfo):
@@ -191,7 +199,7 @@ class FileInfo:
         # Write the file on disk (and compute/get its size in bytes)
         path = str(folder / self.fsName)
         f = open(path, 'wb')
-        if isinstance(value, Object):
+        if isO:
             content = value.value
             self.size = len(content)
             f.write(content)
@@ -221,7 +229,7 @@ class FileInfo:
         # Set mimeType
         self.mimeType = putils.guessMimeType(filePath)
         # Copy the file
-        fsName = putils.osPathJoin(dbFolder, self.fsPath, self.fsName)
+        fsName = dbFolder / self.fsName
         shutil.copyfile(filePath, fsName)
         self.modified = DateTime()
         self.size = os.stat(fsName).st_size
@@ -250,15 +258,27 @@ class FileInfo:
             os.remove(filePath)
             # Get the name of the converted file
             baseName, ext = os.path.splitext(filePath)
-            if (ext == '.%s' % format):
+            if ext == ('.%s' % format):
                 filePath = '%s.res.%s' % (baseName, format)
             else:
                 filePath = '%s.%s' % (baseName, format)
             if not os.path.exists(filePath):
-                o.log(CONVERSION_ERROR % err, type='error')
+                o.log(CONV_ERR % err, type='error')
                 return
         return filePath
 
+    def resize(self, folder, width, o):
+        '''Resize this image if it is a resizable image'''
+        if self.mimeType not in File.resizableImages: return
+        # Get the absolute path to the file on disk
+        path = folder / self.fsName
+        # Get the width, in pixels, that will be used to resize it
+        width = sutils.Normalize.digit(str(width))
+        utils.convert(path, '-resize %sx%s>' % (width, width))
+        o.log(RESIZED % (path, width))
+        # (re)compute p_self's attributes, after resizing
+        self.setAttributes(str(path))
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class File(Field):
     '''Field allowing to upload a binary file'''
@@ -266,6 +286,11 @@ class File(Field):
     # Some elements will be traversable
     traverse = Field.traverse.copy()
 
+    # MIME types for images
+    imageMimeTypes = ('image/png', 'image/jpeg', 'image/gif')
+    # MIME types for resizable images
+    resizableImages = imageMimeTypes
+
     class Layouts(Layouts):
         '''File-specific layouts'''
         b = Layouts(edit='lrv-f', view='l-f')
@@ -314,12 +339,13 @@ class File(Field):
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
       defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
       move=0, readPermission='read', writePermission='write', width=None,
-      height=None, maxChars=None, colspan=1, master=None, masterValue=None,
-      focus=False, historized=False, mapping=None, generateLabel=None,
-      label=None, isImage=False, downloadAction=None, sdefault='', scolspan=1,
-      swidth=None, sheight=None, view=None, cell=None, edit=None, xml=None,
-      translations=None, render=None, icon='paperclip',
-      disposition='attachment', nameStorer=None, cache=False):
+      height=None, inputWidth=None, maxChars=None, colspan=1, master=None,
+      masterValue=None, focus=False, historized=False, mapping=None,
+      generateLabel=None, label=None, isImage=False, downloadAction=None,
+      sdefault='', scolspan=1, swidth=None, sheight=None, view=None, cell=None,
+      edit=None, xml=None, translations=None, render=None, icon='paperclip',
+      disposition='attachment', nameStorer=None, cache=False, resize=False,
+      thumbnail=None):
         # This boolean is True if the file is an image
         self.isImage = isImage
         # "downloadAction" can be a method called every time the file is
@@ -331,10 +357,10 @@ class File(Field):
         self.render = render
         # Icon to use when this file is rendered as an icon
         self.icon = icon
-        # In "nameStorer", you can specify another field that will store the
-        # file name. This field must be a String belonging to the same class as
-        # p_self. As soon as, in the UI, a file is selected in p_self's widget,
-        # its name will be copied into the nameStorer field, without the
+        # In "nameStorer", you can specify the name of another field that will
+        # store the file name. This field must be a String belonging to the same
+        # class as p_self. As soon as, in the UI, a file is selected in p_self's
+        # widget, its name will be copied into the nameStorer field, without the
         # extension, only if this latter is not empty.
         self.nameStorer = nameStorer
         # When downloading the file, the browser may try to open it within the
@@ -348,15 +374,26 @@ class File(Field):
         # here, set the foolowing attribute to True. Attribute p_cache can also
         # store a method that must return a boolean value.
         self.cache = cache
-        # Call the base constructor. Regarding attribute "width":
-        # ----------------------------------------------------------------------
-        #  if used for... | it represents...
-        # ----------------------------------------------------------------------
-        #    a file       | the width of the input field allowing to upload the
-        #                 | file;
-        #   an image      | the maximum width for rendering the image in the
-        #                 | web UI. It will not resize the uploaded image.
-        # ----------------------------------------------------------------------
+        # Attribute "width" is used to specify the width of the image or
+        # document. The width of the input field allowing to upload the file can
+        # be defined in attribute "inputWidth".
+        self.inputWidth = inputWidth
+        # If attribute "resize" is False (the default), the image will be
+        # previewed in the UI with dimensions as defined in attributes "width"
+        # and "height", but without being effectively resized. If "resize" is
+        # True, when uploading the file, it will be resized to self.width,
+        # keeping the width / height ratio.
+        self.resize = resize
+        # If you place here the name of another File field, it will be used to
+        # store a thumbnail of the image stored in the current field. The
+        # thumbnail's dimension is defined, as usual, via attribute "width" of
+        # the File field storing it. Everytime a document is uploaded in p_self,
+        # if a thumbnail is defined in its "thumbnail" attribute, the thumbnail
+        # field will be updated and will store the same image as the one stored
+        # in p_self, resized accordingly (provided you have defined it as
+        # "resizable").
+        self.thumbnail = thumbnail
+        # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
           show, page, group, layouts, move, False, True, None, None, False,
           None, readPermission, writePermission, width, height, None, colspan,
@@ -391,14 +428,12 @@ class File(Field):
         '''Gets the JS code for updaing the name storer when defined'''
         storer = self.nameStorer
         if not storer: return ''
-        return 'updateFileNameStorer(this, "%s")' % storer.name
+        return 'updateFileNameStorer(this, "%s")' % storer
 
     def getStyle(self):
         '''Get the content of the "style" attribute of the "input" tag on the
            "edit" layout for this field.'''
-        if self.width:
-            return 'width: %s' % self.width
-        return ''
+        return 'width: %s' % self.inputWidth if self.inputWidth else ''
 
     def isRenderable(self, layout):
         '''A file with 'icon' rendering is potentially renderable everywhere'''
@@ -445,8 +480,6 @@ class File(Field):
            another file.'''
         return True
 
-    imageMimeTypes = ('image/png', 'image/jpeg', 'image/gif')
-
     def validateValue(self, o, value):
         '''Ensure p_value is valid'''
         if not value and (o.req['%s_action' % self.name] == 'replace'):
@@ -466,7 +499,10 @@ class File(Field):
         #    | originating from a HTTP POST request, having attributes:
         #    |  - "name",    containing the name of an uploaded file;
         #    |  - "type",    containing the MIME type of this file;
-        #    |  - "content", containing its biary content;
+        #    |  - "value",   containing its binary content;
+        #    | or an instance of appy.xml.unmarshalled.UnmarshalledFile, having
+        #    | the same attributes, representing a file unmarshalled from XML
+        #    | content.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  b | a string. In this case, the string represents the path of a file
         #    | on disk;
@@ -486,13 +522,14 @@ class File(Field):
         if value:
             # There is a new value to store. Get the folder on disk where to
             # store the new file.
-            folder = o.getFolder(create=True)
+            folder, relative = o.getFolder(create=True, withRelative=True)
             # Remove the previous file if it existed
             info = o.values.get(self.name)
             if info: info.removeFile(folder)
             # Store the new file. As a preamble, create a FileInfo instance.
-            info = FileInfo(str(folder))
-            if isinstance(value, Object) or isinstance(value, FileInfo):
+            info = FileInfo(relative)
+            if isinstance(value, Object) or isinstance(value, FileInfo) or \
+               isinstance(value, UnmarshalledFile):
                 # Cases (a) and (e)
                 info.writeFile(self.name, value, folder)
             elif isinstance(value, str):
@@ -506,11 +543,16 @@ class File(Field):
                 elif len(value) == 3:
                     name, content, type = value
                 if not name:
-                    raise Exception(WRONG_FILE_TUPLE)
+                    raise Exception(BAD_FTUPLE)
                 type = type or putils.guessMimeType(name)
                 info.writeFile(self.name, (name, content, type), folder)
             # Store the FileInfo instance in the database
             o.values[self.name] = info
+            # Resize the image when relevant
+            if self.resize and self.width: info.resize(folder, self.width, o)
+            # Update the thumbnail if defined
+            fileName = str(folder / info.fsName)
+            if self.thumbnail: o.getField(self.thumbnail).store(o, fileName)
         else:
             # Store value "None", excepted if we find, in the request, the
             # desire to keep the file unchanged.
