@@ -214,8 +214,8 @@ class UiGroup:
        the currently logged user can see.'''
 
     # Render a help icon for a group
-    pxHelp = Px('''<abbr title="obj.translate('help', field=field)"><img
-     src=":url('help')"/></abbr>''')
+    pxHelp = Px(lambda c: '<abbr title="%s"><img class="iconS" src="%s"/>' \
+                    '</abbr>' % (c._('help', field=c.field), c.url('help.svg')))
 
     # Render the group title, description and help
     pxHeader = Px('''
@@ -269,7 +269,7 @@ class UiGroup:
     pxTabs = Px('''
      <table width=":field.wide" class=":groupCss" id=":tagId" name=":tagName">
       <!-- First row: the tabs -->
-      <tr valign="middle"><td class="tabSep">
+      <tr valign="middle"><td class="tabSep tabBottom">
        <table class="tabs" id=":'tabs_%s' % field.name">
         <tr valign="middle">
          <x for="sub in field.elements"
