@@ -105,11 +105,14 @@ class List(Field):
                 fieldName='%s*%d' % (field.name, rowId)">:field.pxRender</td>
       <!-- Icons -->
       <td if="isEdit" align=":dright">
-       <img class="clickable" src=":url('delete')" title=":_('object_delete')"
+       <img class="clickable iconS" src=":url('deleteS.svg')"
+            title=":_('object_delete')"
             onclick=":field.jsDeleteConfirm(q, tableId)"/>
-       <img class="clickable" src=":url('arrowUp')" title=":_('move_up')"
+       <img class="clickable iconS" src=":url('arrow.svg')"
+            style="transform: rotate(180deg)" title=":_('move_up')"
             onclick=":'moveRow(%s,%s,this)' % (q('up'), q(tableId))"/>
-       <img class="clickable" src=":url('arrowDown')" title=":_('move_down')"
+       <img class="clickable iconS" src=":url('arrow.svg')"
+            title=":_('move_down')"
             onclick=":'moveRow(%s,%s,this)' % (q('down'), q(tableId))"/>
        <input class="clickable" type="image" tabindex="0"
               src=":url('addBelow')" title=":_('object_add_below')"
