@@ -18,39 +18,85 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import collections
 from appy.px import Px
+from appy.utils import iconParts
 from appy.model.utils import Object
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NO_CURRENT_PAGE = 'There is no current page to show. The default page for ' \
-  'this object is probably invisible. Please define method getDefaultViewPage '\
-  'and / or getDefaultEditPage To overcome the problem.'
+NO_CUR_PAGE = 'There is no current page to show. The default page for this ' \
+              'object is probably invisible. Please define method ' \
+              'getDefaultViewPage and / or getDefaultEditPage To overcome ' \
+              'the problem.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Page:
     '''Used for describing a page, its related phase, show condition, etc.'''
-    subElements = ('save', 'cancel', 'previous', 'next', 'edit')
+    subElements = ('save', 'cancel', 'edit')
 
     def __init__(self, name, phase='main', show=True, showSave=True,
-                 showCancel=True, showPrevious=False, showNext=False,
-                 showEdit=True, label=None):
+                 showCancel=True, showEdit=True, label=None, icon=None,
+                 sticky='edit'):
+        # A short, camel-cased unique page, that must be unique across all
+        # phases within the same class
         self.name = name
+        # The name of the phase containing this page
         self.phase = phase
+        # When must the page be shown ? The following attribute can hold values
+        # True, False, "view" or "edit", or a method producing such a value,
+        # accepting the current object, as unique arg.
         self.show = show
-        # When editing the page, must I show the "save" button?
+        # When editing the page, must the "save" button be shown ?
         self.showSave = showSave
-        # When editing the page, must I show the "cancel" button?
+        # When editing the page, must the "cancel" button be shown ?
         self.showCancel = showCancel
-        # When editing the page, and when a previous page exists, must I show
-        # the "previous" button?
-        self.showPrevious = showPrevious
-        # When editing the page, and when a next page exists, must I show the
-        # "next" button?
-        self.showNext = showNext
-        # When viewing the page, must I show the "edit" button?
+        # When viewing the page, must the "edit" icon be shown ?
         self.showEdit = showEdit
         # The i18n label may be forced here instead of being deduced from the
         # page name.
         self.label = label
+        # The icon to display. If you want to use:
+        # * one of the standard Appy icons, residing in appy/ui/static, use
+        #   string "appy/<icon_name>", for example "appy/help.svg";
+        # * an icon being in your app's "static" folder, use string
+        #   "<app_name>/<icon_name>";
+        # * an icon that is chosen dynamically, set a method from the class
+        #   where this page is defined. It will be called without arg and must
+        #   return the icon URL, or None if the icon is not available (yet).
+        #   Here are some tips.
+        #   1) If you want to build the URL to a standard Appy icon, use, ie:
+        #      o.buildUrl('help.svg')
+        #      ("o" being any instance of any Appy class)
+        #   2) If you want to build the URL to an icon being in your app's
+        #      "static" folder, use, ie:
+        #      o.buildUrl('myIcon.png', base='MyApp')
+        #   3) If you want to build the URL to an icon that was uploaded in a
+        #      File field, use:
+        #      '%s/<field_name>/download' % o.url
+        self.setIcon(icon or 'page.svg')
+        # Stickyness refers to the complete block of controls that is shown for
+        # the current object, when this page is shown. Attribute "sticky" may
+        # hold True, False, "edit" or "view", and determines if the block must
+        # be sticky when this page is shown on "edit", "view" or both layouts.
+        # When sticky, the block remains visible, even when scrollling down the
+        # page. It is enabled by default on "edit" layouts only, because
+        # stickyness of the block of controls may be in conflict with stickyness
+        # of parts of fields shown on the the same page, ie, headers of tables
+        # for Ref or List fields. If several sticky elements are present, when
+        # scrolling, they will unsightly overlap.
+        self.sticky = sticky
+
+    def clone(self):
+        '''Create a clone of p_self'''
+        r = Page(self.name)
+        for k, v in self.__dict__.items(): setattr(r, k, v)
+        return r
+
+    def setIcon(self, icon):
+        '''Sets this p_icon for p_self'''
+        if callable(icon):
+            self.icon = icon
+            self.iconBase = None
+        else:
+            self.icon, self.iconBase = iconParts(icon)
 
     @staticmethod
     def get(pageData):
@@ -115,21 +161,15 @@ class UiPage:
         # Must the page be shown on view/edit layouts ?
         self.showOnView = showOnView
         self.showOnEdit = showOnEdit
-        # Must other sub-elements (buttons "save", "next", etc) be shown ?
+        # Must sub-elements (buttons "save", "cancel", etc) be shown ?
         phases = uiPhase.container
         o = phases.o
         layout = phases.layout
         for elem in Page.subElements:
             showable = page.isShowable(o, layout, elem)
             setattr(self, 'show%s' % elem.capitalize(), showable)
-        # Make a link between this page and the previous one when relevant =
-        # when there is a previous page and when both pages are visible on the
-        # current layout.
-        self.previous = uiPhase.container.lastPage
-        self.next = None
-        if self.previous and \
-           self.previous.showable(layout) and self.showable(layout):
-            self.previous.next = self
+        # Define the icon to display
+        self.iconUrl = self.getIconUrl(o)
 
     def __repr__(self):
         '''p_self's string representation'''
@@ -147,9 +187,26 @@ class UiPage:
         '''Return True if this page is showable on p_layout'''
         return self.showOnEdit if layout == 'edit' else self.showOnView
 
+    def getIconUrl(self, o):
+        '''Compute the URL to the icon to display for this page'''
+        page = self.page
+        # The icon to show
+        if callable(page.icon):
+            url = page.icon(o)
+            if url: return url
+            # The icon may not be available yet. For example, we may be creating
+            # an object having a field that will be used to store the page icon,
+            # but no image has been uploaded yet.
+            base = 'appy'
+            icon = 'page.svg'
+        else:
+            base, icon = page.iconBase, page.icon
+        return o.buildUrl(icon, base)
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Phase:
     '''A phase is a group of pages within a class'''
+
     # This class is used only in order to generate phase- and page-related i18n
     # labels, and as general information about all phases and pages defined for
     # a class in the app's model.
@@ -174,28 +231,85 @@ class UiPhase:
        depending on user's permissions.'''
 
     view = Px('''
-     <table class="phase">
-      <tr valign="top">
-       <!-- The page(s) within the phase -->
-       <td for="opage in phase.pages.values()"
-           class=":(opage.name == page) and 'currentPage' or ''">
-        <!-- Page name and icons -->
-        <x var="label=opage.getLabel()">
-         <a if="opage.showOnView"
-            href=":o.getUrl(sub='view', page=opage.name, \
-                            popup=popup)">::label</a>
-         <x if="not opage.showOnView">::label</x>
-        </x>
-        <x var="locked=o.Lock.isSet(o, user, opage.name);
-                editable=mayEdit and opage.showOnEdit and opage.showEdit">
+     <div class=":'phase phaseC' if compact else 'phase'"
+          if="phase.showPages(singlePhase, isEdit)">
+
+      <!-- Siblings navigation -->
+      <div var="nav=req.nav"
+           if="not isEdit and (phase.name=='main') and (nav) and (nav != 'no')"
+           var2="self=ui.Siblings.get(nav, tool, _ctx_)">:self.pxNavigate</div>
+
+      <!-- Page(s) within the phase -->
+      <div if="not isEdit" for="pag in phase.pages.values()"
+           var2="isCurrent=pag.name == page"
+           id=":'currentPage' if isCurrent else ''"
+           class=":'currentPage' if isCurrent else ''">
+
+       <!-- This additional div is used to vertically align its content -->
+       <div var="viewUrl=o.getUrl(sub='view',page=pag.name,popup=popup,nav=nav)\
+                  if pag.showOnView else None;
+                 pictoUrl=pag.iconUrl">
+        <!-- The edit or lock icon -->
+        <x var="locked=o.Lock.isSet(o, user, pag.name);
+               editable=mayEdit and mayAct and pag.showOnEdit and pag.showEdit">
          <a if="editable and not locked"
-            href=":o.getUrl(sub='edit', page=opage.name, popup=popup)">
-          <img src=":url('edit')" title=":_('object_edit')"/></a>
-         <x var="page=opage.name; lockStyle=''">::o.Lock.px</x>
+            href=":o.getUrl(sub='edit', page=pag.name, popup=popup, nav=nav)">
+          <img src=":url('edit.svg')" class="pictoT shake"
+               title=":_('object_edit')"/></a>
+         <x var="page=pag.name; lockStyle='iconM pictoT'">::o.Lock.px</x>
         </x>
-       </td>
-      </tr>
-     </table>''')
+
+        <!-- Main icon -->
+        <div><a if="viewUrl" href=":viewUrl">
+              <img src=":pictoUrl" class=":picto"/></a>
+         <img if="not viewUrl" src=":pictoUrl" class=":picto"/>
+        </div>
+
+        <!-- Page name and icons -->
+        <x var="label=Px.truncateText(pag.getLabel(), 25)">
+         <a if="viewUrl" href=":viewUrl">::label</a>
+         <x if="not viewUrl">::label</x>
+        </x>
+       </div>
+      </div>
+
+      <!-- Additional object controls -->
+      <x>:o.pxButtons</x>
+     </div>''',
+
+     # Class names ending with "C" are used when controls are *C*ompact
+     css='''
+      .phase { margin: 0 0 30px -30px; display: flex; flex-wrap: wrap;
+               width: fit-content }
+      .phase > div { height: 115px; width: 125px; text-align: center;
+        text-transform: uppercase; font-weight: bold; font-size: 90%;
+        display: flex; align-items: center; justify-content: center;
+        background-color: white; position: relative; padding: 0 5px;
+        border-right: 1px solid |linkColor|; border-top: 1px solid |linkColor|;
+        border-bottom: 1px solid |linkColor| }
+
+      .phaseC { margin: 0 0 20px 0; border-left: 1px solid |linkColor| }
+      .phaseC > div { height: 80px; width: 110px; font-size: 9pt }
+
+      .picto { width: 55px; height: 55px; margin-bottom: 6px }
+      .pictoT { position: absolute; top: 8px; right: 8px; width: 22px }
+      .pictoC, .pictoB { width: 35px; height: 35px; margin-bottom: 4px }
+      .inverted { filter: invert(66%) }
+
+      .currentPage { background-color: |headerBgColor| !important;
+                     color: |headerColor| }
+      .currentPage img { filter: invert(66%) }
+      .currentPage a { color: |headerColor| }
+
+      // Animation for the "edit" icon
+      @keyframes shakeImage {
+        0% { transform: rotate(0deg); filter: sepia(0) }
+        95% { transform: rotate(0deg); filter: sepia(0) }
+        97% { transform: rotate(10deg); filter: sepia(1) }
+        99% { transform: rotate(-10deg); filter: sepia(0.6) }
+        100% { transform: rotate(0deg); filter: sepia(0.3) }
+      }
+      .shake { animation: shakeImage 10s infinite }''')
 
     def __init__(self, name, phases):
         # The name of the phase
@@ -210,10 +324,6 @@ class UiPhase:
         # The pages that were already walked, but that must not be shown
         self.hidden = {}
 
-    def singlePaged(self):
-        '''Is p_self a phase containing a single page ?'''
-        return len(self.pages) == 1
-
     def addPage(self, page):
         '''Adds page-related information in the phase'''
         # If the page is already there, we have nothing more to do
@@ -235,6 +345,17 @@ class UiPhase:
             # The page must be hidden, and we must remember that fact
             self.hidden[page.name] = None
 
+    def showPages(self, singlePhase, isEdit):
+        '''Pages for this phase must not be shown if there is a single,
+           not-editable page within a single phase.'''
+        # If we are on an "edit" layout, the "pages" zone must always be shown
+        # because it will contain at least the "save" and "cancel" buttons.
+        if isEdit: return True
+        if singlePhase and (len(self.pages) == 1):
+            for page in self.pages.values():
+                return page.showOnEdit
+        return True
+
     def __repr__(self):
         if self.pages:
             pages = ': page(s) %s' % ','.join(list(self.pages.keys()))
@@ -249,14 +370,16 @@ class UiPhases:
 
     # PX displaying all phases of a given object
     view = Px('''
-     <x if="not phases.singlePage()"
-        var2="mayEdit=guard.mayEdit(o);
-              singlePhase=phases.singlePhase();
-              id=o.iid">
+     <x var="mayEdit=guard.mayEdit(o);
+             singlePhase=phases.singlePhase();
+             id=o.iid">
+
       <!-- Single phase: display its pages -->
-      <x if="singlePhase" var2="phase=phases.default">:phase.view</x>
+      <x if="singlePhase or isEdit"
+         var2="phase=phases.default">:phase.view</x>
+
       <!-- Several phases: add a tab for every phase -->
-      <x if="not singlePhase">
+      <x if="not singlePhase and not isEdit">
        <table cellpadding="0" cellspacing="0">
         <!-- First row: the tabs -->
         <tr><td class="tabSep">
@@ -283,7 +406,43 @@ class UiPhases:
        <script>:'initTab(%s,%s)' % \
              (q('tab_%d' % id), q('%d_%s' % (id, phases.default.name)))</script>
       </x>
-     </x>''')
+      <script>:'initPages(%d)' % phases.getStickyIndex(table)</script>
+     </x>''',
+
+     js = '''
+      function invertPage(div, on) {
+        // Do nothing if p_div correspond to the currently selected page
+        if (div.id == 'currentPage') return;
+        // Switch the page to dark mode
+        var classes = div.classList,
+            inverted = classes.contains('currentPage');
+        if (on && !inverted) classes.add('currentPage');
+        else if (!on && inverted) classes.remove('currentPage');
+      }
+
+      function initPages(rowIndex) {
+        // Inject code allowing to switch between the (un)selected page view
+        var phases = document.getElementsByClassName('phase'),
+            divs=null, div=null;
+        for (var i=0; i < phases.length; i++) {
+          divs = phases[i].childNodes;
+          for (var j=0; j < divs.length; j++) {
+            div = divs[j];
+            if (div.tagName == 'DIV') {
+              div.addEventListener('mouseenter',
+                                   function(){invertPage(this,true)});
+              div.addEventListener('mouseleave',
+                                   function(){invertPage(this,false)});
+            }
+          }
+        }
+        if (rowIndex != -1) {
+          // Make the controls sticky
+          var table = document.getElementById('pageLayout'),
+              firstCell = table.childNodes[0].rows[rowIndex].cells[0];
+          firstCell.classList.add('sticky');
+        }
+      }''')
 
     def __init__(self, page, o, layout):
         '''Initialises a structure allowing to render currently visible phases
@@ -343,5 +502,13 @@ class UiPhases:
         # If p_self.currentPage is None, it will crash. Set, as current page,
         # the first visible page, just to avoid a crash.
         if not self.currentPage:
-            raise Exception(NO_CURRENT_PAGE)
+            raise Exception(NO_CUR_PAGE)
+
+    def getStickyIndex(self, table):
+        '''Returns the index of the element, within the page layout represented
+           by p_table, that must be made sticky.'''
+        # If stickyness must not be enabled for the current page, return -1
+        sticky = self.currentPage.page.sticky
+        sticky = (sticky == self.layout) if isinstance(sticky, str) else sticky
+        return table.getRowIndex('b') if sticky else -1
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
