@@ -21,13 +21,13 @@ from DateTime import DateTime
 
 import appy
 from appy.px import Px
-from appy.model.fields import Field
 from appy.utils import path as putils
 from appy.pod.renderer import Renderer
 from appy.model.searches import Search
 from appy.utils import string as sutils
 from appy.utils.string import Normalize
 from appy.model.utils import Object as O
+from appy.model.fields import Field, Show
 from appy.ui.layout import Layouts, Layout
 from appy.model.fields.file import FileInfo
 from appy.pod import PodError, styles_manager
@@ -113,6 +113,43 @@ class Mailing:
            method returns this "label".'''
         return '%s<br/><b>%s</b>' % (self.subject, _('email_body'))
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class ImageFinder:
+    '''Used by the POD renderer (XHTML conversion) to retrieve paths to local
+       images instead of performing HTTP GETs to the server running POD.'''
+
+    def __init__(self, o):
+        self.o = o
+        self.siteUrl = o.siteUrl
+
+    def find(self, url):
+        '''If p_url is local to the site running this code, this method returns
+           the absolute path to the file on disk. Else, it returns None, letting
+           POD perform a HTTP GET request to the "external" web server.'''
+        # Return None if the image is not local to this webserver
+        base = self.siteUrl
+        if not url.startswith(base): return
+        # Get the image path on disk. We make the assumption that the file is
+        # stored in a File field on some object, so URL is supposed to be of the
+        # form
+        #               <base>/<objectId>/<fieldName>/download
+        o = info = None
+        for part in url[len(base):].split('/'):
+            # Ignore empty parts
+            if not part: continue
+            # Get the object on which the File field is defined
+            if o is None:
+                o = self.o.getObject(part)
+                if o is None: return
+                continue
+            # Get the FileInfo instance on this object
+            if info is None:
+                info = getattr(o, part)
+                if not info: return
+                break
+        # If we are here, we can determine the path to the file on disk
+        return str(info.getFilePath(o))
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Pod(Field):
     '''A pod is a field allowing to produce a (PDF, ODT, Word, RTF...) document
@@ -153,21 +190,25 @@ class Pod(Field):
 
     # Icon allowing to generate a given template in a given format
     pxIcon = Px('''
-     <img var="iconSuffix='Frozen' if frozen else '';
-               iconId=field.getIconId(confirm, id, info, fmt);
-               iconGetter='getNode(%s)' % q(iconId) if confirm else 'this';
-               js='generatePod(%s,%s,%s,%s,%s,%s,null,%s)' % \
-                (iconGetter, q(o.url), q(name), q(info.template), q(fmt), \
+     <a var="fmtSuffix='●' if frozen else '';
+             linkId=field.getLinkId(confirm, id, info, fmt);
+             linkGetter='getNode(%s)' % q(linkId) if confirm else 'this';
+             js='generatePod(%s,%s,%s,%s,%s,%s,null,%s)' % \
+                 (linkGetter, q(o.url), q(name), q(info.template), q(fmt), \
                  q(field.getSearchParams(req, layout)), gc)"
-          src=":url(fmt + iconSuffix)" class="clickable" id=":iconId"
-          title=":field.getIconTitle(o, fmt, frozen)"
-          onclick=":confirm and 'askConfirm(%s,%s,%s)' % (q('script'), \
-            q(js,False), field.labelId) or js"/>''')
+        onclick=":confirm and 'askConfirm(%s,%s,%s)' % (q('script'), \
+                 q(js,False), field.labelId) or js"
+        title=":field.getIconTitle(o, fmt, frozen)" class="clickable"
+        style="position: relative" id=":linkId">
+      <img src=":url('download.svg')" class="iconS"/>
+      <span class="fmt">:fmt.upper() + fmtSuffix</span>
+     </a>''')
 
     view = cell = Px('''
      <script if="field.confirm">::field.getJsConfirmVar(o)</script>
      <x var="id=o.iid;
-             gc=field.getCheckHook(o, req)"
+             gc=field.getCheckHook(o, req);
+             onCell=layout == 'cell'"
         for="info in field.getVisibleTemplates(o)"
         var2="mailings=field.getVisibleMailings(o, info.template);
               lineBreak=((loop.info.nb + 1) % field.maxPerRow) == 0;
@@ -236,15 +277,23 @@ class Pod(Field):
         </table>
        </span>
       </x>
-      <!-- Show the specific template name only if there is more than one
-           template. For a single template, the field label already does the
-           job. -->
-      <span if="field.useTemplateName"
+      <!-- Show the specific template name when relevant -->
+      <span if="not onCell and field.useTemplateName"
         class=":'pod smaller' if not loop.info.last and not lineBreak else \
             'podLast smaller'">:field.getTemplateName(o, info.template)</span>
       <br if="lineBreak"/>
      </x>''',
 
+     css='''.phase .fmt { position: absolute; left: 27%; bottom: -80%;
+                          font-size: 85% }
+            .list .fmt { font-weight: normal; margin-right: -8px }
+            @keyframes movedown {
+              from { top: 0px  }
+              50% { top: -5px }
+              to { top: 0px }
+            }
+            .waiting { animation: movedown 0.7s ease infinite }''',
+
      js='''
       podDownloadStatus = function(node, data) {
         // Checks the status of cookie "podDownload"
@@ -282,21 +331,16 @@ class Pod(Field):
         f.action.value = 'generate';
         // Initialise the status of checkboxes
         setChecked(f, checkHook);
-        /* If p_node is an image, replace it with a preloader to prevent
-           double-clicks. */
-        if (node.tagName == 'IMG') {
-          var data = {'src': node.src, 'class': node.className,
-                      'onclick': node.attributes.onclick.value};
-          node.setAttribute('onclick', '');
-          var src2 = node.src.replace(/[\w\d_]+\.png/, 'loadingPod.gif');
-          node.setAttribute('src', src2);
-          /* Initialize the pod download cookie. "False" means: not downloaded
-             yet. */
-          createCookie('podDownload', 'False');
-          // Set a timer that will check the cookie value
-          podTimeout = window.setInterval(function(){
-            podDownloadStatus(node, data)}, 700);
-        }
+        // Disable the link to prevent double-clicks
+        var data = {'class': node.className,
+                    'onclick': node.attributes.onclick.value};
+        node.setAttribute('onclick', '');
+        node.className = 'waiting';
+        // Set the pod download cookie. "False" means: not downloaded yet.
+        createCookie('podDownload', 'False');
+        // Set a timer that will check the cookie value
+        podTimeout = window.setInterval(function(){
+          podDownloadStatus(node, data)}, 700);
         f.submit();
       }
 
@@ -322,19 +366,19 @@ class Pod(Field):
 
     edit = search = ''
 
-    def __init__(self, validator=None, show=('view', 'result'), page='main',
-      group=None, layouts=None, move=0, readPermission='read',
-      writePermission='write', width=None, height=None, maxChars=None,
-      colspan=1, master=None, masterValue=None, focus=False, historized=False,
-      mapping=None, generateLabel=None, label=None, template=None,
-      templateName=None, useTemplateName=None, showTemplate=None,
-      freezeTemplate=None, showFrozenOnly=True, maxPerRow=5, context=None,
-      stylesMapping={}, formats=None, getChecked=None, mailing=None,
-      mailingName=None, showMailing=None, mailingInfo=None, view=None,
-      cell=None, edit=None, xml=None, translations=None, downloadName=None,
+    def __init__(self, validator=None, show=Show.TR, page='main', group=None,
+      layouts=None, move=0, readPermission='read', writePermission='write',
+      width=None, height=None, maxChars=None, colspan=1, master=None,
+      masterValue=None, focus=False, historized=False, mapping=None,
+      generateLabel=None, label=None, template=None, templateName=None,
+      useTemplateName=None, showTemplate=None, freezeTemplate=None,
+      showFrozenOnly=True, maxPerRow=5, context=None, stylesMapping={},
+      formats=None, getChecked=None, mailing=None, mailingName=None,
+      showMailing=None, mailingInfo=None, view=None, cell=None, edit=None,
+      xml=None, translations=None, downloadName=None,
       downloadDisposition='attachment', forceOoCall=False,
       optimalColumnWidths=False, distributeColumns=None, script=None,
-      pdfOptions='ExportNotes=True', tabbedCR=False, confirm=False,
+      pdfOptions='ExportNotes=True', tabbedCR=False, fonts=None, confirm=False,
       raiseOnError=False, action=None, beforeAction=None):
         # Param "template" stores the path to the pod template(s). If there is
         # a single template, a string is expected. Else, a list or tuple of
@@ -352,16 +396,18 @@ class Pod(Field):
         # Param "templateName", if specified, is a method that will be called
         # with the current template (from self.template) as single arg and must
         # return the name of this template. If self.template stores a single
-        # template, you have no need to use param "templateName". Simply use the
-        # field label to name the template. But if you have a multi-pod field
-        # (with several templates specified as a list or tuple in param
-        # "template"), you will probably choose to hide the field label and use
-        # param "templateName" to give a specific name to every template. If
+        # template, you have no need to use param "templateName" (excepted if
+        # you want to render the POD with a name, in the layout "buttons").
+        # In most cases, if you have a single template, simply use the field
+        # label to name the template. If you have a multi-pod field (with
+        # several templates specified as a list or tuple in param "template"),
+        # you will probably choose to hide the field label and use param
+        # "templateName" to give a specific name to every template. If
         # "template" contains several templates and "templateName" is None, Appy
         # will produce names from template filenames.
         self.templateName = templateName
         # If you want to use "templateName" hereabove even if self.template
-        # contains a single template, set "usetTemplateName" to True.
+        # contains a single template, set "useTemplateName" to True.
         if useTemplateName is None:
             self.useTemplateName = len(self.template) > 1
         else:
@@ -521,6 +567,11 @@ class Pod(Field):
         # "tabbedCR" determines how the "carriage return" char is rendered on
         # POD results. See appy.pod.renderer.py for more information.
         self.tabbedCR = tabbedCR
+        # The name of the font passed in "fonts" will be injected in all styles
+        # from the POD template. It must return a string or be a function that
+        # will accept the current template (one among self.template) as unique
+        # arg and return the name of the font to inject in it.
+        self.fonts = fonts
         # If "confirm" is True, a popup will be shown before generating the
         # pod result. "confirm" may also hold a method, that must accept, as
         # single arg, the name of the current template (indeed, self.template
@@ -569,6 +620,10 @@ class Pod(Field):
            p_template.'''
         return Pod.allFormats[self.getExtension(template)]
 
+    def isRenderable(self, layout):
+        '''A single-template POD is renderable on layout "buttons"'''
+        return True if layout != 'buttons' else len(self.template) == 1
+
     def setTemplateFolder(self, folder):
         '''This methods adds a prefix to every template name in self.template'''
         # This can be useful if a plug-in module needs to replace an application
@@ -719,8 +774,8 @@ class Pod(Field):
                 r = 'null'
         return r
 
-    def getIconId(self, confirm, objectId, info, fmt):
-        '''Returns an identifier for an icon allowing to trigger a pod. This
+    def getLinkId(self, confirm, objectId, info, fmt):
+        '''Returns an identifier for a link allowing to trigger a pod. This
            identifier is long and is not needed when p_confirm is False.'''
         if not confirm: return ''
         return '%s_%s_%s_%s' % (objectId, self.name,
@@ -843,13 +898,17 @@ class Pod(Field):
         # Compute PDF options
         options = self.pdfOptions
         options = options(o, template) if callable(options) else options
+        # Compute fonts
+        fonts = self.fonts
+        fonts = fonts(o, template) if callable(fonts) else fonts
         # Compute the renderer's parameters
         rendererParams = {'template': templatePath, 'context': podContext,
           'result': result, 'stylesMapping': smap, 'overwriteExisting': True,
           'forceOoCall': self.forceOoCall, 'raiseOnError': self.raiseOnError,
           'script': script, 'optimalColumnWidths': self.optimalColumnWidths,
           'distributeColumns': self.distributeColumns, 'pdfOptions': options,
-          'tabbedCR': self.tabbedCR}
+          'tabbedCR': self.tabbedCR, 'fonts': fonts,
+          'findImage': ImageFinder(o).find}
         config = o.config.pod or Pod.defaultConfig
         if config.unoEnabledPython:
             rendererParams['pythonWithUnoPath'] = config.unoEnabledPython
