@@ -17,6 +17,9 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+UPLOADED = 'file "%s" uploaded in Ref "%d.%s" via rich "%s".'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import sys
 
@@ -26,17 +29,18 @@ from appy.model.fields import Field
 from appy.xml.cleaner import Cleaner
 from appy.utils.diff import HtmlDiff
 from appy.model.fields.text import Text
+from appy.utils import string as sutils
 from appy.ui.layout import Layouts, Layout
 from appy.model.fields.multilingual import Multilingual
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NOT_IMPLEMENTED = 'This feature is not implemented yet'
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Rich(Multilingual, Field):
     '''Field allowing to encode a "rich" text, based on XHTML and external
        editor "ckeditor".'''
 
+    # Some elements are traversable
+    traverse = {}
+
     # Required Javascript files
     cdnUrl = 'https://cdn.ckeditor.com/%s/%s/ckeditor.js'
 
@@ -45,8 +49,10 @@ class Rich(Multilingual, Field):
 
     class Layouts(Layouts):
         '''Rich-specific layouts'''
-        b = Layouts(edit='lrv-d-f', view='l-f')
-        c = Layouts(edit='lrv-d-f', view='lc-f') # Idem but with history
+        baseEdit = Layout('lrv-d-f', css_class='bottomSpaceS')
+        b = Layouts(edit=baseEdit, view='l-f')
+        c = Layouts(edit=baseEdit, view='lc-f') # Idem but with history
+        f = Layouts(edit=baseEdit, view='f') # Field only (no label) on view
         g = Layouts(edit=Layout('d2-f;rv=', width='99%'),
                     view=Layout('fl', width='99%'))
         gc = Layouts(edit=Layout('drv-f', width='99%'),
@@ -76,10 +82,16 @@ class Rich(Multilingual, Field):
     # Default styles to use
     defaultStyles = ('p', 'h1', 'h2', 'h3', 'h4')
 
-    # Default custom style, allowing to produce, in POD results, a page break
-    # just after the paragraph having this style.
-    customStyles = [{'id': 'div', 'element': 'div',
-                     'attributes': {'style': 'page-break-after:always'}}]
+    # Default custom styles
+    customStyles = [
+      # When this style is applied, at the XHTML level, it has no effet, but
+      # when dumped in a POD, a page break is inserted just after the paragraph
+      # having this style.
+      {'id': 'div', 'element': 'div',
+       'attributes': {'style': 'page-break-after:always'}},
+      # Style for a block of text rendered in a monospace font
+      {'id': 'pre', 'element': 'pre'}
+    ]
 
     # The name of the index class storing values of this field in the catalog
     indexType = 'RichIndex'
@@ -118,7 +130,7 @@ class Rich(Multilingual, Field):
       masterValue=None, focus=False, historized=False, mapping=None,
       generateLabel=None, label=None, sdefault='', scolspan=1, swidth=None,
       sheight=None, persist=True, styles=defaultStyles, customStyles=None,
-      allowImageUpload=False, spellcheck=False, languages=('en',),
+      documents=False, spellcheck=False, languages=('en',),
       languagesLayouts=None, inlineEdit=False, toolbar='Standard', view=None,
       cell=None, edit=None, xml=None, translations=None):
         # The list of styles that the user will be able to select in the styles
@@ -149,9 +161,12 @@ class Rich(Multilingual, Field):
         #            | styles because they will produce a similar result.
         # ----------------------------------------------------------------------
         self.customStyles = customStyles or Rich.customStyles
-        # Do we allow the user to upload images in it ?
-        self.allowImageUpload = allowImageUpload
-        if allowImageUpload: raise Exception(NOT_IMPLEMENTED)
+        # By mentioning, in attribute "documents", the name of a Ref field
+        # pointing to Document instances (see appy/model/document.py), images
+        # upload is actived in this rich field. An illustration can be found in
+        # standard rich field appy/model/page.py::content, that uses, on the
+        # same class, the Ref field named "documents".
+        self.documents = documents
         # Do we run the CK spellchecker ?
         self.spellcheck = spellcheck
         # What toolbar is used ? Possible values are "Standard" or "Simple"
@@ -239,7 +254,7 @@ class Rich(Multilingual, Field):
     def getUniStorableValue(self, o, value):
         '''Gets the p_value as can be stored in the database within p_obj'''
         if not value: return value
-        # Clean the value. When image upload is allowed, ckeditor inserts some
+        # Clean the value. When image upload is enabled, ckeditor inserts some
         # "style" attrs (ie for image size when images are resized). So in this
         # case we can't remove style-related information.
         try:
@@ -297,8 +312,18 @@ class Rich(Multilingual, Field):
                 id = style['id']
                 ckAttrs['format_%s' % id] = style
                 ckAttrs['format_tags'] += ';%s' % id
-        if self.allowImageUpload:
-            ckAttrs['filebrowserUploadUrl'] = '%s/upload' % o.url
+        if self.documents and not o.isTemp():
+            base = o.url
+            # Parameter "type=any" is only useful in order to have a parameter.
+            # Indeed, when a file is cut & pasted or dragged & dropped to
+            # ckeditor, ckeditor calls the URL with suffix "&responseType=json".
+            # If there is no existing parameter, the syntax is incorrect
+            # (absence of '?' between the URL and its parameters) and the
+            # request ends up with a 404 error.
+            ckAttrs['filebrowserUploadUrl'] = '%s/%s/upload?type=any' % \
+                                              (base, self.name)
+            ckAttrs['filebrowserBrowseUrl'] = '%s/pxField?name=%s&popup=True' \
+              '&selector=%d,%s,repl' % (base, self.documents, o.iid, self.name)
         if not o.user.hasRole('Manager'):
             ckAttrs['removeButtons'] = 'Source'
         ck = []
@@ -339,4 +364,19 @@ class Rich(Multilingual, Field):
                "function( event ) { var content = event.editor.getData(); " \
                "doInlineSave('%d','%s','%s','view',true,content,'%s')}}})" % \
                (id, name, self.getCkParams(o,language), id, fieldName, o.url,lg)
+
+    traverse['upload'] = 'perm:write'
+    def upload(self, o):
+        '''Uploads a document to the Ref mentioned in attribute "documents"'''
+        # Get the file from the request
+        ofile = o.req.upload
+        # Create a Document instance and link it to p_o via the Ref mentioned in
+        # self.documents.
+        doc = o.create(self.documents, title=ofile.name, file=ofile)
+        o.H().commit = True
+        o.resp.setContentType('json')
+        r = {'uploaded': 1, 'fileName': doc.title,
+             'url': '%s/file/download' % doc.url}
+        o.log(UPLOADED % (ofile.name, o.iid, self.documents, self.name))
+        return sutils.getStringFrom(r, c='"')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
