@@ -84,7 +84,7 @@ class Selection:
                 method = elems[0]
                 args = elems[1:]
             else:
-                args = ()
+                args = []
             # On what object must we call the method ?
             if method.startswith('tool:'):
                 o = o.tool
@@ -424,7 +424,7 @@ class Select(Field):
                     break
         if error: return o.translate('bad_select_value')
 
-    def getStorableValue(self, o, value):
+    def getStorableValue(self, o, value, complete=False):
         '''Get a multivalued value when appropriate'''
         if value and self.isMultiValued() and \
            (type(value) not in utils.sequenceTypes):
