@@ -616,7 +616,7 @@ class Text(Multilingual, Field):
                  class=":'clickable icon' if inToolbar else 'inlineIcon'"
                  title=":_('object_cancel')"/></div>
       </div>
-      <script>:'prepareForAjaxSave(%s,%d,%s,%s)' % \
+      <script>:'prepareForAjaxSave(%s,%s,%s,%s)' % \
                (q(name), q(o.iid), q(o.url), q(hostLayout))</script>''')
 
     # Unilingual edit
