@@ -35,12 +35,14 @@ from appy.model.group import Group
 from appy.model.fields import Field
 from appy.model.workflow import Role
 from appy.model.searches import Search
+from appy.model.document import Document
 from appy.model.utils import Object as O
 from appy.model.fields.phase import Phase
 from appy.model.fields.string import String
 from appy.model.translation import Translation
 from appy.model.workflow.history import History
 from appy.model.searches.gridder import Gridder
+from appy.model.fields.computed import Computed
 from appy.model.workflow.localRoles import LocalRoles
 from appy.model.fields.phase import Page as FieldPage
 
@@ -191,7 +193,11 @@ class Class(Meta):
             # To get a field value is done via method Field::getValue
             getter = lambda self, field=field: field.getValue(self)    
             # To set a value is done via method Field::store
-            setter = lambda self, v, field=field: field.store(self, v)
+            if field.type == 'Ref':
+                setter = lambda self, v, field=field: \
+                                field.store(self, v, secure=False)
+            else:
+                setter = lambda self, v, field=field: field.store(self, v)
             setattr(class_, name, property(getter, setter))
 
     def getFieldClasses(self, class_):
@@ -239,11 +245,14 @@ class Class(Meta):
                         raise Err(UNALLOWED_FIELD_NAME % (name, self.name))
                     # A field was found.
                     # ~~~
-                    # Special field "title" must be duplicated. Else, all app
-                    # classes will get the modifications that the developer can
-                    # apply to it via "update" methods.
+                    # Special fields "title" and "record" must be duplicated.
+                    # Else, all app classes will get the modifications that the
+                    # developer can apply to it via "update" methods.
                     if name == 'title':
                         field = String(**Base.titleAttributes)
+                    elif name == 'record':
+                        field = Computed(**Base.recordAttributes)
+                        field.page = field.page.clone()
                     # Late-initialize it
                     field.init(self, name)
                     r[name] = field
@@ -346,8 +355,7 @@ class Class(Meta):
         '''Returns the CSS class(es) to apply when displaying a list (=a table)
            containing instances of this class.'''
         css = getattr(self.python, 'listCss', None)
-        if css: return css
-        return 'list' if layout == 'cell' else 'list compact'
+        return css if css else 'list'
 
     def getListTitle(self, o):
         '''Gets p_o's title as it must appear within lists of objects'''
@@ -444,10 +452,6 @@ class Class(Meta):
         # advanced search.
         return advanced.isShowable(tool)
 
-    def mayNavigate(self, o):
-        '''May the user see the navigation panel linked to p_o ?'''
-        return o.mayNavigate() if hasattr(self.python, 'mayNavigate') else True
-
     def isIndexable(self):
         '''Is p_self "indexable" ? In other words: do we need to have a Catalog
            storing index values for instances of this class ?'''
@@ -560,9 +564,7 @@ class Class(Meta):
         '''Gets the CSS class(es) to apply when displaying a table containing
            p_self's instances.'''
         r = getattr(self.python, 'listCss', None)
-        if not r:
-            r = 'list' if layout == 'cell' else 'list compact'
-        return r
+        return r if r else 'list'
 
     def getTitleMode(self):
         '''Gets the "title mode" for instances of this class'''
@@ -595,18 +597,17 @@ class Class(Meta):
     pxAddFrom = Px('''
      <a target="appyIFrame" id=":addFormName + '_from'"
         href=":class_.getCreateLink(tool, create, addFormName, sourceField)">
-      <input var="css='Small' if fromRef else 'Icon';
+      <input var="css='Small' if fromRef else 'Portlet';
                   label=_('object_add_from')"
-         type="button" value=":label if fromRef else ''" 
-         title=":label" class=":'button%s button' % css"
-         onclick="openPopup('iframePopup')" style=":url('addFrom', bg=True)"/>
+         type="button" value=":label" class=":'button%s button' % css"
+         onclick="openPopup('iframePopup')" style=":url('add.svg', bg=True)"/>
      </a>''')
 
     # Form for creating instances of this class from the portlet or a search
     pxAdd = Px('''
      <form var="create=class_.getCreateVia(tool)"
            if="create" class="addForm" name=":'%s_add' % className"
-           var2="target=ui.LinkTarget(class_)"
+           var2="target=ui.LinkTarget(class_.python); label=_('object_add');"
            action=":'%s/new' % tool.url" target=":target.target">
       <input type="hidden" name="className" value=":className"/>
       <input type="hidden" name="template" value=""/>
@@ -615,14 +616,16 @@ class Class(Meta):
       <input type="hidden" name="popup"
            value=":'True' if (popup or (target.target!='_self')) else 'False'"/>
       <!-- Create from an empty form -->
-      <input var="label=_('object_add');
-                  suffix='Small' if asButton else 'Icon'"
-             class=":'button%s button' % suffix" style=":url('add', bg=True)"
-             value=":label if asButton else ''" title=":label" type="submit"
+      <input var="suffix='Small' if asButton else 'Portlet';
+                  size='18px 18px' if asButton else '24px 24px'"
+             class=":'button%s button' % suffix" style=":url('add.svg',bg=size)"
+             value=":label" title=":label" type="submit"
              onclick=":target.getOnClick('searchResults')"/>
       <!-- Create from a pre-filled form when relevant -->
-      <x if="create != 'form'"
+      <div if="create != 'form'" class="addFrom"
          var2="fromRef=False; sourceField=None;
-               addFormName='%s_add' % className">:class_.pxAddFrom</x>
-     </form>''')
+               addFormName='%s_add' % className">:class_.pxAddFrom</div>
+     </form>''',
+
+     css='''.addFrom { margin-top: 5px }''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
