@@ -78,23 +78,28 @@ class UiSearch:
       </x>
 
       <!-- Title -->
-      <h3 if="not popup"><x>::uiSearch.translated</x>
-       <x if="mode.batch and not empty">(
-        <span class="discreet">:mode.batch.total</span>)
+      <div if="not popup" class="pageTitle topSpaceS">
+       <x>::uiSearch.translated</x>
+       <x if="mode.batch and not empty">
+        <img src=":url('arrowsA.svg')" class="iconS"
+             style="transform: rotate(270deg)"/>
+        <x>:mode.batch.total</x>
        </x>
        <x if="uiSearch.search.add and guard.mayInstantiate(class_)"
           var2="asButton=True;
                 nav=mode.getNavInfo(batch.total)">:class_.pxAdd</x>
-       <x if="showNewSearch and mode.newSearchUrl">&nbsp;&mdash;&nbsp;<i> 
-        <a href=":mode.newSearchUrl">:_('search_new')</a></i></x>
-       <x if="mode.fromRef">&nbsp;&mdash;&nbsp;<i> 
-        <a href=":mode.getRefUrl()">:_('goto_source')</a></i></x>
-      </h3>
+      </div>
       <table width="100%">
        <tr valign="top">
         <!-- Search description -->
-        <td if="uiSearch.translatedDescr">
-         <span class="discreet">:uiSearch.translatedDescr</span><br/>
+        <td>
+         <div class="discreet"
+              if="uiSearch.translatedDescr">:uiSearch.translatedDescr</div>
+         <!-- Perform a new search -->
+         <a if="showNewSearch and mode.newSearchUrl"
+            href=":mode.newSearchUrl">:_('search_new')</a>
+         <x if="mode.fromRef">&nbsp;&mdash;&nbsp; 
+          <a href=":mode.getRefUrl()">:_('goto_source')</a></x>
         </td>
         <!-- (Top) navigation -->
         <td if="mode.batch" align=":dright">:mode.batch.pxNavigate</td>
@@ -105,7 +110,7 @@ class UiSearch:
       <x if="not empty" var2="currentNumber=0">:mode.px</x>
 
       <!-- (Bottom) navigation -->
-      <x if="mode.batch">:mode.batch.pxNavigate</x>
+      <div if="mode.batch" align=":dright">:mode.batch.pxNavigate</div>
 
       <!-- No result -->
       <x if="empty">
@@ -215,7 +220,14 @@ class UiSearch:
 
     def highlight(self, text):
         '''Highlight search results within p_text'''
-        return Critera.highlight(self.tool.H(), text)
+        return Criteria.highlight(self.tool.H(), text)
+
+    def parseFilters(self, req):
+        '''Parses the filters from the p_req(uest) or return None if not
+           found.'''
+        r = req.filters
+        if r is None: return {}
+        return sutils.getDictFrom(r)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Search:
@@ -231,7 +243,7 @@ class Search:
                  show=True, showActions='all', actionsDisplay='block',
                  translatedDescr=None, checkboxes=False, checkboxesDefault=True,
                  container=None, add=False, resultModes=None, shownInfo=None,
-                 actions=None, **fields):
+                 actions=None, rowAlign='top', **fields):
         # "name" is mandatory, excepted in some special cases (ie, when used as
         # "select" param for a Ref field).
         self.name = name
@@ -254,7 +266,7 @@ class Search:
         # homonyms on the Ref class.
         self.showActions = showActions
         self.actionsDisplay = actionsDisplay
-        # In dict "fiels", keys are names of indexed fields and values are
+        # In dict "fields", keys are names of indexed fields and values are
         # simple search values or terms built with database operators.
         self.fields = fields
         # Must checkboxes be shown for every object of the search result ?
@@ -278,6 +290,9 @@ class Search:
         # Specify here Action fields that must be shown as custom actions that
         # will be triggered on search results.
         self.actions = actions
+        # When search resulrs are rendered as a list, this attribute specifies
+        # each row's vertical alignment.
+        self.rowAlign = rowAlign
 
     def init(self, class_):
         '''Lazy search initialisation'''
@@ -476,15 +491,19 @@ class Search:
         return r if not ui else UiSearch(r, tool, ctx, initiator, name)
 
     @classmethod
-    def getRefInfo(class_, tool, info=None, nameOnly=True):
-        '''When a search is restricted to objects referenced through a Ref
-           field, this method returns information about this reference: the
-           source object and the Ref field.'''
+    def getRefInfo(class_, tool, info=None, nameOnly=True, sep=':'):
+        '''When a search is triggered in the context of a Ref field, this method
+           returns information about this reference: the source object and the
+           Ref field.'''
+        # It can happen in the following cases.
+        # 1. A search is restricted to objects referenced through a Ref field
+        # 2. A search is executed in the context of a Ref field with
+        #    link="dropdown".
         if not info:
             criteria = Criteria.readFromRequest(tool.H())
             if criteria and ('_ref' in criteria): info = criteria['_ref']
-        if not info: return None, None
-        id, field = info.split(':')
+        if not info or (sep not in info): return None, None
+        id, field = info.split(sep)
         o = tool.getObject(id)
         field = field if nameOnly else o.getField(field)
         return o, field
@@ -549,21 +568,35 @@ class Search:
 
     # Display results of a live search
     liveResults = Px('''
-     <div var="objects=uiSearch.search.run(handler).objects"
-          id=":'%s_LSResults' % className">
-      <p if="not objects" class="lsNoResult">::_('query_no_result')</p>
+     <div var="search=uiSearch.search;
+               filters=uiSearch.parseFilters(req);
+               objects=search.run(handler,filters=filters).objects;
+               io,ifield=search.getRefInfo(tool, pre, nameOnly=False, sep='_')"
+          id=":'%s_LSResults' % pre">
+      <p if="not objects">::_('query_no_result')</p>
       <div for="o in objects" style="padding: 3px 5px">
-       <a href=":o.url" var="content=Px.truncateValue(o.title, width=80)"
+       <a href=":search.onSelectLiveResult(o, io, ifield)"
+          var="content=Px.truncateValue(o.title, width=80)"
           title=":o.title">:content</a>
       </div>
       <!-- Go to the page showing all results -->
-      <div if="objects" align=":dright" style="padding: 3px">
-       <a class="clickable" style="font-size: 80%; font-style: italic"
-          onclick=":'document.forms[%s].submit()' % \
-            q('%s_LSForm' % className)">:_('search_results_all') + '...'</a>
+      <div if="objects and not io" align=":dright" style="padding: 3px">
+       <a href=":'javascript:allLSResults(%s,%s)' % (q(pre), q(className))"
+          class="lsAll">:_('search_results_all') + '...'</a>
       </div>
      </div>''')
 
+    def onSelectLiveResult(self, o, io, ifield):
+        '''Get the action to perform when the user click on a live search
+           result.'''
+        # If there is no *i*initiator *o*bject (io), redirect the user to p_o's
+        # view page.
+        if io is None: return o.url
+        # Else, update the initiator field (io.ifield) in order to add p_o
+        multi = 'true' if ifield.isMultiValued() else 'false'
+        return "javascript:addLSResult('%s','%s','%s',%s)" % \
+               (io.iid, ifield.name, o.iid, multi)
+
     # The advanced search form
     advanced = Px('''
      <x var="refInfo=req.ref;
@@ -575,8 +608,11 @@ class Search:
       <script>var errors = null;</script>
 
       <!-- Search title -->
-      <h1><x>:_('%s_plural' % className)</x> &ndash;
-          <x>:_('search_title')</x></h1>
+      <div class="pageTitle bottomSpace">
+       <x>:_('%s_plural' % className)</x> 
+       <img src=":url('arrowsA.svg')" class="iconS"
+            style="transform: rotate(270deg)"/><x>:_('search_title')</x>
+      </div>
 
       <!-- Form for searching objects of the requested class -->
       <form name="search" action=":'%s/Search/results' % tool.url"
@@ -596,45 +632,55 @@ class Search:
        <input var="label=_('search_button');
                    css=ui.Button.getCss(label, small=False)"
               type="submit" class=":css" value=":label"
-              style=":url('search', bg=True)"/>
+              style=":url('search.svg', bg='18px 18px')"/>
       </form>
      </x>''', template=Template.px, hook='content')
 
-    # The live search form
-    live = Px('''
-     <form var="formId='%s_LSForm' % className"
-           id=":formId" name=":formId" action=":'%s/Search/results' % toolUrl">
-      <input type="hidden" name="className" value=":className"/>
-      <input type="hidden" name="search" value="customSearch"/>
-      <input type="hidden" name="source" value="searchForm"/>
-      <table var="searchLabel=_('search_button')">
-       <tr valign="bottom">
-        <td style="position: relative">
-         <input type="text" size="14" name="w_searchable" autocomplete="off"
-                id=":'%s_LSinput' % className" class="inputSearch"
-                title=":searchLabel"
-                var="jsCall='onLiveSearchEvent(event, %s, %s, %s)' % \
-                             (q(className), q('auto'), q(toolUrl))"
-                onkeyup=":jsCall" onfocus=":jsCall"
-                onblur=":'onLiveSearchEvent(event, %s, %s)' % \
-                         (q(className), q('hide'))"/>
-         <!-- Dropdown containing live search results -->
-         <div id=":'%s_LSDropdown' % className" class="dropdown liveSearch">
-          <div id=":'%s_LSResults' % className"></div>
-         </div>
-        </td>
-        <td><input type="image" src=":url('search')" title=":searchLabel"/></td>
-       </tr>
-      </table>
-     </form>''',
+    # The live search PX. It requires variables:
+    # --------------------------------------------------------------------------
+    #  className | The name of the class whose instances will be searched
+    # --------------------------------------------------------------------------
+    #    pre     | The prefix used to determine identifiers for tags in use
+    #            | within live search PXs. Indeed, if several live searches are
+    #            | present on the same pages, these identifiers will allow to
+    #            | distinguish them. For example, in the portlet, because there
+    #            | is one live search per root class, pre == className. When
+    #            | called from a Ref field with link='dropdown', pre is of the
+    #            | form <Object IID>_<field name>, for example "666_managers".
+    # --------------------------------------------------------------------------
+    #   liveCss  | The CSS class(es) to assign to the main PX tag
+    # --------------------------------------------------------------------------
 
-     css='''
-       .liveSearch a, .liveSearch a:visited { color: #1f549a }
-       .lsSelected { background-color: #d9d7d9 }
-       .lsNoResult { color: #1f549a }''',
+    live = Px('''
+     <div var="searchLabel=_('search_button')" class=":liveCss">
+      <img src=":url('search.svg')" class="iconM"/>
+      <div style="position: relative">
+       <input type="text" size="14" name="w_searchable" autocomplete="off"
+              id=":'%s_LSinput' % pre" class="inputSearch"
+              title=":searchLabel" placeholder=":searchLabel"
+              var="jsCall='onLiveSearchEvent(event,%s,%s,%s)' % \
+                           (q(pre), q(className), q('auto'))"
+              onkeyup=":jsCall" onfocus=":jsCall"
+              onblur=":'onLiveSearchEvent(event,%s,%s,%s)' % \
+                       (q(pre), q(className), q('hide'))"/>
+       <!-- Dropdown containing live search results -->
+       <div id=":'%s_LSDropdown' % pre" class="dropdown">
+        <div id=":'%s_LSResults' % pre"></div>
+       </div>
+      </div>
+     </div>''',
+
+     css='''.lsSelected { background-color: #d9d7d9 }
+            .lsRef { display: flex }
+            .lsSearch { display: flex; margin-top: 2px }
+            .lsAll { font-size: 80%; font-style: italic }''',
 
      # Live-search-related functions (LS)
      js='''
+       function cleanKeywords(keywords) {
+         return keywords.replace(/['",:]/g, '').trim();
+       }
+
        function detectEventType(event) {
          /* After p_event occurred on a live search input field, must we trigger
             a search (a new char has been added), move up/down within the search
@@ -695,31 +741,48 @@ class Search:
            dropdown.style.display = 'none';}, 400);
        }
 
-       // Function managing an p_event triggered from the LS input field
-       function onLiveSearchEvent(event, class_, action, toolUrl) {
-         var dropdown = document.getElementById(class_ + '_LSDropdown');
+       // Retrieve all parameters for performing a live search
+       function getLSParams(pre, className) {
+         // Retrieve search keywords
+         var keywords = document.getElementById(pre + '_LSinput').value;
+         keywords = cleanKeywords(keywords);
+         if (keywords[keywords.length-1] != '*') keywords = keywords + '*';         
+         // Define base parameters
+         var r = {'className':className, 'pre':pre,
+                  'filters':'searchable:' + keywords};
+         // Add parameters depending on the type of search to run
+         if (pre.indexOf('_') == -1) {
+           // A custom search triggered from a root class live search
+           r['search'] = 'customSearch';
+           r['source'] = 'searchForm';
+         }
+         else {
+           // A live search triggered from a Ref field with link=dropdown
+           r['search'] = pre.split('_').join(',') + ',add';
+         }
+         return r;
+       }
+
+       // Manage an p_event triggered from the LS input field
+       function onLiveSearchEvent(event, pre, class_, action) {
+         var dropdown = document.getElementById(pre + '_LSDropdown');
          if (lsTimeout) clearTimeout(lsTimeout);
          // Hide the dropdown if action is forced to 'hide'
          if (action == 'hide') { hideLSDropdown(dropdown, true); return; }
          // Detect if the dropdown must be shown or hidden
-         var input = document.getElementById(class_ + '_LSinput');
-         if (input.value.length > 2) {
+         var input = document.getElementById(pre + '_LSinput'),
+             keywords = cleanKeywords(input.value || '');
+         if (keywords.length > 2) {
            var eventType = detectEventType(event);
            if (!eventType) return;
            if (eventType == 'hide') { hideLSDropdown(dropdown, false); return;}
            if (eventType == 'go') { gotoLSLink(dropdown); return; }
            if (eventType == 'search') {
              // Trigger an Ajax search and refresh the dropdown content
-             var f = document.getElementById(class_ + '_LSForm'),
-                 formElems = f.elements,
-                 params = {}, param;
-             for (var i=0, len=formElems.length; i<len; i++) {
-               param = formElems.item(i);
-               params[param.name] = param.value;
-             }
+             var params = getLSParams(pre, class_);
              lsTimeout = setTimeout(function() {
-               askAjaxChunk(toolUrl + '/Search/liveResults', 'GET', params,
-                            class_+ '_LSResults');
+               askAjaxChunk(siteUrl + '/tool/Search/liveResults', 'GET', params,
+                            pre + '_LSResults');
                dropdown.style.display = 'block';}, 400);
              }
            else { // Move up or down within results
@@ -727,6 +790,31 @@ class Search:
            }
          }
          else { hideLSDropdown(dropdown, true); }
+       }
+
+       // Get all search results
+       function allLSResults(pre, className) {
+         var params = getLSParams(pre, className);
+         post(siteUrl + '/tool/Search/results', params);
+       }
+
+       // Add the selected object to the initiator Ref field
+       function addLSResult(oid, ifield, id, multi) {
+         var hook = oid + '_' + ifield,
+             url = siteUrl + '/' + oid;
+         if (multi) {
+           // Also get the already selected objects in ifield
+           var sel = document.getElementById(ifield),
+               selected=sel.selectedOptions, ids=[];
+           for (var i=0; i<selected.length; i++) {
+             ids.push(selected[i].value);
+           }
+           if (ids.indexOf(id) == -1) ids.push(id);
+           ids = ids.join(',');
+         }
+         else { ids = id; }
+         // Simulate objects selected via a popup
+         askField(hook, url, 'edit', {'semantics': 'checked', 'selected': ids});
        }''')
 
     # An alias to batch results
