@@ -40,7 +40,7 @@ class Mode:
     pxActions = Px('''
      <table>
       <tr><td for="action in actions"
-            var2="field=action; fieldName=field.name;
+            var2="field=action; fieldName=field.name; layout='query';
                   multi=True">:action.pxRender</td></tr>
      </table>''')
 
@@ -152,21 +152,21 @@ class List(Mode):
            var2="field=column.field" width=":column.width" align=":column.align"
            class=":(field == '_checkboxes') and mode.cbClass or ''">
         <x if="column.header">
-         <img if="field == '_checkboxes'" src=":url('checkall')"
-              class="clickable" title=":_('check_uncheck')"
+         <img if="field == '_checkboxes'" src=":url('checkAll.svg')"
+              class="clickable iconM" title=":_('check_uncheck')"
               onclick=":'toggleAllCbs(%s)' % q(mode.checkboxesId)"/>
          <x if="not column.special">
-          <x>::Px.truncateText(_(field.labelId))</x>
+          <span class="htitle">::Px.truncateText(_(field.labelId))</span>
           <!-- Sort icons -->
           <x if="field.isSortable() and (mode.batch.total &gt; 1)">
            <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'desc')"
                 onclick=":'askBunchSorted(%s, %s, %s)' % \
                           (q(mode.hook), q(field.name), q('asc'))"
-                src=":url('sortDown')" class="clickable"/>
+                src=":url('asc.svg')" class="clickable iconS"/>
            <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'asc')"
                 onclick=":'askBunchSorted(%s, %s, %s)' % \
                           (q(mode.hook), q(field.name), q('desc'))"
-                src=":url('sortUp')" class="clickable"/>
+                src=":url('desc.svg')" class="clickable iconS"/>
           </x>
           <!-- Filter widget -->
           <x var="field,filterPx=field.getFilterPx()"
@@ -177,6 +177,7 @@ class List(Mode):
         </x>
        </th>
       </tr>
+
       <!-- Results -->
       <tr if="not mode.objects">
        <td colspan=":len(mode.columns)+1">::_('query_no_result')</td>
@@ -185,19 +186,24 @@ class List(Mode):
          var2="rowCss=loop.o.odd and 'even' or 'odd';
                @currentNumber=currentNumber + 1">:o.pxResult</x>
      </table>
+
      <!-- The button for selecting objects and closing the popup -->
      <div if="popup and mode.cbShown" align=":dleft">
       <input type="button"
              var="label=_('object_link_many'); css=ui.Button.getCss(label)"
-             value=":label" class=":css" style=":url('linkMany', bg=True)"
+             value=":label" class=":css"
+             style=":url('linkMany.svg', bg='18px 18px')"
              onclick=":uiSearch.initiator.jsSelectMany(\
                    q, mode.sortKey, mode.sortOrder, mode.getFiltersString())"/>
      </div>
+
      <!-- Custom actions -->
      <x var="actions=uiSearch.search.getActions(tool)"
         if="actions and not popup">:mode.pxActions</x>
+
      <!-- Init checkboxes if present -->
      <script if="mode.checkboxes">:'initCbs(%s)' % q(mode.checkboxesId)</script>
+
      <!-- Init field focus and store object IDs in the session storage -->
      <script>:'initFocus(%s); %s;' % (q(mode.hook), mode.store)</script>''')
 
@@ -225,7 +231,7 @@ class List(Mode):
         start = int(req.start or '0')
         self.sortKey = req.sortKey
         self.sortOrder = req.sortOrder or 'asc'
-        self.filters = sutils.getDictFrom(req.filters)
+        self.filters = ui.parseFilters(req)
         # Run the search
         self.batch = search.run(tool.H(), start=start, sortBy=self.sortKey,
           sortOrder=self.sortOrder, filters=self.filters,
@@ -233,7 +239,7 @@ class List(Mode):
         self.batch.hook = self.hook
         self.objects = self.batch.objects
         # Show sub-titles ?
-        self.showSubTitles = req.showSubTitles
+        self.showSubTitles = eval(req.showSubTitles or 'False')
         # Every matched object may be selected via a checkbox
         self.rootHook = ui.getRootHook()
         self.checkboxes = ui.checkboxes
@@ -398,7 +404,7 @@ class Calendar(Mode):
         # Format for representing hours in the UI
         self.hourFormat = '%H:%M'
         # If filters are defined from a list mode, get it
-        self.filters = sutils.getDictFrom(req.get('filters'))
+        self.filters = self.uiSearch.parseFilters(req)
 
     def updateAjaxParameters(self, params):
         '''Grid-specific ajax parameters'''
