@@ -17,46 +17,72 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import re, urllib.parse
+from pathlib import Path
 
 import appy.ui
-from appy.all import *
 from appy.tr import po
 from appy.px import Px
 from appy import Config
+from appy.server import Server
 from appy.model.base import Base
 from appy.model.user import User
 from appy.data import nativeNames
 from appy.utils.dates import Date
 from appy.model.group import Group
+from appy.model.fields import Show
+from appy.database import Database
+from appy.ui.layout import Layouts
 from appy.utils.mail import sendMail
 from appy.ui.template import Template
+from appy.model.fields.ref import Ref
 from appy.model.searches import Search
+from appy.model.fields.rich import Rich
 from appy.model.fields import Initiator
+from appy.model.fields.phase import Page
 from appy.model.page import Page as OPage
 from appy.database.catalog import Catalog
+from appy.model.fields.string import String
+from appy.model.fields.action import Action
 from appy.model.translation import Translation
+from appy.model.fields.calendar import Calendar
+from appy.model.fields.computed import Computed
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Tool(Base):
     '''Base class for the Appy tool, a persistent instance storing configuration
        elements and more, like users, group and translations.'''
 
-    # Make some modules and classes available and/or traversable via the tool- -
+    # Make some modules and classes available and/or traversable via the tool
     ui = appy.ui
     Date = Date
     OPage = OPage
     Search = Search
+    
+    # While most users have generally the read permission on the tool and
+    # translations, they must not be allowed to consult tool/view and sub-pages.
+    # Traversing tool/view is thus by default restricted to those having the
+    # write permission on the tool.
     traverse = Base.traverse.copy()
-    traverse.update({'ui': True, 'guard': True, 'Search': True})
+    traverse.update({'ui':True, 'guard':True, 'Search':True,
+                     'Database':True, 'Server':True, 'view': 'perm:write'})
+
     Initiator = Initiator
+
     # The tool is not indexed by default
     indexable = False
 
+    @staticmethod
+    def update(class_):
+        '''Hide pages being not necessary'''
+        fields = class_.fields
+        for name in ('title', 'record'):
+            fields[name].page.show = False
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Tool initialisation
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    def checkPeers(self, config):
+    def checkServers(self, config):
         '''Log the names of servers (LDAP, mail...) this app wants to connect
            to. Servers declared as "testable" are contacted to check the app's
            connection to it.'''
@@ -74,13 +100,12 @@ class Tool(Base):
                 if status: status = ', ' + status
             servers.append('%s (%s%s)' % (cfg, enabled, status))
         if servers:
-            self.log('server(s) %s configured.' % ', '.join(servers))
+            self.log('Server(s) %s configured.' % ', '.join(servers))
 
     def init(self, handler, poFiles):
         '''Ensure all required tool sub-objects are present and coherent'''
         # Get the root database object from the current request p_handler
-        root = handler.connection.root
-
+        root = handler.dbConnection.root
         # Create the default users if they do not exist
         for login, roles in User.defaultUsers.items():
             # Does this user exist in the database ?
@@ -104,10 +129,17 @@ class Tool(Base):
 
         # Load Appy "po" files if we need to inject their values into
         # Translation objects. app's "po" files are already in p_poFiles.
+        # Also load ext's "po" files if an ext is defined.
         config = handler.server.config
         ui = config.ui
         load = ui.loadTranslationsAtStartup
-        if load: appyFiles = po.load(pot=False, languages=ui.languages)
+        extFiles = None
+        if load:
+            appyFiles = po.load(pot=False, languages=ui.languages)
+            if config.ext:
+                extPath = '%s/tr' % __import__(config.ext).__path__[0]
+                extFiles = po.load(path=Path(extPath), pot=False,
+                                   languages=ui.languages)
 
         # Ensure a Translation object exists for every supported language
         for language in ui.languages:
@@ -116,13 +148,18 @@ class Tool(Base):
                 tr = self.create('translations', secure=False, id=language,
                             title='%s (%s)' % (language, nativeNames[language]),
                             sourceLanguage=ui.sourceLanguage)
-                tr.updateFromFiles(appyFiles, poFiles)
+                tr.updateFromFiles(appyFiles, poFiles, extFiles or {})
             else:
                 tr = root.objects[language]
-                if load: tr.updateFromFiles(appyFiles, poFiles)
+                if load: tr.updateFromFiles(appyFiles, poFiles, extFiles or {})
 
         # Check connected servers
-        self.checkPeers(config)
+        self.checkServers(config)
+
+        # Import an Appy 0 site when relevant
+        peers = config.peers
+        if peers and peers.appy0:
+            peers.appy0.pump(self)
 
         # Call method "onInstall" on the tool when available. This hook allows
         # an app to execute code on server initialisation.
@@ -181,7 +218,8 @@ class Tool(Base):
     #  Page "users"
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    userPage = Page('users', show=pageForToolWriters, label='Tool_page_users')
+    userPage = Page('users', show=pageForToolWriters, label='Tool_page_users',
+                    icon='pageUsers.svg')
 
     # The "main" page, reserved to technical elements, may be hidden by apps
     def getDefaultViewPage(self): return 'users'
@@ -189,9 +227,9 @@ class Tool(Base):
     # Ref(User) will maybe be transformed into Ref(AppUser)
     users = Ref(User, multiplicity=(0,None), add=True, link=False,
       composite=True, back=Ref(attribute='toTool', show=False, layouts='f'),
-      page=userPage, queryable=True, queryFields=('title', 'login', 'roles'),
-      show=forToolWriters, showHeaders=True, actionsDisplay='inline',
-      shownInfo=('title', 'name*15%|', 'login*20%|', 'roles*20%|'), **ta)
+      page=userPage, queryable=True, queryFields=('searchable','login','roles'),
+      show=forToolWriters, showHeaders=True, actionsDisplay='right',
+      shownInfo=User.listColumns, **ta)
 
     def getUserName(self, login=None, normalized=False):
         '''Gets the user name corresponding to p_login (or the currently logged
@@ -205,61 +243,17 @@ class Tool(Base):
             if not user: return login
         return user.getTitle(normalized=normalized)
 
-    def computeConnectedUsers(self, expr=None, context=None):
-        '''Computes a table showing users that are currently connected.
-
-           If p_expr is given, it must be an expression that will be evaluated
-           on every connected user. The user will be part ofthe result only if
-           the expression evaluates to True. The user is given to the expression
-           as variable "user".'''
-        # Get and count connected users
-        users = list(self.H().server.loggedUsers.items())
-        users.sort(key=lambda u: u[1], reverse=True) # Sort by last access date
-        count = len(users)
-        # Prepare the expression's context when relevant
-        if expr:
-            if context is None: context = {}
-        # Compute table rows
-        rows = []
-        for login, access in users:
-            user = self.search1('User', noSecurity=True, login=login)
-            if not user: continue # Could have been deleted in the meanwhile
-            if expr:
-                # Update the p_expr's context with the current user
-                context['user'] = user
-                # Evaluate it
-                if not eval(expr, context): continue
-            rows.append('<tr><td><a href="%s">%s</a></td><td>%s</td></tr>' % \
-                        (user.url, user.title, self.Date.format(access)))
-        # Create an empty row if no user has been found
-        if not rows:
-            rows.append('<tr><td colspan="2" align="center">%s</td></tr>' %
-                        self.translate('no_value'))
-            count = 0
-        else:
-            count = len(rows)
-        # Build the entire table
-        r = '<table cellpadding="0" cellspacing="0" class="list compact">' \
-            '<tr><th>(%d)</th><th>%s</th></tr>' % \
-            (count, self.translate('last_user_access'))
-        return r + '\n'.join(rows) + '</table>'
-
-    connectedUsers = Computed(method=computeConnectedUsers, page=userPage,
-                              layouts=Layouts.n, show='view', **ta)
-
     def doSynchronizeExternalUsers(self):
         '''Synchronizes the local User copies with a distant LDAP user base'''
-        cfg = self.config.security.ldap
+        cfg, context = self.config.security.getLdap()
         if not cfg: raise Exception('LDAP config not found.')
-        counts = cfg.synchronizeUsers(self)
-        msg = 'LDAP users: %d created, %d updated, %d untouched, ' \
-              '%d invalid entries.' % counts
-        return True, msg
+        message = cfg.synchronizeUsers(self, sso=context)
+        return True, message
 
     def showSynchronizeUsers(self):
         '''Show this button only if a LDAP connection exists and is enabled'''
-        cfg = self.config.security.ldap
-        if cfg and cfg.enabled: return 'view'
+        cfg, context = self.config.security.getLdap()
+        if cfg and cfg.enabled: return 'buttons'
 
     synchronizeExternalUsers = Action(action=doSynchronizeExternalUsers,
       show=showSynchronizeUsers, confirm=True, page=userPage, **ta)
@@ -271,7 +265,8 @@ class Tool(Base):
     # Ref(Group) will maybe be transformed into Ref(AppGroup)
     groups = Ref(Group, multiplicity=(0,None), add=True, link=False,
       composite=True, back=Ref(attribute='toTool', show=False,layouts='f'),
-      page=Page('groups', show=pageForToolWriters, label='Tool_page_groups'),
+      page=Page('groups', show=pageForToolWriters, label='Tool_page_groups',
+                icon='groups.svg'),
       show=forToolWriters, queryable=True, queryFields=('title', 'login'),
       showHeaders=True, actionsDisplay='inline',
       shownInfo=('title', 'login*20%|', 'roles*20%|'), **ta)
@@ -281,7 +276,7 @@ class Tool(Base):
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
     pt = Page('translations', show=pageForToolWriters,
-              label='Tool_page_translations')
+              label='Tool_page_translations', icon='pageTranslations.svg')
 
     # Ref(Translation) will maybe be transformed into Ref(AppTranslation)
     translations = Ref(Translation, multiplicity=(0,None), add=False,
@@ -292,14 +287,49 @@ class Tool(Base):
     #  Page "pages"
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    pp = Page('pages', show=lambda o: o.allows('write'),
-              label='Tool_page_pages')
+    def showRootPages(self):
+        '''Show root pages if the logged user is allowed to write the tool'''
+        # The page may be edited only if there is at least one page defined (in
+        # order to select it as default page).
+        if not self.allows('write'): return
+        return 'view' if self.isEmpty('pages') else True
+
+    pp = Page('pages', show=showRootPages, label='Tool_page_pages',
+              icon='pagePages.svg')
 
     pages = Ref(OPage, multiplicity=(0,None), add=True, link=False,
-      composite=True, show='view', actionsDisplay='inline', page=pp,
-      back=Ref(attribute='toTool', show=False, layouts='f'), **ta)
+                composite=True, show=Show.VX, actionsDisplay='inline', page=pp,
+                back=Ref(attribute='toTool', show=False, layouts='f'),
+                showHeaders=True, **ta)
 
-    homeText = Rich(page=pp, **ta)
+    # Return the visible root pages
+    def getRootPages(self): return OPage.getRoot(self)
+
+    # The page, among "pages", being defined as the default one
+    defaultPage = Ref(OPage, add=False, link=True, render='links', page=pp,
+                      back=Ref(attribute='toTool2', show=False, layouts='f'),
+                      show=lambda o: not o.isEmpty('pages'),
+                      select=lambda o: o.pages, **ta)
+
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+    #  "admin" zone
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+
+    def forAdmin(self):
+        '''Returns page visibility for pages from the admin zone'''
+        return 'view' if self.user.login == 'admin' else None
+
+    onlineUsers = Computed(method=lambda o: User.getOnline(o), layouts='f',
+      show='view', page=Page('online', phase='admin', show=forAdmin,
+                             label='Tool_page_online'), **ta)
+
+    serverInfo = Computed(method=Server.view, layouts='f',
+      page=Page('server', phase='admin', show=forAdmin,
+                label='Tool_page_server'), **ta)
+
+    databaseInfo = Computed(method=Database.view, layouts='f',
+      page=Page('database', phase='admin', show=forAdmin,
+                label='Tool_page_database'), **ta)
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Main methods
@@ -317,30 +347,19 @@ class Tool(Base):
         isAnon = user.isAnon()
         if isAnon and self.config.ui.discreetLogin:
             return '%s/public' % self.url
-        # If the user must change its password, redirect him to this page
-        if user.changePasswordAtNextLogin:
-            return '%s/edit?page=passwords' % user.url
         # If the app defines a tool method names "getHomePage", call it
         r = None
         if hasattr(self, 'getHomePage'):
             r = self.getHomePage()
         if not r:
-            # Bring anonymous users to the site root, Managers to tool/view and
+            # Bring anonymous users to tool/home, Managers to tool/view and
             # others to user/home.
             if isAnon:
-                r = self.siteUrl
+                r = '%s/home' % self.url
             elif user.hasRole('Manager'):
                 r = '%s/view' % self.url
             else:
                 r = user.url
-        # Did the user come to its home page for authenticating, and must he,
-        # afterwards, be redirected to some other page ?
-        goto = self.req.goto
-        if goto:
-            goto = urllib.parse.quote(goto)
-            sep = '&' if '?' in r else '?'
-            r += '%sgoto=%s' % goto
-            self.say(self.translate('please_authenticate'))
         return r
 
     def computeHomeObject(self, user, popup=False):
@@ -422,14 +441,14 @@ class Tool(Base):
     # when discreet logins are enabled. In this latter case, PX "public",
     # hereafter, is more appropriate.
     home = Px('''
-     <!-- If discreet logins are enabled, redirect the user to the alternative
-          "public" page. -->
-     <x if="cfg.discreetLogin" var2="x=tool.goto(tool.computeHomePage())"></x>
+     <!-- Redirect the user when appropriate -->
+     <x if="not isAnon or cfg.discreetLogin"
+        var2="x=tool.goto(tool.computeHomePage())"></x>
 
      <!-- The home logo and text -->
      <div if="isAnon and not cfg.discreetLogin" class="homeText">
        <img if="cfg.homeLogo" src=":cfg.getImageUrl(siteUrl, 'homeLogo')"/>
-       <x if="not tool.isEmpty('homeText')">::tool.homeText</x>
+       <x if="not tool.isEmpty('defaultPage')">::tool.defaultPage.content</x>
      </div>''',
 
      css='''
@@ -448,8 +467,10 @@ class Tool(Base):
     # discreet logins are enabled.
     public = Px('''
      <!-- The home text -->
-     <div if="not tool.isEmpty('homeText')"
-          class="homeText">::tool.homeText</div>''',
+     <div var="default=tool.defaultPage" class="xhtml">
+      <x if="default">::default.content</x>
+      <x if="not default">::_('no_default_page')</x>
+     </div>''',
      template=Template.px, hook='content', name='public')
 
     # Disable tool removal
