@@ -16,12 +16,24 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-from appy.all import *
 from appy.px import Px
 from appy.model.base import Base
+from appy.model.fields import Show
+from appy.ui.layout import Layouts
+from appy.model.searches import Search
+from appy.database.operators import or_
 from appy.utils import string as sutils
+from appy.model.fields.date import Date
 from appy.utils.string import Normalize
+from appy.model.fields.phase import Page
+from appy.model.fields.group import Group
+from appy.model.fields.string import String
+from appy.model.fields.action import Action
+from appy.model.fields.boolean import Boolean
+from appy.model.fields.computed import Computed
+from appy.model.fields.password import Password
 from appy.model.workflow import standard as workflow
+from appy.model.fields.select import Select, Selection
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class User(Base):
@@ -29,8 +41,9 @@ class User(Base):
     workflow = workflow.Owner
     noLoginUsers = ('system', 'anon')
     specialUsers = noLoginUsers + ('admin',)
-    layouts = Layouts.Page.summary
     searchAdvanced = Search('advanced', actionsDisplay='inline')
+    listColumns = ('title', 'name*15%|', 'login*20%|', 'roles*20%|',
+                   'state*10%|')
 
     # The following users are always present in any Appy database
     defaultUsers = {'admin': ('Manager',), 'system': ('Manager',), 'anon': ()}
@@ -47,7 +60,9 @@ class User(Base):
     @staticmethod
     def update(class_):
         '''Hide the title'''
-        class_.fields['title'].show = 'xml'
+        title = class_.fields['title']
+        title.show = 'xml'
+        title.page.icon = 'pageProfile.svg'
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Password
@@ -67,25 +82,28 @@ class User(Base):
            forced to change its password.'''
         return not self.changePasswordAtNextLogin
 
-    pagePassword = Page('password', showNext=False, show=showPassword,
-                        showCancel=showCancel, label='User_page_password')
+    pagePassword = Page('password', show=showPassword, showCancel=showCancel,
+                        label='User_page_password', icon='pagePassword.svg')
 
     password = Password(multiplicity=(1,1), show=showPassword, label='User',
                         page=pagePassword)
 
-    def checkPassword(self, clearPassword):
-        '''Returns True if p_clearPassword is the correct password for this
-           user.'''
-        return True
-        # Get the stored encrypted password
+    # The encrypted password, carriable via the XML layout
+    def showEncrypted(self):
+        '''Allow to show encrypted password to admins only, via the xml
+           layout.'''
+        if self.user.login == 'admin' and (self.source != 'sso'): return 'xml'
+
+    encrypted = Computed(show=showEncrypted,
+                         method=lambda o: o.values['password'])
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Title, name, first name
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    pm = {'page': Page('main', showPrevious=False, label='User_page_main'),
+    pm = {'page': Page('main', label='User_page_main', icon='pageProfile.svg'),
           'width': 28, 'layouts': Layouts.g, 'label': 'User',
-          'group': Group('main', style='grid', label='User_group')}
+          'group': Group('main', style='grid', hasLabel=False)}
 
     def getTitle(self, normalized=False, useName=True, nameFirst=False,
                  nameTransform=None, firstNameTransform=None):
@@ -234,13 +252,13 @@ class User(Base):
     pm['label'] = 'User'
     email = String(show=showEmail, **pm)
 
-    def getMailRecipient(self):
+    def getMailRecipient(self, normalized=True):
         '''Returns, for this user, the "recipient string" (first name, name,
            email) as can be used for sending an email.'''
         r = self.email or self.login
         # Ensure this is really an email
         if not String.EMAIL.match(r): return
-        return '%s <%s>' % (self.getTitle(), r)
+        return '%s <%s>' % (self.getTitle(normalized=normalized), r)
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Roles and permissions
@@ -248,11 +266,16 @@ class User(Base):
 
     pm['multiplicity'] = (0, None)
     del pm['width']
-    def showRoles(tool):
-        '''Only the admin can edit roles'''
-        user = tool.user
+
+    def showRoles(self):
+        '''Global roles are not visible or editable under all circumstances'''
+        # Local users can't be granted global roles
+        if self.container != self.tool: return
+        # Only a Manager is allowed to grant global roles
+        user = self.user
         if user.hasRole('Manager'): return True
-        if user.hasRole('Owner', user): return Show.E_
+        # The user itself, owner of himself, can consult his roles
+        if user.hasRole('Owner', self): return Show.E_
 
     roles = Select(show=showRoles, indexed=True, width=40, height=10,
                    validator=Selection(lambda o: o.model.getGrantableRoles(o)),
@@ -311,7 +334,7 @@ class User(Base):
         # What are the roles which are granted p_permission on p_o ?
         allowedRoles = o.getWorkflow().getRolesFor(o, permission)
         # Grant access based on global user roles (that include ungrantable
-        # roles like Authenticated or Anonymous), , excepted if p_o is in
+        # roles like Authenticated or Anonymous), excepted if p_o is in
         # "local" mode.
         if not o.localRoles.only:
             for role in self.getRoles():
@@ -331,7 +354,7 @@ class User(Base):
 
     # Where is this user stored ? By default, in the ZODB. But the user can be
     # stored in an external LDAP (source='ldap').
-    source = String(show='xml', default='zodb', layouts='f')
+    source = String(show='xml', default='zodb', layouts='f', label='User')
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Actions
@@ -361,13 +384,14 @@ class User(Base):
             return 'buttons'
 
     resetPassword = Action(action=doResetPassword, show=showResetPassword,
-                          confirm=True, label='User', icon='pwd', render='icon')
+                           confirm=True, label='User', icon='pagePassword.svg',
+                           sicon='password.svg', render='icon')
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Hidden fields
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    hidden = {'show': False, 'layouts': 'f'}
+    hidden = {'show': False, 'layouts': 'f', 'label': 'User'}
     # For external users (source != "zodb"), we store the date of the last time
     # the external user and the local copy were synchronized.
     syncDate = Date(format=Date.WITH_HOUR, **hidden)
@@ -423,8 +447,10 @@ class User(Base):
 
     # Display, in the user strip, links to the User instance of the logged user
     pxUserLink = Px('''
-     <a if="config.ui.userLink" href=":user.url"><img src=":url('user')"/>
-      <span class="headerText">:user.getFirstName()</span></a>
+     <a if="config.ui.userLink" href=":user.url">
+      <img src=":url('user.svg')" class="icon"/>
+      <span class="headerText">:user.getFirstName()</span>
+     </a>
      <x if="not config.ui.userLink">:user.getFirstName()</x>
      <x var="ctx=config.security.authContext" if="ctx">:ctx.pxLogged</x>''')
 
@@ -485,7 +511,7 @@ class User(Base):
             ctx = req.context
             if login: place = 'form'
         # d. Identify the user from a SSO reverse proxy
-        if not login:
+        if not login or (login == '_s_s_o_'):
             sso = handler.config.security.sso
             if sso:
                 login = sso.extractUserLogin(guard)
@@ -507,7 +533,7 @@ class User(Base):
             # (on database initialization). This is why we may create here a
             # fake one.
             if hasattr(handler, 'connection'):
-                user = handler.connection.root.objects.get('system') or \
+                user = handler.dbConnection.root.objects.get('system') or \
                        User.System()
             else:
                 user = User.System()
@@ -540,13 +566,52 @@ class User(Base):
                 config.authContext.isMandatory(tool) and (place != 'basic')):
                 # Disable the authentication cookie
                 guard.Cookie.disable(handler)
-                user = handler.connection.root.objects.get('anon')
+                user = handler.dbConnection.root.objects.get('anon')
             else:
                 # The user is authenticated. Create an authentication cookie
                 # for him.
                 guard.Cookie.write(handler, user.login, password, ctx)
         return user
 
+    @classmethod
+    def getOnline(class_, tool, expr=None, context=None):
+        '''Returns a XHTML table showing users being currently connected'''
+        # If p_expr is given, it must be an expression that will be evaluated on
+        # every connected user. The user will be part ofthe result only if the
+        # expression evaluates to True. The user is given to the expression as
+        # variable "user".
+        # ~~~
+        # Get and count connected users
+        users = list(tool.H().onlineUsers.items())
+        users.sort(key=lambda u: u[1], reverse=True) # Sort by last access date
+        count = len(users)
+        # Prepare the expression's context when relevant
+        if expr:
+            if context is None: context = {}
+        # Compute table rows
+        rows = []
+        for login, access in users:
+            user = tool.search1('User', login=login)
+            if not user: continue # Could have been deleted in the meanwhile
+            if expr:
+                # Update the p_expr's context with the current user
+                context['user'] = user
+                # Evaluate it
+                if not eval(expr, context): continue
+            rows.append('<tr><td><a href="%s">%s</a></td><td>%s</td></tr>' % \
+                        (user.url, user.title, tool.formatDate(access)))
+        # Create an empty row if no user has been found
+        if not rows:
+            rows.append('<tr><td colspan="2" align="center">%s</td></tr>' %
+                        tool.translate('no_value'))
+            count = 0
+        else:
+            count = len(rows)
+        # Build the entire table
+        r = '<table class="list"><tr><th>(%d)</th><th>%s</th></tr>' % \
+            (count, tool.translate('last_user_access'))
+        return r + '\n'.join(rows) + '</table>'
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Appy methods
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
