@@ -113,7 +113,7 @@ class Event(persistent.Persistent):
     pxDetail = Px('''
      <x var="eventId=event.getId()">
       <img if="event.mayEditComment(config, user, isManager)"
-           class="clickable" style="float:right" src=":url('edit')"
+           class="clickable iconS" style="float:right" src=":url('edit.svg')"
            title=":_('object_edit')"
            onclick=":'onHistoryEvent(%s,%s,%s,%s)' % \
                     (q('Edit'), q(o.iid), q(event.date), q(eventId))"/>
@@ -187,7 +187,8 @@ class Trigger(Event):
         if self.transition == '_init_':
             r = 'Base_creator'
         else:
-            r = o.getWorkflow().transitions.get(self.transition).labelId 
+            transition = o.getWorkflow().transitions.get(self.transition)
+            r = transition.labelId if transition else self.transition
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -205,7 +206,7 @@ class Action(Event):
 
     def getLabel(self, o):
         '''Returns the label for the corresponding action'''
-        return o.fields.get(self.action).labelId
+        return o.getField(self.action).labelId
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Change(Event):
@@ -233,7 +234,7 @@ class Change(Event):
                 field=o.getField(fname)">
        <td><x>::_(field.labelId)</x>
            <x if="lg">:' (%s)' % ui.Language.getName(lg)</x></td>
-       <td>::change[1][0]</td>
+       <td>::value</td>
       </tr>
      </table>''')
 
@@ -323,8 +324,10 @@ class History(PersistentList):
                isManager=user.hasRole('Manager')"
           if="batch.length" id=":batch.hook">
       <script>:history.getAjaxData(batch)</script>
+
       <!-- Navigate between history pages -->
-      <x>:batch.pxNavigate</x>
+      <div align=":dright">:batch.pxNavigate</div>
+
       <!-- History -->
       <table width="100%" class="history">
        <tr>
@@ -336,8 +339,8 @@ class History(PersistentList):
        <tr for="event in batch.objects"
            class=":loop.event.odd and 'even' or 'odd'" valign="top">
         <td><x>:_(event.getLabel(o))</x>
-         <img if="isManager and event.deletable" class="clickable"
-              src=":url('delete')"
+         <img if="isManager and event.deletable" class="clickable iconS"
+              src=":url('deleteS.svg')"
               onclick=":'onHistoryEvent(%s,%s,%s)' % \
                         (q('Delete'), q(o.iid), q(event.date))"/></td>
         <td var="creator=o.search1('User', login=event.login)">
@@ -350,19 +353,10 @@ class History(PersistentList):
 
     view = Px('''
      <div if="not o.isTemp()"
-       var2="history=o.history;
-             hasHistory=not history.isEmpty();
-             collapsible=collapsible|True;
-             collapse=ui.Collapsible('objectHistory', req) \
-                      if collapsible else None">
+       var2="history=o.history; hasHistory=not history.isEmpty()">
       <table width="100%" class="header" cellpadding="0" cellspacing="0">
        <tr>
         <td colspan="2" class="by">
-         <!-- Plus/minus icon for accessing history -->
-         <x if="hasHistory and collapsible"><x>:collapse.px</x>
-          <x>:_('object_history')</x> &mdash; 
-         </x>
-
          <!-- Creator and last modification date -->
          <x>:_('Base_creator')</x> 
           <x>:user.getTitleFromLogin(o.creator)</x> 
@@ -384,14 +378,20 @@ class History(PersistentList):
        </tr>
 
        <!-- History entries -->
-       <tr if="hasHistory">
-        <td colspan="2">
-         <span id=":collapse.id"
-          style=":collapsible and collapse.style or ''">:history.events</span>
-        </td>
-       </tr>
+       <tr if="hasHistory"><td colspan="2">:history.events</td></tr>
       </table>
-     </div>''')
+     </div>''',
+
+     css='''
+      .header { margin-bottom: 5px; background-color: #f3f3f7;
+                border: 3px solid white }
+      .by { padding: 7px }
+      .history td { padding: 8px }
+      .history th { font-style: italic; text-align: left; padding: 10px 5px;
+                    background-color: white }
+      .changes { margin: 4px 0 }
+      .changes td, .changes th { border: 1px solid #e0e0e0 }
+     ''')
 
     def __init__(self, o):
         PersistentList.__init__(self)
@@ -400,18 +400,19 @@ class History(PersistentList):
         # The last time the object has been modified
         self.modified = None
 
-    def show(self):
-        '''May the user view history ?'''
-        # Method "showHistory" can be defined on the objet
-        class_ = self.o.class_.python
-        show = getattr(class_, 'showHistory', None)
-        return show(self.o) if show and callable(show) else show
-
     def add(self, type, state=None, **params):
-        ''''Adds a new event of p_type ("Trigger", "Action" or "Change") into
+        ''''Adds a new event of p_type (=the name one an Event sub-class into
             the history.'''
         # Get the login of the user performing the action
-        login = self.o.user.login
+        if 'login' in params:
+            login = params.pop('login')
+        else:
+            login = self.o.user.login
+        # Get the date and time of the action
+        if 'date' in params:
+            date = params.pop('date')
+        else:
+            date = DateTime()
         # For a trigger event, p_state is the new object state after the
         # transition has been triggered. p_state is a name (string) and not a
         # State instance. The name of the triggering transition must be in
@@ -422,7 +423,7 @@ class History(PersistentList):
         # store the object's current state.
         state = state or self[0].state
         # Create the event
-        event = eval(type)(login, state, DateTime(), **params)
+        event = eval(type)(login, state, date, **params)
         # Insert it at the first place within the anti-chronological list
         self.insert(0, event)
         # Initialise self.modified if still None
@@ -439,10 +440,8 @@ class History(PersistentList):
         # An history containing a single entry is considered empty: this is the
         # special _init_ virtual transition representing the object creation.
         if len(self) == 1: return True
-        # Return False if the user can't consult the history
-        if not self.show(): return
         # At this point, the complete history can be considered not empty
-        if name is None: return True
+        if name is None: return
         # Check if history is available for field named p_name
         empty = True
         for event in self.iter(eventType='Change', \
@@ -471,18 +470,14 @@ class History(PersistentList):
            values.'''
         o = self.o
         # Remove, from p_previous, any value that was not changed
-        for name, prev in previous.items():
+        for name in list(previous.keys()):
+            prev = previous[name]
             field = o.getField(name)
             curr = field.getValue(o)
-            try:
-                if (prev == curr) or ((prev is None) and (curr == '')) or \
-                   ((prev == '') and (curr is None)):
-                    del(previous[name])
-                    continue
-            except UnicodeDecodeError:
-                # The string comparisons above may imply silent encoding-related
-                # conversions that may produce this exception.
-                continue
+            if (prev == curr) or ((prev is None) and (curr == '')) or \
+               ((prev == '') and (curr is None)):
+                del(previous[name])
+                continue            
             # The previous value may need to be formatted
             field.updateHistoryValue(o, previous)
         # Add the entry in the history (if not empty)
@@ -501,9 +496,9 @@ class History(PersistentList):
         '''Returns the Batch instance allowing to navigate within p_o's
            history.'''
         # Compute the batch start and size
-        size = int(size) if isinstance(size, str) else (size or 5)
+        size = int(size) if isinstance(size, str) else (size or 30)
         start = int(o.req.start or 0)
-        return Batch(objects=self[start:start + size], total=len(self),
+        return Batch(objects=self.data[start:start + size], total=len(self),
                      size=size, start=start, hook='history')
 
     def getAjaxData(self, batch):
