@@ -17,6 +17,7 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
+from appy.utils import iconParts
 from appy.model.fields.group import Group
 from appy.model.workflow.state import State
 from appy.model.workflow import emptyDict, Role
@@ -31,8 +32,8 @@ class Transition:
     traverse = {}
 
     def __init__(self, states, condition=True, action=None, show=True,
-                 confirm=False, group=None, icon=None, redirect=None,
-                 historizeActionMessage=False):
+                 confirm=False, group=None, icon=None, sicon=None,
+                 redirect=None, historizeActionMessage=False):
         # In its simpler form, "states" is a list of 2 states:
         # (fromState, toState). But it can also be a list of several
         # (fromState, toState) sub-lists. This way, you may define only 1
@@ -48,8 +49,14 @@ class Transition:
         # the transition. It will only be possible by code.
         self.confirm = confirm # If True, a confirm popup will show up.
         self.group = Group.get(group)
-        # The user may specify a specific icon to show for this transition.
-        self.icon = icon or 'transition'
+        # If you want to specify an icon being in folder <yourApp>/static,
+        # specify "<yourApp>/<yourImage>". Else, default icon "appy/action.svg"
+        # will be used.
+        self.icon, self.iconBase = iconParts(icon or 'action.svg')
+        # You may specify, in attribute "sicon", an alternate icon suitable when
+        # rendered as a small icon.
+        sicon = sicon or icon or 'action.svg'
+        self.sicon, self.siconBase = iconParts(sicon)
         # If redirect is None, once the transition will be triggered, Appy will
         # perform an automatic redirect:
         # (a) if you were on some "view" page, Appy will redirect you to this
@@ -224,7 +231,7 @@ class Transition:
                     found = True
                     break
             if not found: return
-        # Check that the condition is met, excepted if noSecurity is True
+        # Check that the condition is met, excepted if secure is False
         if not secure: return True
         user = o.user
         # We will need the workflow's prototypical instance
@@ -293,14 +300,15 @@ class Transition:
            dict containing custom data that will be integrated into the history
            event.'''
         # Is that the special _init_ transition ?
-        isInit = self.name == '_init_'
+        name = self.name
+        isInit = name == '_init_'
         # "Triggerability" and security checks
         if not isInit and not self.isTriggerable(o, secure=secure):
             raise Transition.Error(UNTRIGGERABLE % (name, o.url))
         # Identify the target state for this transition
         target = forceTarget or self.getTargetState(o)
         # Add the event in the object history
-        event = o.history.add('Trigger', target.name, transition=self.name,
+        event = o.history.add('Trigger', target.name, transition=name,
                               comment=comment)
         # Remember the source state, it will be necessary for executing the
         # common action.
@@ -310,7 +318,7 @@ class Transition:
         # action is executed before the transition-specific action (if any).
         proto = self.workflow.proto
         if doAction and hasattr(proto, 'onTrigger'):
-            proto.onTrigger(o, self.name, fromState)
+            proto.onTrigger(o, name, fromState)
         # Execute the transition-specific action
         msg = self.executeAction(o) if doAction and self.action else None
         # Append the action message to the history comment when relevant
@@ -384,36 +392,58 @@ class UiTransition:
     '''Represents a widget that displays a transition'''
     px = Px('''
      <x var="label=transition.title;
-             inButtons=layout == 'buttons';
-             css=ui.Button.getCss(label, inButtons)">
+             mayTrigger=transition.mayTrigger;
+             inButtons=layout == 'buttons'">
+
+      <!-- Picto -->
+      <a if="not inButtons" class=":'clickable' if mayTrigger else 'help fake'"
+         var="back=transition.getBackHook(o, inButtons, q, backHook)"
+         name=":transition.name"
+         title=":'' if mayTrigger else transition.reason"
+         onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
+                    (q(formId), q(transition.confirm), back) \
+                    if mayTrigger else ''">
+       <img src=":url(transition.icon, base=transition.iconBase)"
+            class=":picto"/><br/>
+       <x>:label</x>
+      </a>
 
       <!-- Real button -->
-      <input if="transition.mayTrigger" type="button" class=":css"
-             var="back=transition.getBackHook(o, inButtons, q, backHook)"
-             name=":transition.name" style=":url(transition.icon, bg=True)"
-             value=":label"
-             onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
-                        (q(formId), q(transition.confirm), back)"/>
+      <x if="inButtons" var2="css=ui.Button.getCss(label, inButtons)">
+       <input if="mayTrigger" type="button" class=":css" value=":label"
+              var="back=transition.getBackHook(o, inButtons, q, backHook)"
+              name=":transition.name"
+              style=":url(transition.sicon, base=transition.siconBase, \
+                          bg='18px 18px')"
+              onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
+                         (q(formId), q(transition.confirm), back)"/>
 
-      <!-- Fake button, explaining why the transition can't be triggered -->
-      <input if="not transition.mayTrigger" type="button"
-             class=":'fake %s' % css" style=":url('fake', bg=True)"
-             value=":label" title=":transition.reason"/></x>''',
+       <!-- Fake button, explaining why the transition can't be triggered -->
+       <input if="not mayTrigger" type="button"
+              class=":'%s fake' % css"
+              style=":url(transition.icon, base=transition.siconBase, \
+                      bg='18px 18px')"
+              value=":label" title=":transition.reason"/></x></x>''',
 
      js='''
       // Function used for triggering a workflow transition
       function triggerTransition(formId, transition, msg, back) {
         var f = document.getElementById(formId);
-        f.action = f.baseUrl + '/' + transition + '/fire';
+        f.action = f.dataset.baseurl + '/' + transition + '/fire';
         submitForm(formId, msg, true, back);
       }''')
 
+    # Fields to copy from Transition to UiTransition
+    copied = ('name', 'icon', 'iconBase', 'sicon', 'siconBase')
+
     def __init__(self, transition, o, mayTrigger):
         _ = o.translate
-        self.name = transition.name
+        # The tied p_transition
         self.transition = transition
+        # Copy p_transitions fields here
+        for name in UiTransition.copied:
+            setattr(self, name, getattr(transition, name))
         self.type = 'transition'
-        self.icon = transition.icon
         labelId = transition.labelId
         self.title = _(labelId)
         if transition.confirm:
