@@ -28,15 +28,11 @@ from appy.pod.odf_parser import OdfEnvironment
 from appy.utils.path import getOsTempFolder, getTempFileName
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FILE_NOT_FOUND = "'%s' does not exist or is not a file."
+FILE_NOT_FOUND   = "'%s' does not exist or is not a file."
 PDF_TO_IMG_ERROR = 'A PDF file could not be converted into images. Please ' \
                    'ensure that Ghostscript (gs) is installed on your ' \
                    'system and the "gs" program is in the path.'
-CONVERT_ERROR = 'Program "convert" (imagemagick) must be installed and ' \
-  'in the path for performing this operation. If you wanted to convert a ' \
-  'SVG file into a PNG file, conversion of SVG files must also be enabled. ' \
-  'On Ubuntu: apt-get install librsvg2-bin'
-TO_PDF_ERROR = 'ConvertImporter error while converting a doc to PDF: %s.'
+TO_PDF_ERROR     = 'ConvertImporter error while converting a doc to PDF: %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def getUuid(removeDots=False):
@@ -59,7 +55,7 @@ class DocImporter:
         self.format = format
         if at:
             self.at = self.checkAt(at)
-        self.res = u''
+        self.res = ''
         self.ns = renderer.currentParser.env.namespaces
         # Unpack some useful namespaces
         self.textNs = self.ns[OdfEnvironment.NS_TEXT]
@@ -350,13 +346,13 @@ class Image:
        and size.'''
     jpgTypes = ('jpg', 'jpeg')
 
-    def __init__(self, path, format):
+    def __init__(self, path, format, unit='cm'):
         self.path = path # The image absolute path on disk
         self.format = format
         # Determine image size in pixels (again, by reading its first bytes)
-        self.width, self.height = self.getSizeInPx()
+        self.width, self.height = self.getSizeIn(unit)
 
-    def getSizeInPx(self):
+    def getSizeIn(self, unit='cm'):
         '''Reads the first bytes from the image on disk to get its size'''
         x = y = None
         # Get the file format from the file name of absent
@@ -393,47 +389,29 @@ class Image:
                 # else: invalid/corrupted GIF (bad header)
                 x, y, u = struct.unpack("<HHB", buf)
         f.close()
-        if x and y:
+        if (x and y) and (unit == 'cm'):
             return float(x)/px2cm, float(y)/px2cm
         else:
+            # Return the size, in pixels, or as a tuple (None, None) if
+            # uncomputable.
             return x, y
 
-# Compute size of images - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class ImageImporter(DocImporter):
     '''This class allows to import into the ODT result an image stored
        externally.'''
+
+    # Ways to anchor images in an ODF document
     anchorTypes = ('page', 'paragraph', 'char', 'as-char')
     WRONG_ANCHOR = 'Wrong anchor. Valid values for anchors are: %s.'
+
+    # The folder, within an unzipped ODF file, containing images
     pictFolder = '%sPictures%s' % (os.sep, os.sep)
+
     # Path of a replacement image, to use when the image to import is not found
     imageNotFound = os.path.join(os.path.dirname(appy.pod.__file__),
                                  'imageNotFound.jpg')
 
-    def getZopeImage(self, at):
-        '''Gets the Zope Image via an image resolver'''
-        resolver = self.renderer.imageResolver
-        if not resolver or not at.startswith(resolver.absolute_url()): return
-        # The resolver is a Zope application or a sub-object within it. From it,
-        # we will retrieve the object on which the image is stored and get the
-        # file to download.
-        urlParts = urllib.parse.urlsplit(at)
-        path = urlParts[2][1:].split('/')
-        image = None
-        try:
-            image = resolver.unrestrictedTraverse(path)
-        except Exception:
-            # Maybe a rewrite rule as added some prefix to all URLs ?
-            try:
-                path = path[1:]
-                # Make sure we still have a path, or it gets the resolver
-                if path:
-                    image = resolver.unrestrictedTraverse(path)
-            except Exception:
-                pass # The image was not found
-        # Return it only if it is an image
-        if image and not hasattr(image, 'getBlobWrapper'): image = None
-        return image
-
     def isImage(self, response):
         '''Cheks that the p_response received after a HTTP GET supposed to
            retrieve an image actually contains an image.'''
@@ -450,24 +428,21 @@ class ImageImporter(DocImporter):
            existing file. We will dump a replacement image instead.'''
         at = DocImporter.checkAt(self, at, raiseOnError=False)
         if at.startswith('http'):
-            # Try to get the image
+            # Try to get the image via the "findImage" function
+            fun = self.renderer.findImage
+            if fun:
+                path = fun(at)
+                if path: return path
+            # Try to get the image via a HTTP request
             try:
-                response = Resource(at).get(followRedirect=False)
-            except Resource.Error:
+                response = Resource(at).get(at, followRedirect=False)
+            except Resource.Error as err:
                 response = None # Can't get the distant image
             if self.isImage(response):
                 # Remember the response
                 self.httpResponse = response
                 return at
-            # The HTTP GET did not work, maybe for security reasons (we probably
-            # have no permission to get the file). But maybe the URL was a local
-            # one, from an application server running this POD code. In this
-            # case, if an image resolver has been given to POD, use it to
-            # retrieve the image.
-            self.zopeImage = self.getZopeImage(at)
-            if self.zopeImage:
-                return at
-            # We could not find the image
+            # If we are here, the image could not be found
             self.format = 'jpg'
             return self.imageNotFound
         elif at.startswith('data:'):
@@ -525,19 +500,6 @@ class ImageImporter(DocImporter):
                 f.write(response.body)
                 f.close()
                 return importPath
-        # The image has (maybe) been retrieved from Zope
-        zopeImage = getattr(self, 'zopeImage', None)
-        if zopeImage:
-            blobWrapper = zopeImage.getBlobWrapper()
-            self.format = utils.mimeTypesExts[blobWrapper.content_type]
-            importPath += self.format
-            blob = blobWrapper.getBlob()
-            # If we do not check 'readers', the blob._p_blob_committed is
-            # sometimes None.
-            blob.readers
-            blobPath = blob._p_blob_committed
-            shutil.copy(blobPath, importPath)
-            return importPath
 
     def init(self, anchor, wrapInPara, size, sizeUnit, maxWidth, cssAttrs,
              keepRatio, convertOptions):
@@ -573,11 +535,7 @@ class ImageImporter(DocImporter):
                 image = Image(self.importPath, self.format)
                 options = self.convertOptions(image)
             if options:
-                # Ensure we have the right to modify the file@self.importPath
-                cmd = ['convert', self.importPath] + options.split() + \
-                      [self.importPath]
-                out, err = utils.executeCommand(cmd)
-                if err: raise Exception(CONVERT_ERROR)
+                utils.convert(self.importPath, options)
                 transformed = True
         # Avoid creating an Image instance twice if no transformation occurred
         if image and not transformed:
@@ -618,16 +576,17 @@ class ImageImporter(DocImporter):
                     # Compute height and width ratios and apply the minimum
                     # ratio to both width and height.
                     ratio = min(w/width, h/height)
-                    width = width * ratio
-                    height = height * ratio
+                    width *= ratio
+                    height *= ratio
         if not width or not height: return ''
         # Take care of max width if specified
         maxWidth = self.maxWidth
         if maxWidth and (width > maxWidth):
             ratio = maxWidth / width
             width = maxWidth
-            if keepRatio:
-                height = height * ratio
+            # Even if keepRatio is False, in that case, nevertheless apply the
+            # ratio on heigth, too.
+            height *= ratio
         # Return the ODF attributes
         s = self.svgNs
         return ' %s:width="%fcm" %s:height="%fcm"' % (s, width, s, height)
@@ -641,16 +600,6 @@ class ImageImporter(DocImporter):
         i = self.importPath.rfind(self.pictFolder)
         imagePath = self.importPath[i+1:].replace('\\', '/')
         self.fileNames[imagePath] = self.at
-        # In the case of SVG files, perform an image conversion to PNG
-        if imagePath.endswith('.svg'):
-            newImportPath = os.path.splitext(self.importPath)[0] + '.png'
-            out, err = utils.executeCommand(['convert', self.importPath,
-                                            newImportPath])
-            if err: raise Exception(CONVERT_ERROR)
-            os.remove(self.importPath)
-            self.importPath = newImportPath
-            imagePath = os.path.splitext(imagePath)[0] + '.png'
-            self.format = 'png'
         # Compute image alignment if CSS attr "float" is specified
         floatValue = getattr(self.cssAttrs, 'float', None)
         if floatValue:
