@@ -140,14 +140,14 @@ class PodEnvironment(OdfEnvironment):
                 r[tagName] = insert
         return r
 
-    def manageInserts(self):
+    def manageInserts(self, tag):
         '''We just dumped the start of an elem. Here we will insert any odt
            chunk if needed.'''
-        if self.currentTag.nsName in self.inserts:
-            insert = self.inserts[self.currentTag.nsName]
+        if tag in self.inserts:
+            insert = self.inserts[tag]
             self.currentBuffer.write(insert.odtChunk)
             # The insert is destroyed after single use
-            del self.inserts[self.currentTag.nsName]
+            del self.inserts[tag]
 
     def onStartElement(self, tag, attrs):
         ns = self.namespaces
@@ -311,7 +311,7 @@ class PodParser(OdfParser):
                 if (tag == (e.tags['span'])) and not e.currentContent.strip():
                     e.currentBuffer.dumpStartElement(tag, attrs)
                     e.exprHasStyle = True
-        e.manageInserts()
+        e.manageInserts(tag)
 
     def endElement(self, tag):
         e = self.env
