@@ -16,6 +16,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from pathlib import Path
 from collections import UserDict
 import zipfile, shutil, xml.sax, os, os.path, re, mimetypes, time
 
@@ -90,12 +91,12 @@ class Renderer:
     def __init__(self, template, context, result, pythonWithUnoPath=None,
       ooServer='localhost', ooPort=2002, stream='auto', stylesMapping={},
       html=False, forceOoCall=False, finalizeFunction=None,
-      overwriteExisting=False, raiseOnError=False, imageResolver=None,
+      overwriteExisting=False, raiseOnError=False, findImage=None,
       stylesTemplate=None, optimalColumnWidths=False, distributeColumns=False,
       script=None, managePageStyles=None, resolveFields=False,
       expressionsHolders=defaultExpressionsHolders, metadata=True,
       pdfOptions='ExportNotes=True', deleteTempFolder=True, protection=False,
-      pageStart=1, tabbedCR=False):
+      pageStart=1, tabbedCR=False, fonts=None):
         '''Base on a document template (whose path is in p_template), which is
            an ODT or ODS file containing special expressions and statements
            written in Python, this renderer generates an ODT file (whose path is
@@ -160,10 +161,20 @@ class Renderer:
         # during the generation of the result file will be dumped into it, as a
         # Python traceback within a note. Else, the error will be raised.
 
-        # p_imageResolver allows POD to retrieve images, from "img" tags within
-        # XHTML content. Indeed, POD may not be able (ie, may not have the
-        # permission to) perform a HTTP GET on those images. Currently, the
-        # resolver can only be a Zope application object.
+        # When, in the process of converting a chunk of XHTML code to ODF, a
+        # "img" tag is encountered, POD performs a HTTP GET to retrieve it. If
+        # the image is stored on the same server running POD, this can be
+        # problematic in two ways:
+        # 1) regarding performance, it would be better to retrieve the image
+        #    from disk;
+        # 2) via HTTP, POD may not be allowed to retrieve the image.
+        # In order to overcome these problems, place a function in "findImage".
+        # While converting chunks of XHTML code, everytime an "img" tag is
+        # encountered, this function will be called, with the image URL as
+        # unique arg. If the function returns the absolute path to the image, it
+        # will be used instead of performing a HTTP GET request. If the function
+        # returns None (ie: it has detected that is is a "truly external" URL),
+        # the HTTP GET will nevertheless be performed.
 
         # p_stylesTemplate can be the path to a LibreOffice file (ie, a .ott
         # file) whose styles will be imported within the result.
@@ -261,6 +272,13 @@ class Renderer:
         # that way may cause problems when exporting the POD result to Microsoft
         # Word. This is why attribute "tabbedCR" exists.
 
+        # Attribute "fonts" may hold the name of a font that must exist on the
+        # machine running this renderer. If such a name is passed, all base
+        # styles defined in the POD template will be redefined with this font.
+        # On (most? all?) Linux machines, list the available fonts via a command
+        # like:
+        #                      fc-list :lang=en family
+
         self.template = template
         self.result = result
         self.resultType = os.path.splitext(result)[1].strip('.')
@@ -284,7 +302,7 @@ class Renderer:
         self.finalizeFunction = self.formatFinalizeFunction(finalizeFunction)
         self.overwriteExisting = overwriteExisting
         self.raiseOnError = raiseOnError
-        self.imageResolver = imageResolver
+        self.findImage = findImage
         self.stylesTemplate = stylesTemplate
         self.optimalColumnWidths = optimalColumnWidths
         self.distributeColumns = distributeColumns
@@ -298,6 +316,7 @@ class Renderer:
         self.protection = protection
         self.pageStart = pageStart
         self.tabbedCR = tabbedCR
+        self.fonts = fonts
         # If sub-renderers are called, keep a trace of them
         self.children = {} # ~{s_templatePath: Renderer}~
         # Keep trace of the original context given to the renderer
@@ -328,11 +347,6 @@ class Renderer:
         self.setStylesMapping(stylesMapping)
         # Store the p_html parameter
         self.html = html
-        # While working, POD may identify "dynamic styles" to insert either in
-        # the "automatic-styles" section of content.xml (ie, the column styles
-        # of tables generated from XHTML tables via xhtml2odt.py), or in the
-        # "styles" section of styles.xml (ie, bullet styles).
-        self.dynamicStyles = {'content': [], 'styles': []}
 
     def reinit(self, result, context):
         '''Re-initialise this renderer (p_self) for recycling him and produce
@@ -342,15 +356,15 @@ class Renderer:
         # Re-create POD parsers
         self.createParsers(context)
         # Reinitialise attributes being specific to a given result
-        self.dynamicStyles = {'content': [], 'styles': []}
-        sm = self.stylesManager.stylesMapping
-        self.stylesManager = StylesManager(self)
-        self.stylesManager.stylesMapping = sm
+        smap = self.stylesManager.stylesMapping
+        self.stylesManager = sm.StylesManager(self)
+        self.stylesManager.stylesMapping = smap
 
     # Attributes to clone to a sub-renderer when using m_clone hereafter
     cloneAttributes = ('pythonWithUnoPath', 'ooServer', 'ooPort', 'html',
-      'raiseOnError', 'imageResolver', 'stylesTemplate', 'optimalColumnWidths',
-      'distributeColumns', 'expressionsHolders', 'protection', 'tabbedCR')
+      'raiseOnError', 'findImage', 'stylesTemplate', 'optimalColumnWidths',
+      'distributeColumns', 'expressionsHolders', 'protection', 'tabbedCR',
+      'fonts')
 
     def clone(self, template, context, result, **params):
         '''Creates another Renderer instance, similar to p_self, but for
@@ -391,7 +405,8 @@ class Renderer:
         # Incorporate the default, unalterable, context
         r.update({'xhtml': self.renderXhtml, 'text': self.renderText,
           'document': self.importDocument, 'image': self.importImage,
-          'pod': self.importPod, 'test': self.evalIfExpression,
+          'pod': self.importPod, 'cell': self.importCell,
+          'shape': self.drawShape, 'test': self.evalIfExpression,
           'TableProperties': sm.TableProperties,
           'BulletedProperties': sm.BulletedProperties,
           'NumberedProperties': sm.NumberedProperties,
@@ -497,9 +512,10 @@ class Renderer:
       pageBreakAfter=False, convertOptions=None):
         '''Implements the POD statement "do... from document"'''
         # If p_at is not None, it represents a path or url allowing to find the
-        # document. If p_at is None, the content of the document is supposed to
-        # be in binary format in p_content. The document p_format may be: odt or
-        # any format in imageFormats.
+        # document. It can be a string or a pathlib.Path instance. If p_at is
+        # None, the content of the document is supposed to be in binary format
+        # in p_content. The document p_format may be: odt or any format in
+        # imageFormats.
 
         # p_anchor, p_wrapInPara, p_size, p_sizeUnit, p_style and p_keepRatio
         # are only relevant for images:
@@ -550,9 +566,10 @@ class Renderer:
         # your function does not return a string containing the convert options,
         # no conversion will occur.
 
-        importer = None
         # Is there someting to import ?
         if not content and not at: raise PodError(DOC_NOT_SPECIFIED)
+        # Convert p_at to a string if it is not the case
+        at = str(at) if isinstance(at, Path) else at
         # Guess document format
         if not format:
             # It should be deduced from p_at
@@ -563,8 +580,8 @@ class Renderer:
             # If format is a mimeType, convert it to an extension
             if format in utils.mimeTypesExts:
                 format = utils.mimeTypesExts[format]
-        isImage = False
-        isOdt = False
+        isImage = isOdt = False
+        importer = None
         if format in self.ooFormats:
             importer = importers.OdtImporter
             self.forceOoCall = True
@@ -638,6 +655,30 @@ class Renderer:
                  resolveFields, forceOoCall)
         return imp.run()
 
+    def importCell(self, content, style='Default'):
+        '''Creates a chunk of ODF code ready to be dumped as table cell'''
+        # The generated cell will contain this textual p_content and this
+        # p_style will be applied.
+        return '<table:table-cell table:style-name="%s">' \
+               '<text:p>%s</text:p></table:table-cell>' % (style, content)
+
+    def drawShape(self, name, type='rect', stroke='none', strokeWidth='0cm',
+                  strokeColor='#666666', fill='solid', fillColor='#666666',
+                  anchor='char', width='1.0cm', height='1.0cm',
+                  deltaX='0.0cm', deltaY='0.0cm', target='styles'):
+        '''Renders a shape within a POD result'''
+        # Generate a style for this shape and add it among dynamic styles
+        style = self.stylesManager.stylesGenerator.addGraphicalStyle(target,
+          stroke=stroke, strokeWidth=strokeWidth, strokeColor=strokeColor,
+          fill=fill, fillColor=fillColor)
+        # Return the ODF code representing the shape
+        return '<draw:%s text:anchor-type="%s" draw:z-index="1" ' \
+               'draw:name="%s" draw:style-name="%s" ' \
+               'draw:text-style-name="MP1" svg:width="%s" svg:height="%s"' \
+               ' svg:x="%s" svg:y="%s"><text:p/></draw:%s>' % \
+               (type, anchor, name, style.name, width, height, deltaX, deltaY,
+                type)
+
     def _insertBreak(self, type):
         '''Inserts a page or column break into the result'''
         name = 'podPageBreakAfter' if type == 'page' else 'podColumnBreak'
@@ -678,8 +719,6 @@ class Renderer:
         if self.fileNames:
             toInsert = ''
             for fileName in self.fileNames.keys():
-                if fileName.endswith('.svg'):
-                    fileName = os.path.splitext(fileName)[0] + '.png'
                 mimeType = mimetypes.guess_type(fileName)[0]
                 toInsert += ' <manifest:file-entry manifest:media-type="%s" ' \
                             'manifest:full-path="%s"/>\n' % (mimeType, fileName)
@@ -869,29 +908,26 @@ class Renderer:
         if mps is not None:
             pageStyles = self.stylesManager.pageStyles.init(mps)
         # Patch styles.xml and content.xml
+        dynamic = self.stylesManager.dynamicStyles
         for name in ('styles', 'content'):
             # Copy the [content|styles].xml file from the temp to the zip folder
             fn = '%s.xml' % name
             shutil.copy(j(self.tempFolder, fn), j(self.unzipFolder, fn))
-            # For styles.xml, complete dynamic styles with default styles for
-            # bulleted and numbered lists.
-            ds = self.dynamicStyles[name]
-            if name == 'styles':
-                env = self.stylesParser.env
-                n = {'text': env.ns(env.NS_TEXT), 'style': env.ns(env.NS_STYLE)}
-                numbered = sm.NumberedProperties()
-                bulleted = sm.BulletedProperties()
-                ds.insert(0, numbered.dumpStyle('podNumberedList', n))
-                ds.insert(0, bulleted.dumpStyle('podBulletedList', n))
-            # Get the file content and inject dynamic styles into it
+            # Get the file content
             fn = os.path.join(self.unzipFolder, fn)
             f = open(fn)
-            content = f.read().replace('<!DYNAMIC_STYLES!>',
-                                       b''.join(ds).decode())
+            content = f.read()
+            f.close()
+            # Inject self.fonts, when present, in styles.xml
+            isStylesXml = name == 'styles'
+            if isStylesXml and self.fonts:
+                content = sm.FontsInjector(self.fonts).injectIn(content)
+            # Inject dynamic styles
+            if isStylesXml: content = dynamic.injectIn('styles_base', content)
+            content = dynamic.injectIn(name, content)
             # Rename the page styles
             if pageStyles:
                 content = pageStyles.renameIn(name, content)
-            f.close()
             # Write the updated content to the file
             f = open(fn, 'w')
             f.write(content)
