@@ -21,7 +21,7 @@
 # Contributors: Gauthier Bastien, Fabio Marcuzzi, IMIO
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re, xml.sax
+import re, xml.sax, math
 
 from appy.pod import *
 from appy.xml.escape import Escape
@@ -522,7 +522,7 @@ class HtmlTable(Element):
                (s, s, self.name, s, s, s, s, formatNumber(tableWidth, sep='.'),
                 s, percentage, self.tableNs, align, self.tableNs, align,
                 margins, kwn, unbreak, s)
-        self.env.renderer.dynamicStyles['content'].append(decl.encode('utf-8'))
+        self.env.stylesManager.dynamicStyles.add('content', decl)
         return self.name, tableWidthPx, originalTableWidthPx
 
     def setColumnWidth(self, width):
@@ -535,11 +535,30 @@ class HtmlTable(Element):
         if widths[self.cellIndex] is None:
             widths[self.cellIndex] = width
 
-    def computeColumnStyles(self, renderer):
+    def softenWidths(self):
+        '''Reduce columns widths being proportionnaly too large'''
+        sizes = self.columnContentSizes
+        i = len(sizes) - 1
+        while i >= 0:
+            # Calibrate the value in order to soften differences between small
+            # and large column widths.
+            value = sizes[i]
+            if not value:
+                i -= 1
+                continue
+            elif value < 5: value = 10
+            elif value < 50: value = value * 2.5
+            else:
+                value = min(value, 1000)
+            sizes[i] = int(math.pow(math.log(value), 3))
+            i -= 1
+        return sizes
+
+    def computeColumnStyles(self):
         '''Once the table has been completely parsed, self.columnContentSizes
            should be correctly filled. Based on this, we can deduce the width
            of every column and create the corresponding style declarations, in
-           p_renderer.dynamicStyles.'''
+           p_self.env.stylesManager.dynamicStyles.'''
         # The objective is to compute, in "widths", relative column widths,
         # as percentages, from 0 to 1.0.
         widths = []
@@ -593,7 +612,7 @@ class HtmlTable(Element):
         # content from all columns.
         contentTotal = 0
         i = 0
-        contentSizes = self.columnContentSizes
+        contentSizes = self.softenWidths()
         longestWords = self.columnLongestWords
         while i < self.nbOfColumns:
             # Ignore columns for which a width has already been computed
@@ -618,13 +637,16 @@ class HtmlTable(Element):
                 else:
                     contentSize = longest = 0
                 # Compute the column width
-                width = (float(contentSize) / contentTotal) * remainingPc
-                if width < minCellWidth:
-                    surplus += minCellWidth - width
-                    val = (minCellWidth, True)
+                if contentSize == 0:
+                    val = 0
                 else:
-                    remainingCount += 1
-                    val = (width, False)
+                    width = (float(contentSize) / contentTotal) * remainingPc
+                    if width < minCellWidth:
+                        surplus += minCellWidth - width
+                        val = (minCellWidth, True)
+                    else:
+                        remainingCount += 1
+                        val = (width, False)
                 widths[i] = val
             i += 1
         # "Convert" stored tuples into final values
@@ -647,18 +669,19 @@ class HtmlTable(Element):
         i = 0
         total = 65534.0
         while i < self.nbOfColumns:
+            # Compute the width of this column, relative to "total"
             widths[i] = int(widths[i] * total)
             i += 1
         # Compute style declaration corresponding to every column
         s = self.styleNs
         i = 0
+        dynamic = self.env.stylesManager.dynamicStyles
         for width in widths:
             i += 1
-            # Compute the width of this column, relative to "total"
             decl = '<%s:style %s:name="%s.%d" %s:family="table-column">' \
                    '<%s:table-column-properties %s:rel-column-width="%d*"' \
                    '/></%s:style>' % (s, s, self.name, i, s, s, s, width, s)
-            renderer.dynamicStyles['content'].append(decl.encode('utf-8'))
+            dynamic.add('content', decl)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class XhtmlEnvironment(Environment):
@@ -680,14 +703,15 @@ class XhtmlEnvironment(Environment):
       'upper-latin': ('A',), 'lower-roman': ('i',), 'upper-roman': ('I',),
       'decimal': ('1',),
       # Bullet formats
-      'disc': (u'•'), 'circle': (u'◦'), 'square': (u'▪'), 'none': ''}
+      'disc': ('•'), 'circle': ('◦'), 'square': ('▪'), 'none': ''}
 
     def __init__(self, renderer):
         Environment.__init__(self)
         self.renderer = renderer
+        self.stylesManager = renderer.stylesManager
         self.ns = renderer.currentParser.env.namespaces
-        self.res = u''
-        self.currentContent = u''
+        self.res = ''
+        self.currentContent = ''
         self.currentTags = [] # Stack of currently walked tags
         self.currentLists = [] # Stack of currently walked lists (ul or ol)
         self.currentTables = [] # Stack of currently walked tables
@@ -792,7 +816,7 @@ class XhtmlEnvironment(Environment):
                 for table in self.currentTables:
                     table.cellContentSize += contentSize
                     table.cellLongestWord = max(table.cellLongestWord, longest)
-            self.currentContent = u''
+            self.currentContent = ''
 
     def getOdfAttributes(self, xhtmlElem):
         '''Gets the ODF attributes to dump for p_xhtmlElem'''
@@ -1002,7 +1026,7 @@ class XhtmlEnvironment(Environment):
             table = self.currentTables.pop()
             if table.nbOfColumns:
                 # Computes the column styles required by the table
-                table.computeColumnStyles(self.parser.caller.renderer)
+                table.computeColumnStyles()
             # Dumps the content of the last parsed table into the parent buffer
             self.dumpString(table.res)
         elif elem in TABLE_ROW_TAGS:
@@ -1018,7 +1042,7 @@ class XhtmlEnvironment(Environment):
                     table.res+= '<%s:table-column %s:style-name="%s.%d"/>' % \
                                 (self.tableNs, self.tableNs, table.name, i)
                 table.res += table.tempRes
-                table.tempRes = u''
+                table.tempRes = ''
         elif elem in TABLE_COL_TAGS:
             table = self.currentTables[-1]
             # If we are walking a td or th, update "columnContentSizes" and
@@ -1037,8 +1061,7 @@ class XhtmlEnvironment(Environment):
                     wordSizes.append(None)
                 longest = max(sizes[i] or 0, cellContentSize, 5)
                 wordLongest = max(wordSizes[i] or 0, cellLongestWord, 5)
-                # Put a maximum
-                sizes[i] = min(longest, 100)
+                sizes[i] = longest
                 wordSizes[i] = min(wordLongest, 25)
             table.inCell = table.cellContentSize = table.cellLongestWord = 0
         if current.tagsToClose:
@@ -1243,39 +1266,46 @@ class XhtmlParser(Parser):
 
     def endDocument(self):
         '''Dump all collected list styles'''
-        ds = self.caller.renderer.dynamicStyles['styles']
-        env = self.env
-        ns = {'text': env.textNs, 'style': env.styleNs}
-        for name, props in env.listProperties.items():
-            ds.append(props.dumpStyle(name, ns))
+        styles = self.caller.stylesManager.dynamicStyles.styles_base
+        for name, props in self.env.listProperties.items():
+            styles.append(props.dumpStyle(name))
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class XhtmlPreprocessor:
     '''The Xhtml2OdtConverter (see below) requires valid XHTML as input, because
        it uses a SAX parser to do its job. This preprocessor ensures it is the
        case, by applying some modifications to this input.'''
+
     # Regular expression representing a HTML void tag
     voidTags = ('area', 'base', 'br', 'col', 'hr', 'img', 'input', 'link',
                 'meta', 'param', 'command', 'keygen', 'source')
     voidTag = re.compile('<(%s)([^>]*?)(/)?\s*>' % '|'.join(voidTags), re.S)
 
-    @staticmethod
-    def patchTag(match):
+    # Regular expression representing a "pre" tag
+    preTag = re.compile('<pre.*?>(.*?)</pre>', re.S)
+
+    @classmethod
+    def patchTag(class_, match):
         '''Add the trailing slash if absent'''
         # If char "/" is already here, do not change anything
         if match.group(3): return match.group(0)
         return '<%s%s/>' % (match.group(1), match.group(2))
 
-    @staticmethod
-    def html2xhtml(s):
-        '''Converts valid HTML p_s to valid XHTML'''
-        return re.sub(XhtmlPreprocessor.voidTag, XhtmlPreprocessor.patchTag, s)
+    @classmethod
+    def replacePreTag(class_, match):
+        '''Replace the matched "pre" tag by a "table" tag'''
+        return '<table style="margin: 15px 0"><tr><td>' \
+               '<p>%s</p></td></tr></table>' % \
+               match.group(1).replace('\n', '</p><p>')
 
-    @staticmethod
-    def preprocess(s, html=False):
-        '''Converts string p_s to valid XHTML and r_eturns it. If p_html is
-           True, p_s is supposed to be valid HTML: void tags must be converted
-           to XHTML's self-closing start tags.'''
+    @classmethod
+    def preprocess(class_, s, html=False, pre=True):
+        '''Converts string p_s to valid XHTML and r_eturns it'''
+        # If p_html is True, p_s is supposed to be valid HTML: void tags must be
+        # converted to XHTML's self-closing start tags.
+        # ~~~
+        # If p_pre is True, "pre" tags are converted to one-cell tables
+        # ~~~
         if s is None: s = ''
         # Surround p_s with a tag in order to get a XML-compliant file (we need
         # a root tag). We also remove special blank chars that produce SAX
@@ -1283,8 +1313,9 @@ class XhtmlPreprocessor:
         s = '<p>%s</p>' % s.replace('\f', '').replace('\v', '')
         # Convert, when required, conversion of HTML's void tags to XHTML's
         # self-closing start tags.
-        if html:
-            s = XhtmlPreprocessor.html2xhtml(s)
+        #if html: s = class_.voidTag.sub(class_.patchTag, s)
+        if html: s = class_.voidTag.sub(class_.patchTag, s)
+        if pre:  s = class_.preTag.sub(class_.replacePreTag, s)
         return s
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
