@@ -22,8 +22,9 @@ from itertools import count
 from http.server import HTTPServer
 import os, sys, time, socket, logging, pathlib, queue, atexit, threading, ctypes
 
-from appy import utils
+from appy.px import Px
 from appy.model import Model
+from appy import utils, version
 from appy.database import Database
 from appy.utils import url as uutils
 from appy.model.utils import Object as O
@@ -32,44 +33,56 @@ from appy.server.handler import HttpHandler, InitHandler
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 START_CLASSIC = ':: Starting server ::'
-START_CLEAN = ':: Starting clean mode ::'
-START_RUN = ':: Starting run mode (%s) ::'
-READY = '%s:%s ready (process ID %d).'
-STOP_CLASSIC = ':: %s:%s stopped ::'
-STOP_CLEAN = ':: Clean end ::'
-STOP_RUN = ':: Run end ::'
+START_CLEAN   = ':: Starting clean mode ::'
+START_RUN     = ':: Starting run mode (%s) ::'
+READY         = '%s:%s ready (process ID %d).'
+STOP_CLASSIC  = ':: %s:%s stopped ::'
+STOP_CLEAN    = ':: Clean end ::'
+STOP_RUN      = ':: Run end ::'
+APPY_VERSION  = 'Appy is "%s".'
 
-MIN_THREADS_KO = 'At least one thread must be in use.'
-THREAD_LIMITS_KO = 'killThreadLimit (%d) should be > hungThreadLimit (%d)'
-SPAWN_IF_KO = 'spawnIfUnder (%d) should be < than the number of threads (%d)'
+# Server errors  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+CONN_RESET    = 'Connection reset by peer (client port %d).'
+BROKEN_PIPE   = 'Broken pipe (client port %d).'
 
-INITIAL_WORKER = 'Initial worker'
-ADDED_WORKER = 'Worker added in response to lack of idle workers'
-ADDED_WORKER_KILLED = 'Worker added for replacing killed thread %s'
-ADDED_WORKER_REPLACED = 'Voluntary replacement for thread %s'
-WORK_DONE = ' > handled by thread %s (%i tasks queued).'
-CHECK_ADD_CHECK = 'No idle workers for task; checking if we need to make ' \
-  'more workers'
-SPAWNING = 'No idle tasks and only %d busy task(s): adding %d more workers'
-NO_SPAWNING = 'No extra workers needed (%d busy worker(s))'
-CULL_WORKERS = 'Culling %s extra workers. %s idle workers present: %s'
-CULL_KILLED = 'Killed thread %s no longer around'
-KILL_KO = 'PyThreadState_SetAsyncExc failed'
-KILLING_THREAD = 'Killing thread with ID=%s...'
+#  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#                        Pool of threads errors
+#  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+#           THR = thread * WK = worker * SHUTD = shutdown
+
+MIN_THR_KO    = 'At least one thread must be in use.'
+THR_LIMITS_KO = 'killThreadLimit (%d) should be > hungThreadLimit (%d)'
+SPAWN_IF_KO   = 'spawnIfUnder (%d) should be < than the number of threads (%d)'
+
+INITIAL_WK    = 'Initial worker'
+ADDED_WK      = 'Worker added in response to lack of idle workers'
+WK_KILLED     = 'Worker added for replacing killed thread %s'
+WK_REPLACED   = 'Voluntary replacement for thread %s'
+WORK_DONE     = ' > handled by thread %s (%i tasks queued).'
+CHECK_ADD_THR = 'No idle workers for task; checking if we need to make ' \
+                'more workers'
+SPAWNING      = 'No idle tasks and only %d busy task(s): adding %d more workers'
+NO_SPAWNING   = 'No extra workers needed (%d busy worker(s))'
+CULL_WORKERS  = 'Culling %s extra workers. %s idle workers present: %s'
+CULL_KILLED   = 'Killed thread %s no longer around'
+KILL_KO       = 'PyThreadState_SetAsyncExc failed'
+KILLING_THR   = 'Killing thread with ID=%s...'
 KILLED_HUNGED = 'Workers killed forcefully'
-WORKER_TO_KILL = 'Killing worker %s... (working on task for %i seconds)'
-HUNG_STATUS = "killHungThreads status: %d threads (%d working, %d idle, " \
-  "%d starting), average time %s, maxTime %.2fsec, killed %d worker(s)."
+WK_TO_KILL    = 'Killing worker %s... (working on task for %i seconds)'
+HUNG_STATUS   = "killHungThreads status: %d threads (%d working, %d idle, " \
+                "%d starting), average time %s, maxTime %.2fsec, killed %d " \
+                "worker(s)."
 ZOMBIES_FOUND = 'Zombie thread(s) found: %s.'
-EXITING = 'Exiting process because %s zombie threads is more than %s limit.'
-NEW_WORKER = 'Started new worker %s: %s'
-STOPPING_THREAD = 'Thread %s processed %i requests (limit %s); stopping it'
-SHUTDOWN_RECEIVED = 'Worker %s is asked to shutdown'
-SHUTDOWN_POOL = 'Shutting down the threadpool (%d threads)'
-SHUTDOWN_HUNG = "%s worker(s) didn't stop properly, and %s zombie(s)"
-SHUTDOWN_OK = 'All workers stopped'
-SHUTDOWN_FORCED = 'Forcefully exiting process'
-SHUTDOWN_FORCED_OK = 'All workers eventually killed'
+EXITING       = 'Exiting process because %s zombie threads is more than %s limit.'
+NEW_WORKER    = 'Started new worker %s: %s'
+STOPPING_THR  = 'Thread %s processed %i requests (limit %s); stopping it'
+SHUTD_RECV    = 'Worker %s is asked to shutdown'
+SHUTD_POOL    = 'Shutting down the threadpool (%d threads)'
+SHUTD_HUNG    = "%s worker(s) didn't stop properly, and %s zombie(s)"
+SHUTD_OK      = 'All workers stopped'
+SHUTD_FORCED  = 'Forcefully exiting process'
+SHUTD_F_OK    = 'All workers eventually killed'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -129,15 +142,27 @@ class Config:
         appPath = pathlib.Path(appFolder)
         self.static = StaticConfig(appPath)
 
-    def getUrl(self, handler):
-        '''Returns the base URL for this Appy server'''
-        # If key 'Host' is specified as header key in the HTTP request, use it
-        host = handler.headers.get('Host')
-        if not host:
-            host = self.address
-            if self.port != 80:
-                host += ':%d' % self.port
-        return '%s://%s' % (self.protocol, host)
+    def getUrl(self, handler, relative=False):
+        '''Returns the base URL for URLs produced by this Appy server'''
+        if relative:
+            base = ''
+        else:
+            # Use, in that order, keys 'X-Forwarded-Host' or 'Host' if found
+            # among the HTTP header.
+            headers = handler.headers
+            host = headers.get('X-Forwarded-Host') or headers.get('Host')
+            if not host:
+                host = self.address
+                if self.port != 80:
+                    host += ':%d' % self.port
+            # Get protocol from header key 'X-Forwarded-Proto' when present
+            protocol = headers.get('X-Forwarded-Proto') or self.protocol
+            # Build the base of the URL
+            base = '%s://%s' % (protocol, host)
+        # Add a potential path prefix to URLs
+        prefix = headers.get('X-Forwarded-Prefix')
+        prefix = '/%s' % prefix if prefix else ''
+        return '%s%s' % (base, prefix)
 
     def inUse(self):
         '''Returns True if (self.address, self.port) is already in use'''
@@ -156,11 +181,11 @@ class Config:
     def check(self):
         '''Ensure this config is valid'''
         # At least one thread must be specified
-        assert (self.threads > 1), MIN_THREADS_KO
+        assert (self.threads > 1), MIN_THR_KO
         # Ensure limits are consistent
         kill = self.killThreadLimit
         hung = self.hungThreadLimit
-        assert (not kill or (kill >= hung)), (THREAD_LIMITS_KO % (kill, hung))
+        assert (not kill or (kill >= hung)), (THR_LIMITS_KO % (kill, hung))
         # Ensure number of threads are consistent
         spawnIf = self.spawnIfUnder
         threads = self.threads
@@ -250,7 +275,7 @@ class ThreadPool:
             atexit.register(self.shutdown)
         # Create the threads
         for i in range(config.threads):
-            self.addWorker(message=INITIAL_WORKER)
+            self.addWorker(message=INITIAL_WK)
 
     def debug(self, message):
         '''Logs a message'''
@@ -270,7 +295,7 @@ class ThreadPool:
             # spawnIfUnder can come into effect. Count busy threads.
             busy = 0
             now = time.time()
-            self.debug(CHECK_ADD_THREAD)
+            self.debug(CHECK_ADD_THR)
             for worker in self.workers:
                 # Ignore uninitialized threads
                 if not hasattr(worker, 'thread_id'): continue
@@ -279,11 +304,11 @@ class ThreadPool:
                     if (now - started) < cfg.hungThreadLimit:
                         busy += 1
             if cfg.spawnIfUnder > busy:
-                delta = self.spawnIfUnder - busy
+                delta = cfg.spawnIfUnder - busy
                 self.debug(SPAWNING % (busy, delta))
                 self.lastAddedNewIdleWorkers = time.time()
                 for i in range(delta):
-                    self.addWorker(message=ADDED_WORKER)
+                    self.addWorker(message=ADDED_WK)
             else:
                 self.debug(NO_SPAWNING % busy)
         # Kill supernumerary workers when relevant
@@ -310,7 +335,7 @@ class ThreadPool:
         except KeyError:
             pass
 
-    def getTracked(self):
+    def getTracked(self, formatted=True):
         '''Returns a dict summarizing info about the threads in the pool'''
         r = O(idle=[], busy=[], hung=[], dying=[], zombie=[])
         now = time.time()
@@ -328,8 +353,19 @@ class ThreadPool:
         for id, (killed, worker) in self.dyingThreads.items():
             culled = self.cullThreadIfDead(id)
             if culled: continue
-            attr = 'zombie' if now - killed > self.dyingLimit else 'dying'
+            attr = 'zombie' if now - killed > cfg.dyingLimit else 'dying'
             getattr(r, attr).append(worker)
+        if formatted:
+            # Return it as a XHTML table
+            rows = []
+            for name, threads in r.items():
+                info = []
+                for thread in threads:
+                    info.append('%s - %s' % (thread.getName(), thread.ident))
+                rows.append('<tr><th>%s</th><td>%s</td></tr>' % \
+                            (name, '<br/>'.join(info)))
+            r = '<table class="small"><tr><th>Type</th><th>Threads</th></tr>' \
+                '%s</table>' % '\n'.join(rows)
         return r
 
     def killWorker(self, id):
@@ -353,11 +389,11 @@ class ThreadPool:
             del self.tracked[id]
         except KeyError:
             pass
-        self.debug(KILLING_THREAD % id)
+        self.debug(KILLING_THR % id)
         if thread in self.workers:
             self.workers.remove(thread)
         self.dyingThreads[id] = (time.time(), thread)
-        self.addWorker(message=ADDED_WORKER_KILLED % id)
+        self.addWorker(message=WK_KILLED % id)
 
     def addWorker(self, *args, **kwargs):
         index = next(self.workersCount)
@@ -393,7 +429,7 @@ class ThreadPool:
             maxTime = max(maxTime, lifetime)
             totalTime += lifetime
             if lifetime > cfg.killThreadLimit:
-                self.logger.warn(WORKER_TO_KILL % id, lifetime)
+                self.logger.warn(WK_TO_KILL % (id, lifetime))
                 self.killWorker(id)
                 killed += 1
         average = '%.2fsec' % (totalTime / working) if working else 'N/A'
@@ -437,14 +473,14 @@ class ThreadPool:
                 max = cfg.maxRequests
                 if max and (processed > max):
                     # Replace me then die
-                    self.debug(STOPPING_THREAD % (id, processed, max))
+                    self.debug(STOPPING_THR % (id, processed, max))
                     replace = True
                     break
                 # Get the job I have to do
                 runnable = self.queue.get()
                 if runnable is ThreadPool.SHUTDOWN:
                     # This is not a job, but a shutdown command
-                    self.debug(SHUTDOWN_RECEIVED % id)
+                    self.debug(SHUTD_RECV % id)
                     break
                 # Note myself as working
                 try:
@@ -480,12 +516,12 @@ class ThreadPool:
             except KeyError:
                 pass
             if replace:
-                self.addWorker(message=ADDED_WORKER_REPLACED % id)
+                self.addWorker(message=WK_REPLACED % id)
 
     def shutdown(self, forceQuitTimeout=0):
         '''Shutdown the queue (after finishing any pending requests)'''
         count = len(self.workers)
-        self.debug(SHUTDOWN_POOL % count)
+        self.debug(SHUTD_POOL % count)
         # Add a shutdown request for every worker
         for i in range(count):
             self.queue.put(ThreadPool.SHUTDOWN)
@@ -500,7 +536,7 @@ class ThreadPool:
             if self.threadExists(id):
                 zombies.append(id)
         if hung or zombies:
-            self.debug(SHUTDOWN_HUNG % (len(hung), len(zombies)))
+            self.debug(SHUTD_HUNG % (len(hung), len(zombies)))
             if hung:
                 for worker in hung:
                     self.killWorker(worker.thread_id)
@@ -526,20 +562,17 @@ class ThreadPool:
                         if mod == 'threading':
                             atexit._exithandlers.remove(callback)
                     atexit._run_exitfuncs()
-                    self.debug(SHUTDOWN_FORCED)
+                    self.debug(SHUTD_FORCED)
                     os._exit(3)
                 else:
-                    self.debug(SHUTDOWN_FORCED_OK)
+                    self.debug(SHUTD_F_OK)
         else:
-            self.debug(SHUTDOWN_OK)
+            self.debug(SHUTD_OK)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Server(HTTPServer):
     '''Appy HTTP server'''
 
-    # Store, for every logged user, the date/time of its last access
-    loggedUsers = {}
-
     def logStart(self, method):
         '''Logs the appropriate "ready" message, depending on p_self.mode'''
         # Uncomment this line to get more debug info from the pool of threads
@@ -550,7 +583,9 @@ class Server(HTTPServer):
             text = START_CLEAN
         elif self.mode == 'run':
             text = START_RUN % method
-        self.loggers.app.info(text)
+        logger = self.loggers.app
+        logger.info(text)
+        logger.info(APPY_VERSION % version.verbose)
 
     def logShutdown(self):
         '''Logs the appropriate "shutdown" message, depending on p_self.mode'''
@@ -563,9 +598,12 @@ class Server(HTTPServer):
             text = STOP_RUN
         self.loggers.app.info(text)
 
-    def __init__(self, config, mode, method=None):
+    def __init__(self, config, mode, method=None, ext=None):
         # p_config is the main app config
         self.config = config
+        self.appyVersion = version.verbose
+        # If an ext is there, load it
+        if ext: __import__(ext.name)
         # Ensure the config is valid
         config.check()
         # p_mode can be:
@@ -642,6 +680,10 @@ class Server(HTTPServer):
         '''Processes an incoming p_request'''
         try:
             self.finish_request(request, client_address)
+        except ConnectionResetError:
+            self.loggers.app.error(CONN_RESET % client_address[1])
+        except BrokenPipeError:
+            self.loggers.app.error(BROKEN_PIPE % client_address[1])
         except Exception:
             self.handle_error(request, client_address)
         finally:
@@ -698,7 +740,9 @@ class Server(HTTPServer):
         # Patch p_base if the static resource is in RAM
         if ram: base = cfg.static.ramRoot
         r = '%s/%s/%s%s' % (cfg.getUrl(handler), cfg.static.root, base, name)
-        return r if not bg else 'background-image: url(%s)' % r
+        if not bg: return r
+        suffix = ';background-size:%s' % bg if isinstance(bg, str) else ''
+        return 'background-image:url(%s)%s' % (r, suffix)
 
     def getUrlParams(self, params):
         '''Return the URL-encoded version of dict p_params as required by
@@ -731,11 +775,11 @@ class Server(HTTPServer):
         # ----------------------------------------------------------------------
         # The presence of parameter "popup=True" in the URL will open the
         # corresponding object-related page in the Appy iframe, in a
-        # minimalistic way (ie, without portlet).
+        # minimalistic way (ie, without portlet, header and footer).
         # ----------------------------------------------------------------------
         # The base app URL
-        r = '' if relative else self.config.server.getUrl(o.H())
-        # Prefix the object ID with a dash if the object is temporary
+        r = self.config.server.getUrl(o.H(), relative=relative)
+        # Add the object ID
         r = '%s/%s' % (r, o.id)
         # Manage p_sub
         r = '%s/%s' % (r, sub) if sub else r
@@ -754,4 +798,23 @@ class Server(HTTPServer):
         else:
             parameters = params
         return '%s?%s' % (r, self.getUrlParams(parameters))
+
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    #                                 PXs
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    view = Px('''
+     <x var="cfg=config.server; server=handler.server">
+      <h2>Server configuration</h2>
+      <table class="small">
+       <tr><th>Server</th><td>:cfg.address</td></tr>
+       <tr><th>Port</th><td>:cfg.port</td></tr>
+       <tr><th>Protocol</th><td>:cfg.protocol</td></tr>
+       <tr><th>Mode</th><td>:server.mode</td></tr>
+       <tr><th>Appy</th><td>:server.appyVersion</td></tr>
+      </table>
+      <x if="server.pool">
+       <h2>Threads status (initial number of threads=<x>:cfg.threads</x>).</h2>
+       <x>::server.pool.getTracked()</x>
+      </x></x>''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
