@@ -20,14 +20,22 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from pathlib import Path
+
+from DateTime import DateTime
+
 from appy.px import Px
 from appy.utils import multicall
 from appy.model.user import User
 from appy.server.cookie import Cookie
+from appy.utils import path as putils
 from appy.model.utils import Object as O
+from appy.model.fields.string import String
+from appy.model.fields.password import Password
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-INVALID_LOGIN_TRANSFORM = 'Invalid value "%s" for attribute "loginTransform".'
+KO_LOG_TFM  = 'Invalid value "%s" for attribute "loginTransform".'
+KO_PERM     = '%s "%s": "%s" disallowed.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -64,12 +72,29 @@ class Config:
         # of appy.server.sso.Config in the field below.
         self.sso = None
 
+    def getLdap(self):
+        '''Returns the LDAP config if it exists'''
+        # The LDAP config can be found either in attribute "ldap" or in the
+        # context of a SSO system. This method returns a tuple
+        #                     (ldapConfig, context)
+        # "context" being None if the global LDAP config is returned, or being
+        # the SSO config if the LDAP is the one embedded in the SSO config.
+        if self.ldap:
+            r = self.ldap, None
+        else:
+            sso = self.sso
+            if not sso or not sso.ldap:
+                r = None, None
+            else:
+                r = sso.ldap, sso
+        return r
+
     def check(self):
         '''Check this config'''
         # Check attribute "loginTransform"
         transform = self.loginTransform
         if transform and (transform not in self.transformValues):
-            raise Exception(INVALID_LOGIN_TRANSFORM % transform)
+            raise Exception(KO_LOG_TFM % transform)
         return True
 
     def transformLogin(self, login):
@@ -97,6 +122,9 @@ class Guard:
         self.config = handler.server.config
         # Authenticate the currently logged user and get its User instance
         self.user = User.authenticate(self)
+        # Remember that this user hits the server now
+        if not handler.fake:
+            handler.onlineUsers[self.user.login] = DateTime()
         # Info about the currently selected authentication context
         self.authContext = None
         # Cache info about this user
@@ -194,7 +222,8 @@ class Guard:
            ((permOnly == 'specific') and (permission != 'write')): return r
         # An additional, user-defined condition, may refine the base permission
         r = multicall(o, 'mayEdit', True)
-        if not r and raiseError: o.raiseUnauthorized()
+        if not r and raiseError:
+            o.raiseUnauthorized(KO_PERM % (o.class_.name, o.id, permission))
         return r
 
     def mayView(self, o, permission='read', raiseError=False):
@@ -205,7 +234,8 @@ class Guard:
         if not r: return
         # An additional, user-defined condition, may refine the base permission
         r = multicall(o, 'mayView', True)
-        if not r and raiseError: self.raiseUnauthorized()
+        if not r and raiseError:
+            o.raiseUnauthorized(KO_PERM % (o.class_.name, o.id, permission))
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -236,6 +266,10 @@ class Guard:
                 # Redirect to the page where the user can change its password
                 back = '%s/edit?page=password' % user.url
                 label = 'login_ok_change_password'
+            elif req.goto:
+                # The user came to its home page for authenticating: he must now
+                # be redirected to the page at request key "goto".
+                back = req.goto
             else:
                 # Redirect to the app's home page
                 back = user.tool.computeHomePage()
@@ -275,6 +309,77 @@ class Guard:
         return "javascript:post('%s', {'action': 'Create', 'className': " \
                "'User', 'nav': 'ref.tool.users.0.0'})" % url
 
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # Password reinitialisation
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    traverse['askPasswordReinit'] = True
+    def askPasswordReinit(self, tool):
+        '''A user (anonymous) does not remember its password. Send him a mail
+           containing a link that will trigger password re-initialisation.'''
+        login = tool.req.rlogin
+        _ = tool.translate
+        if not login:
+            return tool.goto(message=_('action_ko'))
+        login = login.strip()
+        user = tool.search1('User', login=login)
+        message = _('reinit_mail_sent')
+        if not user:
+            # Return the message nevertheless. This way, malicious users can't
+            # deduce information about existing users.
+            return tool.goto(message=message)
+        # If login is an email, use it. Else, use user.email instead.
+        email = user.login
+        if not String.EMAIL.match(email):
+            email = user.email
+        if not email:
+            # Impossible to re-initialise the password
+            return tool.goto(message=message)
+        # Create a temporary file whose name is the user login and whose
+        # content is a generated token.
+        token = Password().generate()
+        with (Path(putils.getOsTempFolder()) / login).open('w') as f:
+            f.write(token)
+        # Send an email
+        initUrl = '%s/guard/doPasswordReinit?login=%s&token=%s' % \
+                  (tool.url, login, token)
+        subject = _('reinit_password')
+        map = {'url':initUrl, 'siteUrl':tool.siteUrl}
+        body= _('reinit_password_body', mapping=map, asText=True)
+        tool.sendMail(email, subject, body)
+        return tool.goto(message=message)
+
+    traverse['doPasswordReinit'] = True
+    def doPasswordReinit(self, tool):
+        '''Performs the password re-initialisation'''
+        req = tool.req
+        login = req.login
+        token = req.token
+        _ = tool.translate
+        # Check if such token exists in temp folder
+        siteUrl = tool.siteUrl
+        tokenFile = Path(putils.getOsTempFolder()) / login
+        if tokenFile.exists():
+            with tokenFile.open() as f:
+                storedToken = f.read()
+                if storedToken == token:
+                    # A commit is required
+                    tool.H().commit = True
+                    # Generate a new password for this user
+                    user = tool.search1('User', login=login)
+                    newPassword = Password().generate()
+                    user.password = newPassword
+                    # Send the new password by email
+                    recipient = user.getMailRecipient()
+                    subject = _('new_password')
+                    map = {'password': newPassword, 'siteUrl': siteUrl}
+                    body = _('new_password_body', mapping=map, asText=True)
+                    tool.sendMail(recipient, subject, body)
+                    tokenFile.unlink()
+                    return tool.goto(siteUrl, _('new_password_sent'))
+        # If here, something went wrong
+        return tool.goto(siteUrl, _('wrong_password_reinit'))
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -285,7 +390,7 @@ class Guard:
      <div if="isAnon and config.security.activateForgotPassword"
           id="askPasswordReinitPopup" class="popup">
       <form id="askPasswordReinitForm" method="post"
-            action=":'%s/askPasswordReinit' % tool.url">
+            action=":'%s/guard/askPasswordReinit' % tool.url">
        <div align="center">
         <p>:_('app_login')</p>
         <input type="text" size="35" name="rlogin" id="rlogin" value=""/>
@@ -305,19 +410,20 @@ class Guard:
           style=":'display:%s' % ('none' if discreet else 'block')">
 
       <!-- Allow to hide the box when relevant -->
-      <img if="discreet" src=":url('close')" style="float:right"
-           onclick="toggleLoginBox(false)" class="clickable"/>
+      <img if="discreet" src=":url('close.svg')" style="float:right"
+           onclick="toggleLoginBox(false)" class="clickable iconXS"/>
 
       <h1>:_('app_name')</h1>
-      <b>:_('please_connect')</b>
+      <center>:_('please_connect')</center>
+      <div class="loginSpace"></div>
       <form id="loginForm" name="loginForm" method="post" class="login"
             action=":'%s/guard/enter' % tool.url">
 
        <!-- Login fields  -->
-       <div><input type="text" name="login" id="login" value=""
-                   placeholder=":_('app_login')" class="loginField"/></div>
-       <div><input type="password" name="password" id="password"
-                   placeholder=":_('app_password')" class="loginField"/></div>
+       <center><input type="text" name="login" id="login" value=""
+                   placeholder=":_('app_login')" class="loginField"/></center>
+       <center><input type="password" name="password" id="password"
+                placeholder=":_('app_password')" class="loginField"/></center>
 
        <!-- The authentication context -->
        <x var="ctx=cfg.authContext"
@@ -331,9 +437,9 @@ class Guard:
        <input type="hidden" name="goto" value=":req.get('goto', '')"/>
 
        <!-- The "submit" button -->
-       <div><input type="submit" name="submit" var="label=_('app_connect')"
-                   value=":label" alt=":label" style="margin: 12px 0 5px 0"/>
-       </div>
+       <div align="right">
+         <input type="submit" name="submit" var="label=_('app_connect')"
+                class="loginSubmit" value=":label" alt=":label"/></div>
 
        <!-- Forgot password ? -->
        <div class="lostPassword" if="cfg.activateForgotPassword">
@@ -366,7 +472,7 @@ class Guard:
      css='''
       .loginBox {
          border: 2px solid #9fa8c1; padding: 35px; color: #335256;
-         background-color:white; width: 260px; height: 300px;
+         background-color:white; width: 333px; height: 373px;
          position: absolute; top: 50%; left: 50%; z-index: 10;
          transform: translate(-50%,-50%); -ms-transform: translate(-50%,-50%);
          box-shadow: 0 4px 8px 0 rgba(160, 104, 132, 0.2),
@@ -374,10 +480,11 @@ class Guard:
       }
       .loginBox a, .loginBox a,:visited {
          font-weight: bold; letter-spacing: 1px }
-      .loginBox h1 { padding-bottom: 0 }
-      .loginField { background-color: #6b7b9c !important;
-         width: 220px !important; padding: 12px !important;
-         margin: 6px 0  !important; color: whitesmoke }
+      .loginBox h1 { padding-bottom: 0; text-align: center }
+      .loginField { background-color: #4a5f73 !important;
+         width: 310px !important; padding: 12px !important;
+         margin: 6px 0  !important; color: #f3f3f7 }
+      .loginSubmit { margin: 22px 0; padding: 7px }
       .lostPassword { padding: 10px 0 }
       /* Placeholder color for the login form */
       .loginBox ::-webkit-input-placeholder {
@@ -385,6 +492,8 @@ class Guard:
       .loginBox ::-moz-placeholder {
         color: white; font-weight: bold; font-size:120% }
       .loginBox :-ms-input-placeholder {
-      color: white; font-weight: bold; font-size: 120% }
+        color: white; font-weight: bold; font-size: 120% }
+      .loginSpace { height: 35px }
+      #rlogin { color: white }
      ''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
