@@ -30,6 +30,11 @@ from appy.model.utils import Object as O
 from appy.server.response import Response
 from appy.server.languages import Languages
 from appy.server.traversal import Traversal
+from appy.pod.actions import EvaluationError
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+TRANS_RETRY = 'Conflict occurred (%s). Replay transaction (#%d)...'
+TRANS_ERR   = 'Unresolved conflict occurred (%s).'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class MethodsCache(dict):
@@ -107,7 +112,9 @@ class Handler:
     @classmethod
     def add(class_, handler):
         '''Adds a new p_handler into the registry'''
-        class_.registry[threading.get_ident()] = handler
+        id = threading.get_ident()
+        if id in class_.registry: return
+        class_.registry[id] = handler
 
     @classmethod
     def remove(class_):
@@ -124,8 +131,6 @@ class Handler:
 
     def init(self):
         '''Define attributes which are common to any handler'''
-        # Add p_self to the registry of handlers
-        Handler.add(self)
         # Direct access to the app's configuration
         self.config = self.server.config
         # A lot of object methods are executed while handling a request. Dict
@@ -145,11 +150,16 @@ class Handler:
         if not hasattr(self.tool, 'initialiseHandler'): return
         self.tool.initialiseHandler(self)
 
+    def clientIP(self):
+        '''Gets the IP address of the client'''
+        # Check header key X-Forwarded-For first
+        return self.headers.get('X-Forwarded-For') or self.client_address[0]
+
     # This dict allows, on a concrete handler, to find the data to log
-    logAttributes = O(ip='self.client_address[0]',
-      port='str(self.client_address[1])', command='self.command',
-      path='self.path', protocol='self.request_version', message='message',
-      user='self.guard.userLogin', agent='self.headers.get("User-Agent")')
+    logAttributes = O(ip='self.clientIP()', port='str(self.client_address[1])',
+      command='self.command', path='self.path', protocol='self.request_version',
+      message='message', user='self.guard.userLogin',
+      agent='self.headers.get("User-Agent")')
 
     def log(self, type, level, message=None):
         '''Logs, in the logger determined by p_type, a p_message at some
@@ -180,9 +190,15 @@ Base.Handler = Handler
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class HttpHandler(Handler, BaseHTTPRequestHandler):
     '''Handles incoming HTTP requests'''
+
+    protocol_version = 'HTTP/1.1'
+
     # This is a "real" request handler
     fake = False
 
+    # Store, for every logged user, the date/time of its last access
+    onlineUsers = {}
+
     # Every time a HTTP request hits the server, an instance of this class is
     # created to handle it. It inherits from BaseHTTPRequestHandler, the base
     # handler shipped with the Python standard library for handling HTTP
@@ -191,7 +207,8 @@ class HttpHandler(Handler, BaseHTTPRequestHandler):
     def determineType(self):
         '''Determines the type of the request and split, in p_self.parts, the
            request path into parts.'''
-        parts = self.path.split('/')
+        self.parsedPath = parsed = urllib.parse.urlparse(self.path.strip('/'))
+        parts = parsed.path.split('/')
         # Remove any blank part among self.parts
         i = len(parts) - 1
         while i >= 0:
@@ -208,7 +225,7 @@ class HttpHandler(Handler, BaseHTTPRequestHandler):
             del parts[0]
         # If the unique part corresponds to a file (ie, any name containing a
         # dot), it is static content, too (favicon.ico, robots.txt...)
-        elif (len(parts) == 1) and Static.isFilename(parts[0]):
+        elif (len(parts) == 1) and ('.' in parts[0]):
             self.static = True
             # This kind of file must exist in the app
             parts.insert(0, self.server.config.model.appName)
@@ -240,46 +257,98 @@ class HttpHandler(Handler, BaseHTTPRequestHandler):
     def init(self):
         '''Appy-specific handler initialisation for handling non-static content
            (called by m_do_GET or m_do_POST).'''
-        # Extract parameters, create a Request instance in p_self.req and remove
-        # parameters from self.parts[-1].
-        self.req = Request.create(self)
         # Create a Response object
         self.resp = Response(self)
-        # An instance of appy.ui.validate.Validator wil be created here, when
+        # An instance of appy.ui.validate.Validator will be created here, when
         # data sent via the ui requires to be validated.
         self.validator = None
         # Search criteria may be cached
         self.criteria = None
-        # Initialise a database connection
-        self.connection = self.server.database.openConnection()
         # Must we commit data into the database ?
         self.commit = False
         # Set here a link to the tool. The handler object will be heavily
         # consulted by a plethora of objects during request handling. This is
         # why it is convenient to define such attributes on it.
-        self.tool = self.connection.root.objects.get('tool')
-        # Cal the base handler's method
+        self.tool = self.dbConnection.root.objects.get('tool')
+        # Call the base handler's method
         Handler.init(self)
 
-    def finish(self):
-        '''Appy-specific handler termination when serving dynamic content
-          (called by the base handler).'''
-        # Not-initialized handlers are run for an unknown reason
-        if not hasattr(self, 'path'):
-            self.log('app', 'error', 'Uninitialized handler run')
-            return
-        # Subsequent code is for non-static handlers only
-        if self.static: return
-        # Remove myself from the registry (for now)
-        Handler.remove()
-        # Close the database connection
-        self.server.database.closeConnection(self.connection)
+    def manageGuardError(self, resp, traversal):
+        '''A security-related error has occurred: the logged is not allowed to
+           access the desired URL. If the user is anon, redirect him to the
+           login page. Else, return an error.'''
+        # Return the response content. In the case of a redirect, None will be
+        # returned.
+        if self.guard.user.isAnon():
+            siteUrl = self.config.server.getUrl(self)
+            gotoUrl = urllib.parse.quote('%s%s' % (siteUrl, self.path))
+            resp.goto(url='%s/tool/home?goto=%s' % (siteUrl, gotoUrl),
+                      message=self.tool.translate('please_authenticate'))
+            r = None
+        else:
+            # Return a 403 error: unauthorized
+            resp.code = 403
+            r = Error.get(resp, traversal)
+        return r
+
+    def manageDatabaseRequest(self):
+        '''Manage an HTTP request whose response must imply using the
+           database and return the response content as a string.'''
+        # Initialise p_self's base data structures
+        self.init()
+        # Run a traversal
+        self.traversal = traversal = Traversal(handler=self)
+        resp = self.resp
+        try:
+            r = traversal.run()
+        except Traversal.Error as err:
+            resp.code = 404
+            r = Error.get(resp, traversal)
+        except Guard.Error as err:
+            r = self.manageGuardError(resp, traversal)
+        except EvaluationError as err:
+            if isinstance(err.originalError, Guard.Error):
+                r = self.manageGuardError(resp, traversal)
+            else:
+                resp.code = 500
+                r = Error.get(resp, traversal)
+        return r
+
+    def attemptDatabaseRequest(self):
+        '''Tries to m_manageDatabaseRequest. Perform several attempts depending 
+           on the occurrence of database conflict errors. Returns the content of
+           the page to serve, as a string.'''
+        maxAttempts = self.server.config.database.conflictRetries
+        attempts = 0
+        database = self.server.database
+        while True:
+            try:
+                r = self.manageDatabaseRequest()
+                # Perform a database commit when appropriate
+                if self.commit: database.commit(self)
+            except database.ConflictErrors as err:
+                attempts += 1
+                # Abort the transaction that could not be committed
+                transaction.abort()
+                # Return an error if is was the last attempt
+                if attempts == maxAttempts:
+                    self.log('site', 'error', TRANS_ERR % str(err))
+                    self.resp.code = 503
+                    r = Error.get(self.resp, self.traversal)
+                    break
+                else:
+                    self.log('site', 'warning',
+                             TRANS_RETRY % (str(err), attempts))
+            except Exception as err:
+                self.resp.code = 500
+                r = Error.get(self.resp, self.traversal)
+                break
+            else:
+                break
+        return r
 
     def do_GET(self):
         '''Called when a HTTP GET request hits the server'''
-        # Uncomment this for more log in debug level
-        #self.log('app', 'debug', '%s %s' % (self.command,
-        #                                    getattr(self, 'path' or '?')))
         # Determine the type of the request: static or dynamic (boolean
         # self.static). Static content is any not-database-stored-and-editable
         # image, Javascript, CSS file, etc. Dynamic content is any request
@@ -289,28 +358,25 @@ class HttpHandler(Handler, BaseHTTPRequestHandler):
             # Manage static content
             code = Static.get(self)
         else:
-            # Initialise the handler
-            self.init()
-            # Run a traversal
-            self.traversal = traversal = Traversal(handler=self)
-            resp = self.resp
-            try:
-                r = traversal.run()
-            except Traversal.Error as err:
-                resp.code = 404
-                r = Error.get(resp, traversal)
-            except Guard.Error as err:
-                resp.code = 403
-                r = Error.get(resp, traversal)
-            except Exception as err:
-                resp.code = 500
-                r = Error.get(resp, traversal)
+            # Add myself to the registry of handlers
+            Handler.add(self)
+            # Initialise a database connection
+            database = self.server.database
+            self.dbConnection = database.openConnection()
+            # Extract parameters and create a Request instance in p_self.req
+            self.req = Request.create(self)
+            # Compute dynamic content
+            r = self.attemptDatabaseRequest()   
             # Build the HTTP response (if not done yet)
-            if not resp.built:
-                resp.build(r)
-            code = resp.code
-            # Perform a database commit when appropriate
-            if self.commit: self.server.database.commit(self)
+            try:
+                resp = self.resp
+                if not resp.built: resp.build(r)
+                code = resp.code
+            finally:
+                # Close the DB connection
+                database.closeConnection(self.dbConnection)
+                # Remove myself from the registry (for now)
+                Handler.remove()
         # Log this hit and the response code on the site log
         self.log('site', 'info', str(code))
 
@@ -333,7 +399,12 @@ class HttpHandler(Handler, BaseHTTPRequestHandler):
 
     # Disable standard log
     def log_message(self, format, *args): pass
-    def log_error(self, format, *args): pass
+
+    # Log base handler errors
+    def log_error(self, format, *args):
+        '''Called by the base handler to log any error occurring during base
+           handler code execution like parsing the request.'''
+        self.log('app', 'error', format % args)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class InitHandler(Handler):
@@ -357,7 +428,9 @@ class InitHandler(Handler):
         self.resp = Response(self)
         # Call the base handler's method
         Handler.init(self)
+        # Add myself to the registry of handlers
+        Handler.add(self)
         # The following attributes will be initialised later
-        self.connection = None
+        self.dbConnection = None
         self.tool = None
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
