@@ -29,6 +29,9 @@ emptyMultiparts = (b'', b'--\r\n')
 # Regular expression for parsing a Content-Disposition (cd) directive  - - - - -
 cdRex = re.compile(b'name="([^"]+)"(?:.+filename="([^"]*)")?')
 
+# Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+EMPTY_POST = 'No Content-Length header for request with Content-Type "%s".'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Request(Object):
     '''Represents data coming from a HTTP request'''
@@ -41,19 +44,17 @@ class Request(Object):
         # Create the Request instance
         req = Request()
         # Get parameters from a GET request if any
-        parts = handler.parts
-        last = parts[-1] if parts else None
-        if last and ('?' in last):
-            lastPart, params = last.split('?', 1)
-            req.parse(params)
-            # As a side-effect, remove parameters from the last part
-            parts[-1] = lastPart
+        qs = handler.parsedPath.query
+        if qs: req.parse(qs)
         # Get POST form data if any
         contentType = handler.headers['Content-Type']
         if contentType:
             # Get the request length and content
-            length = int(handler.headers['Content-Length'])
-            content = handler.rfile.read(length)
+            length = handler.headers['Content-Length']
+            if length is None:
+                handler.log('app', 'warning', EMPTY_POST % contentType)
+                return req
+            content = handler.rfile.read(int(length))
             # Several content encodings are possible
             if contentType == 'application/x-www-form-urlencoded':
                 # Form elements are encoded in a way similar to GET parameters
