@@ -194,7 +194,7 @@ class Config:
         #   identification failure.
         login = headers and headers.get('HTTP_%s' % self.loginKey, None)
         if login:
-            user = tool.search1('User', noSecurity=True, login=login)
+            user = tool.search1('User', login=login)
             if user and (user.source != 'sso'):
                 # Convert it when relevant
                 if self.convertLocalUsers:
@@ -282,14 +282,14 @@ class Config:
                 # "login" is a pattern of groups. Query all groups and link the
                 # user to those matching the pattern.
                 suffix = login[1:]
-                for group in tool.search('Group', noSecurity=True):
+                for group in tool.search('Group'):
                     if group.login.endswith(suffix):
                         group.link('users', user)
                         if self.afterLinkGroup:
                             self.afterLinkGroup(user, group)
             else:
                 # "login" is the login of a single group
-                group = tool.search1('Group', noSecurity=True, login=login)
+                group = tool.search1('Group', login=login)
                 if group:
                     group.link('users', user)
                     if self.afterLinkGroup:
@@ -359,7 +359,7 @@ class Config:
         if user.state == 'inactive':
             msg = SSO_REACTIVATED % user.login
             user.log(msg, noUser=True)
-            user.do('reactivate', noSecurity=True, doHistory=False, comment=msg)
+            user.do('reactivate', doHistory=False, comment=msg)
 
     def createUser(self, tool, login, source, isRequest=True, userParams=None):
         '''Creates the local User copy of a SSO user, from user info as present
@@ -372,8 +372,7 @@ class Config:
         if userParams:
             if params: userParams.update(params)
             params = userParams
-        user = tool.create('users', noSecurity=True, login=login,
-                           source='sso', **params)
+        user = tool.create('users', login=login, source='sso', **params)
         # The last sync date for this user is now
         user.syncDate = DateTime()
         # Set user global roles and groups (if in use)
@@ -392,7 +391,7 @@ class Config:
         if not self.enabled: return
         # Do we already have a local User instance for this user ?
         req = tool.request
-        user = tool.search1('User', noSecurity=True, login=login)
+        user = tool.search1('User', login=login)
         if user:
             # Update the user only if we have not done it since "syncInterval"
             # minutes. The user may not have "syncDate" initialized yet.
