@@ -17,6 +17,7 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import inspect, pathlib, mimetypes, os.path, email.utils, collections
+
 from DateTime import DateTime
 
 import appy
@@ -108,6 +109,14 @@ class Static:
     # The dict of RAM resources (see doc in class Config hereabove)
     ram = collections.OrderedDict()
 
+    @classmethod
+    def initResponse(class_, handler, code, size=0, end=False):
+        '''Set code and default headers in the HTTP response'''
+        handler.send_response(code)
+        handler.send_header('Connection', 'close')
+        handler.send_header('Content-Length', size)
+        if end: handler.end_headers()
+
     @classmethod
     def notFound(class_, handler, config):
         '''Raise a HTTP 404 error if the resource defined by p_handler.parts was
@@ -115,8 +124,7 @@ class Static:
         code = 404
         path = '/%s/%s' % (config.root, '/'.join(handler.parts))
         handler.log('app', 'error', '%d@%s' % (code, path))
-        handler.send_response(code)
-        handler.end_headers()
+        class_.initResponse(handler, code, end=True)
         return code
 
     @classmethod
@@ -124,24 +132,23 @@ class Static:
         '''Write the file content to the socket, either from disk (p_path) or
            from RAM (p_content).'''
         if content is not None:
-            handler.wfile.write(content.encode())
+            handler.wfile.write(content)
         else:
-            f = open(path, 'rb')
-            while True:
-                chunk = f.read(Static.BYTES)
-                if not chunk: break
-                try:
-                    handler.wfile.write(chunk)
-                except BrokenPipeError:
-                    handler.log('app', 'error', BROKEN_PIPE % path)
-            f.close()
+            with open(path, 'rb') as f:
+                while True:
+                    chunk = f.read(Static.BYTES)
+                    if not chunk: break
+                    try:
+                        handler.wfile.write(chunk)
+                    except BrokenPipeError:
+                        handler.log('app', 'error', BROKEN_PIPE % path)
 
     @classmethod
     def writeUnchanged(class_, handler, response=None):
-        '''Returns the adhoc response if the file has not changed since the last
-           time the browser has downloaded it.'''
+        '''Returns the ad-hoc response if the file has not changed since the
+           last time the browser has downloaded it.'''
         code = 304
-        handler.send_response(code)
+        class_.initResponse(handler, code)
         if response:
             response.build(complete=False)
             response.code = code
@@ -150,7 +157,8 @@ class Static:
 
     @classmethod
     def write(class_, handler, path, modified, content=None, response=None,
-              fileInfo=None, disposition='attachment', enableCache=True):
+              fileInfo=None, disposition='attachment', downloadName=None,
+              enableCache=True):
         '''Serves, to the browser, the content of the file whose path on disk is
            given in p_path or whose content in RAM is given in p_content.'''
         # ~~~
@@ -169,7 +177,7 @@ class Static:
         # ~~~
         # If p_path corresponds to a DB-controlled file, his corresponding
         # p_fileInfo is given. In that case, p_disposition will be taken into
-        # account. Else, it will be ignored.
+        # account. Else, it will be ignored, unless p_downloadName is specified.
         # ~~~
         # For privacy reasons, p_enableCache may be disabled (ie, for
         # potentially sensitive content from File fields). This way, it cannot
@@ -180,29 +188,31 @@ class Static:
         smodified = email.utils.formatdate(modified) # RFC 822 string
         if not browserDate or (smodified > browserDate):
             code = 200
-            handler.send_response(code)
+            # Compute content length
             hasContent = content is not None
-            # Initialise response headers
-            set = handler.send_header
-            # ~~~ Identify MIME type ~~~
-            mimeType, encoding = mimetypes.guess_type(path)
-            mimeType = mimeType or 'application/octet-stream'
-            set('Content-Type', mimeType)
-            # ~~~ Compute content length ~~~
+            if hasContent: content = content.encode()
             if fileInfo:
                 size = fileInfo.size
             else:
                 size = len(content) if hasContent else os.path.getsize(path)
-            set('Content-Length', size)
-            # ~~~ Set file encoding ~~~
-            if encoding: set('Content-Encoding', encoding)
-            # ~~~ Define content disposition ~~~
-            if fileInfo:
-                set('Content-Disposition',
-                    '%s;filename="%s"' % (disposition, fileInfo.uploadName))
+            # Initialise response headers
+            class_.initResponse(handler, code, size=size)            
+            # Identify MIME type
+            mimeType, encoding = mimetypes.guess_type(path)
+            mimeType = mimeType or 'application/octet-stream'
+            set = handler.send_header
+            set('Content-Type', mimeType)
+            # Define content disposition
+            if fileInfo or downloadName:
+                niceName = downloadName or fileInfo.uploadName
+                disp = '%s;filename="%s"' % (disposition, niceName)
+                try:
+                    set('Content-Disposition', disp)
+                except UnicodeEncodeError:
+                    # BaseHTTPRequestHandler requires latin-1 encoding
+                    set('Content-Disposition', disp.encode().decode('latin-1'))
             # For now, disable byte serving (value "bytes" instead of "none")
             set('Accept-Ranges', 'none')
-            set('Connection', 'close')
             # ~~~ Manage caching ~~~
             if enableCache:
                 set('Last-Modified', smodified)
@@ -221,11 +231,13 @@ class Static:
         return code
 
     @classmethod
-    def writeFromDisk(class_, handler, path):
-        '''Serve a static file from disk, whose URL path is p_path'''
+    def writeFromDisk(class_, handler, path,
+                      disposition='attachment', downloadName=None):
+        '''Serve a static file from disk, whose path is p_path'''
         # The string version of p_path
         spath = str(path)
-        return class_.write(handler, spath, os.path.getmtime(spath))
+        return class_.write(handler, spath, os.path.getmtime(spath),
+                            disposition=disposition, downloadName=downloadName)
 
     @classmethod
     def writeFromRam(class_, handler, config):
@@ -250,15 +262,6 @@ class Static:
             return class_.notFound(handler, config)
         return class_.write(handler, key, config.created, content=content)
 
-    @classmethod
-    def removeParams(class_, handler):
-        '''Remove potential GET parameters in the last part within
-           p_handler.parts.'''
-        if not handler.parts: return
-        last = handler.parts[-1]
-        if '?' in last:
-            handler.parts[-1] = last[:last.index('?')]
-
     @classmethod
     def get(class_, handler):
         '''Returns the content of the static file whose splitted path is defined
@@ -267,8 +270,6 @@ class Static:
         config = handler.server.config.server.static
         # The currently walked path
         path = None
-        # Remove the potential GET params
-        class_.removeParams(handler)
         # Walk parts
         for part in handler.parts:
             if path is None:
@@ -289,12 +290,4 @@ class Static:
             return class_.notFound(handler, config)
         # Read the file content and return it
         return class_.writeFromDisk(handler, path)
-
-    @classmethod
-    def isFilename(class_, name):
-        '''Does p_name correspond to a filename ?'''
-        # Returns True if a dot is found with p_name. Ignore any content being
-        # after a question mark (p_name can be part of a URL).
-        qm = name.find('?')
-        return '.' in name if qm == -1 else name.find('.', 0, qm) != -1
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
