@@ -18,12 +18,11 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import collections
+import sys, collections
 
 from appy.tr import po
 from appy import Config
 from appy.model.utils import Object as O
-from appy.model.utils import importModule
 from appy.model import Config as ModelConfig
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -170,7 +169,8 @@ class Updater:
         # Load the model in order to get all classes, fields, workflows, states
         # and transitions for which i18n labels must be created.
         appPath = self.appFolder / '__init__.py'
-        self.config = importModule('app', str(appPath)).Config
+        sys.path.append(str(self.appFolder.parent))
+        self.config = __import__(self.appFolder.name).Config
         modelConfig = ModelConfig()
         modelConfig.set(self.appFolder)
         self.model = modelConfig.get(self.config, appOnly=True)
@@ -298,8 +298,7 @@ class Updater:
         # in the p_class_.
         if len(class_.phases) != 1:
             self.addLabel('%s_phase_%s' % (class_.name, phase.name), phase.name)
-        # Produce one label for every page, excepted if there is a single page
-        if len(phase.pages) == 1: return
+        # Produce one label for every page
         for page in phase.pages.values():
             if page.label is None:
                 self.addLabel('%s_page_%s' % (class_.name,page.name), page.name)
