@@ -18,7 +18,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os.path, re, urllib.parse
+import os.path, re
 
 from appy.px import Px
 from appy.ui.layout import ColumnLayout
@@ -28,6 +28,7 @@ from appy.model.utils import Object as O
 from appy.ui.js import Quote
 from appy.ui.title import Title
 from appy.ui.iframe import Iframe
+from appy.ui.message import Message
 from appy.ui.portlet import Portlet
 from appy.ui.globals import Globals
 from appy.ui.navigate import Siblings
@@ -38,6 +39,26 @@ from appy.ui.validate import Validator
 # Some elements in this module will be traversable - - - - - - - - - - - - - - -
 traverse = {'Language': True}
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+# In the UI config as defined hereafter, several attributes will hold the path
+# to a static resource (image, CSS file, etc) coming from Appy itself, from your
+# app or ext. In your app/ext, if you use the standard Appy conventions, any
+# static resource is located in its "static" sub-folder. If you want to refer to
+# such a static resource from your app/ext, use a string of the form
+#
+#                "<app or ext name>/<name of the static resource>"
+#
+# Here is an example. Suppose you app is named "myApp". In myApp/static, you
+# have an image name "myBackground.png" that you want to use as background image
+# for the home page.
+#
+# In your UI Config instance, you must specify, for attribute "homeBackground",
+#
+#                         "myApp/myBackground.png"
+#
+# Resources coming from Appy itself follow the same syntax, with one difference:
+# the <app name> part is replaced with "appy".
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
     '''Represents user-interface configuration for your app'''
@@ -49,13 +70,16 @@ class Config:
     cssVariable = re.compile('\|(\w+?)\|', re.S)
 
     def __init__(self):
+        # For specifying paths to static resources, consult the doc at the start
+        # of this file.
+        # ~~~
         # The home page's background image
         self.homeBackground = 'appy/sunrise.jpg'
         # The logo shown on the home page, in the top left corner
         self.homeLogo = None
         # The logo within the page header
-        self.headerLogo = 'appy/headerLogo.png'
-        # The logo at the bottom of the portlet
+        self.headerLogo = None
+        # The logo at the top of the portlet
         self.portletLogo = 'appy/portletLogo.png'
         # The following attributes determines when the header and footers must
         # be shown. If you place a function in it, it will be called with, as
@@ -83,11 +107,32 @@ class Config:
         self.css = None
         # Some input fields will get this background color once they will
         # contain erroneous content.
-        self.wrongTextColor = '#f9edbe'
-        # Border style for tabs
-        self.tabBorder = '1px solid #ff8040'
+        self.wrongTextColor = '#009ba4'
+        # Text color representing a problem or warning (some flavour of red)
+        self.warnColor = '#e15151'
         # Text color for home text
         self.homeTextColor = 'white'
+        # Text color in the header
+        self.headerColor = 'white'
+        # Background color for the header
+        self.headerBgColor = '#002039'
+        # Text color for links
+        self.linkColor = '#002039'
+        self.visitedColor = '#002039'
+        # Alternate text color, also used for button borders other discreet
+        # places.
+        self.altColor = '#009aa4'
+        # Variant being lighter
+        self.altColorLight = '#e9f2f3'
+        # Color used as "fill" color, ie, for the message box
+        self.fillColor = '#dab823'
+        # Header height
+        self.headerHeight = '60px'
+        # Border style for tabs
+        self.tabBorder = '1px solid %s' % self.linkColor
+        # In the popup, everything is smaller. If you want to achieve the same
+        # result in the main window, set "compact" to True.
+        self.compact = False
         # Application-wide default formats for hours and dates
         self.dateFormat = '%d/%m/%Y'
         self.hourFormat = '%H:%M'
@@ -110,6 +155,10 @@ class Config:
         # self.languages. Else, the browser-defined language will be used for
         # choosing the language of returned pages.
         self.languageSelector = False
+        # If the language selector is shown, the default selectable languages
+        # will be those from p_self.languages hereabove, excepted if you specify
+        # a sub-set of it in the following attribute.
+        self.selectableLanguages = None
         # If "forceLanguage" is set, Appy will not take care of the browser
         # language, will always use the forced language and will hide the
         # language selector, even if "languageSelector" hereabove is True.
@@ -176,16 +225,12 @@ class Config:
             r += ' (%s)' % date.strftime(self.hourFormat)
         return r
 
-    def getHeaderMessages(self, tool, handler):
-        '''Get the permanent messages that must appear in the page header'''
-        # Get the "test" message
+    def getHeaderText(self, tool):
+        '''Get the permanent text that must appear in the page header'''
+        # Get the text via config attribute "test"
         r = self.test
         if callable(r): r = test(tool)
-        r = r or ''
-        # Get the browser incompatibility message
-        bi = Browser.getIncompatibilityMessage(tool, handler)
-        bi = bi = '<div class="wrongBrowser">%s</div>' % bi if bi else ''
-        return r + bi
+        return r or ''
 
     def getBackground(self, px, siteUrl, type):
         '''Return the CSS properties allowing to include this background p_image
@@ -200,11 +245,8 @@ class Config:
         elif type == 'header':
             # This is the background logo for the page header
             image = self.headerLogo
+            if not image: return ''
             attrs = 'background-position: center'
-        elif type == 'portlet':
-            # This is the background image within the portlet
-            attrs = 'background-position: 45% 90%'
-            image = self.portletLogo
         return 'background-image: url(%s/static/%s); background-repeat: ' \
                'no-repeat; %s' % (siteUrl, image, attrs)
 
@@ -232,8 +274,8 @@ class Config:
            any.'''
         # Apply default CSS classes
         if part == 'main':
-            bg = '' if context.popup else ' mainBg'
-            r = 'main rel%s' % bg
+            compact = ' mainC' if self.compact or context.popup else ''
+            r = 'main rel%s' % compact
         else:
             r = ''
         # Add specific classes when relevant
@@ -269,47 +311,6 @@ class Config:
             r = tool.allows('write')
         return r
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Message:
-    '''Manages the "message zone" allowing to display messages coming from the
-       Appy server to the end user.'''
-
-    @classmethod
-    def consumeAll(class_, handler, unlessRedirect=False):
-        '''Returns the list of messages to show to a web page'''
-        # Do not consume anything if p_unlessRedirect is True and we are
-        # redirecting the user.
-        if unlessRedirect and ('Appy-Redirect' in handler.resp.headers): return
-        # Try to get messages from the 'AppyMessage' cookie
-        message = handler.req.AppyMessage
-        if message:
-            # Dismiss the cookie
-            handler.resp.setCookie('AppyMessage', 'deleted')
-            return urllib.parse.unquote(message)
-
-    @classmethod
-    def hasValidationErrors(class_, handler):
-        '''Returns True if there are validaton errors collected by the
-           (handler.)validator.'''
-        return handler.validator and handler.validator.errors
-
-    # The message zone
-    px = Px('''
-     <div var="validErrors=ui.Message.hasValidationErrors(handler)"
-          class=":'messagePopup message' if popup else 'message'"
-          style=":'display:none' if not validErrors else 'display:block'"
-          id="appyMessage">
-      <!-- The icon for closing the message -->
-      <img src=":url('close')" align=":dright" class="clickable"
-           onclick="this.parentNode.style.display='none'"/>
-      <!-- The message content -->
-      <div id="appyMessageContent">:validErrors and _('validation_error')</div>
-      <div if="validErrors"
-           var2="validator=handler.validator">:handler.validator.pxErrors</div>
-     </div>
-     <script var="messages=ui.Message.consumeAll(handler)"
-             if="messages">::'showAppyMessage(%s)' % q(messages)</script>''')
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class LinkTarget:
     '''Represents information about the target of an HTML "a" tag'''
@@ -362,7 +363,7 @@ class Collapsible:
     def get(class_, zone, align, req):
         '''Gets a Collapsible instance for showing/hiding some p_zone
            ("portlet" or "sidebar").'''
-        icons = (align == 'left') and 'showHide' or 'showHideInv'
+        icons = 'showHide' if align == 'left' else 'showHideInv'
         return Collapsible('appy%s' % zone.capitalize(), req,
                            default='expanded', icons=icons, align=align)
 
@@ -414,13 +415,29 @@ class Sidebar:
         if callable(sidebar): sidebar = sidebar(tool)
         if not sidebar: return
         if sidebar.show in (True, layout):
-            return sidebar.width or '230px'
+            # Complete user info
+            sidebar.width = sidebar.width or '320px'
+            sidebar.minWidth = sidebar.minWidth or '220px'
+            return sidebar
+
+    @classmethod
+    def getStyle(class_, sidebar, collapse):
+        '''Gets the CSS properties to apply to the sidebar'''
+        return '%s;width:%s;min-width:%s' % \
+               (collapse.style, sidebar.width, sidebar.minWidth)
 
     px = Px('''
-     <x var="page,grouped,css,js,phases=o.getGroupedFields('main', 'sidebar')">
+     <div var="page,grouped,css,js,phases=o.getGroupedFields('main','sidebar');
+               collapse=ui.Collapsible.get('sidebar', dright, req)"
+          id=":collapse.id" class="sidebar"
+          style=":ui.Sidebar.getStyle(showSidebar, collapse)">
       <x>::ui.Includer.getSpecific(tool, css, js)</x>
       <x var="layout='view'">:o.pxFields</x>
-     </x>''')
+     </div>''',
+
+     css='''.sidebar { padding: 28px 8px 30px 0px;
+                       position: sticky; position: -webkit-sticky; top: 0;
+                       overflow-y:auto; overflow-x: auto }''')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Breadcrumb:
@@ -441,7 +458,12 @@ class Breadcrumb:
         # - "title" is the title of the object represented by this part;
         # - "url"   is the URL to this object.
         self.parts = None
-        self.compute(popup=popup)
+        # The CSS classes to apply to the main breacrumb tag
+        self.css = 'pageTitle bottomCell'
+        if popup: self.css += ' pageTitleP'
+        # No breadcrumb is computed for the tool
+        if o != o.tool:
+            self.compute(popup=popup)
 
     def compute(self, o=None, popup=False):
         '''Computes the breadcrumb to p_self.o, or add the part corresponding to
@@ -491,7 +513,7 @@ class Button:
         # CSS for a small button. No minimum width applies: small buttons are
         # meant to be small.
         if small:
-            part = (render == 'icon') and 'Icon' or 'Small'
+            part = 'Icon' if render == 'icon' else 'Small'
             return 'button%s button' % part
         # CSS for a normal button. A minimum width (via buttonFixed) is defined
         # when the label is small: it produces ranges of buttons of the same
@@ -516,8 +538,8 @@ class Footer:
 class Browser:
     '''Determines if the browser is compatible with Appy'''
 
-    ieRex = re.compile('MSIE\s+(\d\.\d)')
-    ieMin = '9' # We do not support IE below this version
+    ieRex = re.compile('MSIE\s+\d\.\d')
+    ieRex2 = re.compile('Trident.*rv\:')
 
     @classmethod
     def getIncompatibilityMessage(class_, tool, handler):
@@ -525,17 +547,9 @@ class Browser:
            with Appy.'''
         # Get the "User-Agent" request header
         agent = handler.headers.get('User-Agent')
-        r = class_.ieRex.search(agent)
-        if not r: return
-        version = r.group(1)
-        if (version < class_.ieMin) and ('ompatible' not in agent):
-            # If term "compatible" is found, it means that we have a "modern" IE
-            # but that is configured to mimick an older version. Because we have
-            # set a "X-UA-Compatible" directive in all Appy pages, normally,
-            # this kind of IE should not apply this old compatibility mode with
-            # Appy apps, so we do not display the warning in that case.
-            mapping = {'version': version, 'min': class_.ieMin}
-            return tool.translate('wrong_browser', mapping=mapping)
+        if not agent: return
+        if class_.ieRex.search(agent) or class_.ieRex2.search(agent):
+            return tool.translate('wrong_browser')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Columns:
@@ -603,4 +617,16 @@ class Columns:
         if addNumber and not numberFound:
             r.insert(0, class_.standard.number)
         return r
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+def getActionsCss(display):
+    '''Gets the CSS class to apply to the zone rendering object actions, having
+       the specified p_display.'''
+    if display == 'block':
+        r = 'objectActions'
+    else:
+        r = 'inline'
+        if display == 'right':
+            r += ' alignedActions'
+    return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
