@@ -1,4 +1,4 @@
-'''Module implementing CSS-related functionality'''
+'''Parsing of CSS properties'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Copyright (C) 2007-2020 Gaetan Delannay
@@ -91,14 +91,13 @@ def parseStyleAttribute(value, asDict=False):
     '''Returns a list of CSS (name, value) pairs (or a dict if p_asDict is
        True), parsed from p_value, which holds the content of a HTML "style"
        tag.'''
-    if asDict: res = {}
-    else:      res = []
+    r = {} if asDict else []
     for attr in value.split(';'):
         if not attr.strip(): continue
         name, value = attr.split(':', 1)
-        if asDict: res[name.strip()] = value.strip()
-        else:      res.append( (name.strip(), value.strip()) )
-    return res
+        if asDict: r[name.strip()] = value.strip()
+        else: r.append( (name.strip(), value.strip()) )
+    return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class CssValue:
@@ -233,6 +232,10 @@ class CssStyles:
     directions = ('top', 'right', 'bottom', 'left')
     # Values, on combined attributes, that prevent splitting
     unsplittable = ('auto',)
+    # Values that must be ignored
+    ignore = {'width': ['auto']}
+    # The '!important' CSS rule
+    importantRule = '!important'
 
     def __init__(self, elem=None, attrs=None, other=None, **kwargs):
         '''Analyses styles as found in p_attrs and/or deduced from XHTML element
@@ -336,6 +339,11 @@ class CssStyles:
            also return it. p_name can be the original name of the CSS attribute
            (possibly containing dashes).'''
         if name == 'classes': raise Exception(ADD_CLASS_ERROR)
+        # Ignore potential "!important" suffix
+        if isinstance(value, str) and value.endswith(self.importantRule):
+            value = value[:-len(self.importantRule)].strip()
+        # Do not add anything if the value must be ignored
+        if (name in self.ignore) and (value in self.ignore[name]): return
         attrName = name.replace('-', '')
         if not override and hasattr(self, attrName): return
         # Define the new value or combine it with the existing value when
