@@ -114,12 +114,11 @@ class Layout:
     # - symbols, allowing to define column or row breaks, alignment, etc.
 
     # Letters for a page -------------------------------------------------------
-    #  e - The object h*e*ader, containing its title, breadcrumb, siblings
-    #      navigation, phases and pages.
-    #  s - An object's hi*s*tory
+    #  e - The object h*e*ader, containing its title, breadcrumb, siblings and
+    #      navigation.
+    #  b - The range of *b*uttons (pages and phases, save, delete, actions and
+    #      workflow transitions.
     #  w - The fields (= *w*idgets) to render on some page for some object
-    #  b - The range of *b*uttons (intra-object navigation, save, edit,
-    #      delete...)
 
     # Letters for a field ------------------------------------------------------
     #  l - "label"        The field label
@@ -128,7 +127,7 @@ class Layout:
     #                     with a tooltip displaying online help
     #  v - "validation"   The icon that is shown when a validation error occurs
     #                     (typically only used on "edit" layouts)
-    #  r - "required"     The icon specifying that the field is required
+    #  r - "required"     The asterisc meaning that the field is required
     #                     (if relevant; typically only used on "edit" layouts)
     #  f - "field"        The field value, or input for entering a value.
     #  c - "changes"      The button for displaying changes to a field
@@ -137,8 +136,7 @@ class Layout:
     # hereabove letters.
     pxs = {
       # Page-related elements
-      'e': 'pxHeader', 's': ('history', 'view'), 'w': 'pxFields',
-      'b': 'pxButtons',
+      'e': 'pxHeader', 'w': 'pxFields', 'b': 'pxControls',
       # Field-related elements
       'l': 'pxLabel', 'd': 'pxDescription', 'h': 'pxHelp', 'v': 'pxValidation',
       'r': 'pxRequired', 'c': 'pxChanges'
@@ -225,7 +223,7 @@ class Layout:
         # The following attribute will store a special Row instance used for
         # defining column properties.
         self.headerRow = None
-        # The content rows will be stored hereafter.
+        # The content rows will be stored hereafter
         self.rows = []
         self.decodeRows(self.layoutString)
 
@@ -276,6 +274,19 @@ class Layout:
         if rowContent:
             self.rows.append(Row(rowContent, 'middle'))
 
+    def getRowIndex(self, char):
+        '''Gets the index of the row containing this p_char. If p_char is not
+           found, return -1.'''
+        # Get the PX corresponding to p_char
+        px = Layout.pxs[char]
+        i = 0
+        for row in self.rows:
+            for cell in row.cells:
+                if px in cell.content:
+                    return i
+            i += 1
+        return -1
+
     def removeElement(self, elem):
         '''Removes given p_elem from myself'''
         toRemove = Layout.pxs[elem]
@@ -521,7 +532,7 @@ class Layouts(dict):
 b = 'lrv-f'
 d = 'lrv-d-f'
 Layouts.b   = Layouts(Layout(b, width=None))
-Layouts.c   = Layouts(Layout(b + '|', width=None))
+Layouts.c   = Layouts(Layout('lrv|-f|', width=None, align='center'))
 Layouts.d   = Layouts(d)
 Layouts.dc  = Layouts(d + '|')
 Layouts.h   = Layouts('lhrv-f')
@@ -555,15 +566,11 @@ Layouts.defaults = {'normal': Layouts.b, 'grid': Layouts.g}
 
 class PageLayouts:
     # The default layouts
-    defaults = Layouts(edit=Layout('w-b', width=None), view=Layout('e-w-b'))
-    # With the page summary
-    summary = Layouts(edit=Layout('w-b', width=None), view=Layout('e-s-w-b'))
+    defaults = Layouts(edit=Layout('b-w', width=None), view=Layout('e-b-w'))
     # Wide
-    wide = Layouts(edit=Layout('w-b'), view=Layout('e-w-b'))
-    # Wide + summary
-    wideSummary = Layouts(edit=Layout('w-b'), view=Layout('s-w-b'))
+    wide = Layouts(edit=Layout('b-w'), view=Layout('e-b-w'))
     # Centered
-    centered = Layouts(edit=Layout('w|-b|', width=None, align='center'),
-                       view=Layout('e-w|-b|', align='center'))
+    centered = Layouts(edit=Layout('b|-w|', width=None, align='center'),
+                       view=Layout('e-b|-w|', align='center'))
 Layouts.Page = PageLayouts
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
