@@ -29,8 +29,11 @@ class Sibling:
     types = ('previous', 'next', 'first', 'last')
 
     # Names of icons corresponding to sibling types
-    icons = {'previous': 'arrowLeft',  'next': 'arrowRight',
-             'first':    'arrowsLeft', 'last': 'arrowsRight'}
+    icons = {'previous': 'arrow.svg',  'next': 'arrow.svg',
+             'first':    'arrows.svg', 'last': 'arrows.svg'}
+
+    # Rotations for icons
+    rotate = {'previous': 90, 'next': 270, 'first': 90, 'last': 270}
 
     def __init__(self, o, type, nav, page, siblings):
         # The main Siblings instance
@@ -76,9 +79,11 @@ class Sibling:
         # Build the JS code allowing to navigate to the sibling element
         js = "gotoSibling(%s,'%s','%s','%s')" % \
              (url, self.nav, self.page, self.popup)
-        return '<img src="%s" title="%s" class="clickable" onclick="%s"/>' % \
-               (ctx.url(Sibling.icons[self.type]),
-                ctx._('goto_%s' % self.type), js)
+        type = self.type
+        return '<img src="%s" title="%s" class="clickable iconXS" ' \
+               'style="transform: rotate(%ddeg)" onclick="%s"/>' % \
+               (ctx.url(Sibling.icons[type]), ctx._('goto_%s' % type),
+                Sibling.rotate[type], js)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Siblings:
@@ -92,15 +97,14 @@ class Siblings:
               False: ('nextSibling',  'lastSibling')}
 
     # Icons for going to the current object's siblings    
-    pxNavigate = Px('''
+    pxNavigate = Px('''<div>
       <!-- Go to the source URL (search or referred object) -->
-      <div if="not popup" class="goSource">::self.getGotoSource(url, _)</div>
+      <div if="not popup">::self.getGotoSource(url, _)</div>
 
       <!-- Show other navigation icons only when relevant -->
-      <x if="self.total">
+      <div if="self.total" class="navSib">
        <!-- Form used to navigate to any sibling -->
-       <form name=":self.siblingsFormName" method="post" action=""
-             style="display: inline">
+       <form name=":self.siblingsFormName" method="post" action="">
         <input type="hidden" name="page" value=""/>
         <input type="hidden" name="popup" value=""/>
         <input type="hidden" name="criteria" value=""/>
@@ -110,8 +114,7 @@ class Siblings:
        <x>::self.getIcons(_ctx_, previous=True)</x>
 
        <!-- Explain which element is currently shown -->
-       <span class="discreet"> 
-        <x>:self.number</x> <b>//</b> <x>:self.total</x> </span>
+       <x>:self.number</x> / <x>:self.total</x> 
 
        <!-- Go to the next and/or last page -->
        <x>::self.getIcons(_ctx_, previous=False)</x>
@@ -119,9 +122,8 @@ class Siblings:
        <!-- Go to the element number... -->
        <x if="self.showGotoNumber()"
           var2="field=self.field; sourceUrl=self.sourceObject.url;
-                total=self.total"><br/><x>:self.pxGotoNumber</x></x></x>''',
-
-     css=".goSource { font-weight: bold }",
+                total=self.total"><br/><x>:self.pxGotoNumber</x></x>
+      </div></div>''',
 
      js='''
        function gotoCustomSearch() {
@@ -240,12 +242,11 @@ class Siblings:
             sibling = Sibling(o, siblingType, nav % (index+1), page, self)
             setattr(self, name, sibling)
 
-    def getSuffixedBackText(self, _):
-        '''Gets the p_backText, produced by m_getBackText (see sub-classes),
-           suffixed with a standard text.'''
-        text = self.getBackText()
-        goto = _('goto_source')
-        return '%s - %s' % (text, goto) if text else goto
+    def getBackTexts(self, _):
+        '''Gets a tuple with the "back texts" = a tuple of 2 texts: the standard
+           text (ie, "back") and a more detailed text, based on the source ref
+           or search.'''
+        return _('goto_source'), self.getBackText() or ''
 
     def getIcons(self, ctx, previous=True):
         '''Produce icons for going to
@@ -260,8 +261,9 @@ class Siblings:
 
     def getGotoSource(self, url, _):
         '''Get the link allowing to return to the source URL'''
-        return '<a href="%s"><img src="%s"/>%s</a>' % \
-               (self.sourceUrl, url('gotoSource'), self.getSuffixedBackText(_))
+        text, message = self.getBackTexts(_)
+        return '<a href="%s" title="%s"><img src="%s" class="back"/>%s</a>'% \
+               (self.sourceUrl, message, url('arrowsA.svg'), text)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class RefSiblings(Siblings):
@@ -320,7 +322,7 @@ class RefSiblings(Siblings):
 
     def showGotoNumber(self):
         '''Show "goto number" if the Ref field is numbered'''
-        return self.field.isNumbered(self.sourceObject)
+        return self.field.isNumbered(self.sourceObject) and (self.total > 1)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class SearchSiblings(Siblings):
@@ -387,16 +389,17 @@ class SearchSiblings(Siblings):
             return Siblings.getGotoSource(self, url, _)
         # For a custom search, post a form with the search criteria retrieved
         # from the browser's session.
+        text, message = self.getBackTexts(_)
         return '<form method="post" action="%s" name="gotoSource">' \
           '<input type="hidden" name="className" value="%s"/>' \
           '<input type="hidden" name="search" value="customSearch"/>' \
           '<input type="hidden" name="start" value="%d"/>' \
           '<input type="hidden" name="ref" value="%s"/>' \
           '<input type="hidden" name="criteria" value=""/></form>' \
-          '<a class="clickable" onclick="gotoCustomSearch()">' \
-          '<img src="%s"/>%s</a>' % \
+          '<a class="clickable" title="%s" onclick="gotoCustomSearch()">' \
+          '<img src="%s" class="back"/>%s</a>' % \
           (self.sourceUrl, self.className, self.computeStartNumber(),
-           self.req.ref or '', url('gotoSource'), self.getSuffixedBackText(_))
+           self.req.ref or '', message, url('arrowsA.svg'), text)
 
     def showGotoNumber(self): return
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
