@@ -40,50 +40,62 @@ class Portlet:
                if hasattr(tool, 'showPortletAt') else True
 
     px = Px('''
-     <x var="toolUrl=tool.url;
-             queryUrl='%s/Search/results' % toolUrl;
-             currentSearch=req.search;
-             currentClass=req.className;
-             currentPage=handler.parts[-1];
-             rootClasses=handler.server.model.getRootClasses()">
+     <div var="collapse=ui.Collapsible.get('portlet', dleft, req);
+           toolUrl=tool.url;
+           queryUrl='%s/Search/results' % toolUrl;
+           cfg=config.ui;
+           currentSearch=req.search;
+           currentClass=req.className;
+           currentPage=handler.parts[-1];
+           rootClasses=handler.server.model.getRootClasses()"
+      id="appyPortlet" class="portlet" style=":collapse.style">
+
+      <!-- The portlet logo when present -->
+      <center>
+       <img if="cfg.portletLogo" src=":cfg.getImageUrl(siteUrl, 'portletLogo')"
+            class="portletLogo"/></center>
 
       <!-- One section for every searchable root class -->
       <x for="class_ in rootClasses" if="class_.maySearch(tool, layout)"
          var2="className=class_.name">
 
        <!-- A separator if required -->
-       <div class="portletSep" if="not loop.class_.first"></div>
+       <div class="portletSep"
+            if="not (loop.class_.first and not cfg.portletLogo)"></div>
 
        <!-- Section title (link triggers the default search) -->
        <div class="portletContent"
-            var="searches=class_.getGroupedSearches(tool, _ctx_)">
+            var="searches=class_.getGroupedSearches(tool, _ctx_);
+                 labelPlural=_(className + '_plural');
+                 indexable=class_.isIndexable()">
         <div class="portletTitle">
-         <a var="queryParam=searches.default.name if searches.default else ''"
+         <a if="indexable"
+            var="queryParam=searches.default.name if searches.default else ''"
             href=":'%s?className=%s&amp;search=%s' % \
                    (queryUrl, className, queryParam)"
             onclick="clickOn(this)"
             class=":(not currentSearch and (currentClass==className) and \
                     (currentPage == 'pxResults')) and \
-                    'current' or ''">::_(className + '_plural')</a>
-
-         <!-- Create instances of this class -->
-         <x if="guard.mayInstantiate(class_)"
-            var2="asButton=False; nav='no'">:class_.pxAdd</x>
+                    'current' or ''">::labelPlural</a>
+         <x if="not indexable">::labelPlural</x>
         </div>
 
+        <!-- Create instances of this class -->
+        <div if="guard.mayInstantiate(class_)"
+             var2="asButton=False; nav='no'">:class_.pxAdd</div>
+
         <!-- Searches -->
-        <x if="class_.maySearchAdvanced(tool)">
+        <x if="indexable and class_.maySearchAdvanced(tool)">
 
          <!-- Live search -->
-         <x>:tool.Search.live</x>
+         <x var="pre=className; liveCss='lsSearch'">:tool.Search.live</x>
 
          <!-- Advanced search -->
          <div var="highlighted=(currentClass == className) and \
-                               (currentPage == 'search')"
-              class=":highlighted and 'portletSearch current' or \
-                     'portletSearch'"
-              align=":dright" style="margin-bottom: 4px">
-          <a var="text=_('search_title')" style="font-size: 88%"
+                               (req.search == 'customSearch')"
+              class=":'portletAdv current' if highlighted else 'portletAdv'">
+          <img src=":url('search.svg')" class="iconM"/>
+          <a var="text=_('search_title')"
              href=":'%s/Search/advanced?className=%s' % (toolUrl, className)"
              title=":text"><x>:text</x>...</a>
          </div>
@@ -95,27 +107,31 @@ class Portlet:
         </x>
 
         <!-- Portlet bottom, potentially customized by the app -->
-        <x var="pxBottom=class_.getPortletBottom(tool)"
+        <x var="pxBottom=getattr(class_.python, 'portletBottom', None)"
            if="pxBottom">:pxBottom</x>
        </div>
       </x>
-     </x>''',
+     </div>''',
 
      css='''
-       .portlet { width: 170px; border: none; color: white;
-                  background-color: #8399b7; padding-top: 30px;
-                  vertical-align: top; margin-bottom: 30px; position: relative }
-       .portlet a, .portlet a:visited { color: white; padding: 0px 5px 0 0 }
-       .portlet a:hover { background-color: white; color: #305e9d }
-       .portletContent { padding: 0 0 0 20px; background: none; width: 180px }
+       .portlet { padding: 30px 50px 0 0; box-shadow: 2px 2px 5px #002039;
+                  position: sticky; position: -webkit-sticky; top: 0;
+                  overflow-y:auto; overflow-x: auto; width: 250px;
+                  min-width: 140px; font-size: 98%; z-index: 1 }
+       .portletSearch a:hover { background-color:|linkColor|;
+                  color:|headerColor| }
+       .portletContent { padding-left: 18px; width: 200px }
        .portletContent input[type=text] { background-color: white }
-       .portletTitle { font-size: 110%; padding: 5px 0; margin: 0;
-                       text-transform: uppercase }
-       .portletSep { border-top: 10px solid transparent }
+       .portletTitle { padding: 5px 0; text-transform: uppercase }
+       .portletLogo { margin-bottom: 15px }
+       .portletSep { border-top: 1px solid #002039; margin: 25px 0 20px 0 }
        .portletGroup { text-transform: uppercase; padding: 5px 0 0 0;
-                       margin: 0.1em 0 0.3em }
-       .portletSearch { font-size: 110%; text-align: left }
+                  margin: 0.5em 2px 0.3em }
+       .portletAdv { margin: 3px 0 10px }
+       .portletAdv img { margin: -2px 4px 0 1px; float: left}
+       .portletSearch { text-align: left }
        .portletCurrent { font-weight: bold }
-       .portlet form { margin-left: -3px }
+       .portlet form { margin-top: 5px }
+       input.buttonPortlet { border: 0; background-position: 0 }
      ''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
