@@ -1,10 +1,21 @@
-var loadingLink = '<img src="/static/appy/loading.gif"/>',
-    loadingButton = '<img align="center" src="/static/appy/loadingBtn.gif"/>',
-    loadingIcon = '<img align="center" src="/static/appy/loadingPod.gif"/>',
-    loadingZone = '<div align="center"><img src="/static/appy/?.gif"/></div>',
-    lsTimeout, // Timout for the live search
+var lsTimeout,  // Timout for the live search
     podTimeout; // Timeout for checking status of pod downloads
 
+// Builds the URL to a static resource named p_name
+function buildUrl(name) { return siteUrl + '/static/appy/' + name; }
+
+// Compute a chunk of XHTML code containing a "loading" animated icon
+function loading(name, icon) {
+ // Get the name of the icon to use: p_icon or a default one, based on p_name
+ if (!icon) icon = 'loading_' + name + '.gif';
+ var img = `<img src="${buildUrl(icon)}"`, r=null;
+ if      (name == 'link')   r = `${img}/>`;
+ else if (name == 'button') r = `${img} align="center"/>`;
+ else if (name == 'icon')   r = `${img} align="center"/>`;
+ else if (name == 'zone')   r = `<div align="center">${img}/></div>`;
+ return r
+}
+
 // Add to form p_f a hidden field named p_name with this p_value
 function addFormField(f, name, value) {
   // If a field named p_name already exists, simply set its value to p_value
@@ -182,9 +193,9 @@ function showPreloader(hook, waiting) {
   /* p_hook may be null if the ajax result would be the same as what is
      currently shown, as when inline-editing a rich text field). */
   if (!hook || (waiting == 'none')) return;
-  // What waiting icon to show?
-  if (!waiting) waiting = 'loadingBig';
-  injectChunk(getNode(hook), loadingZone.replace('?', waiting), true);
+  // What waiting icon to show ?
+  if (!waiting) waiting = 'loading_big.gif';
+  injectChunk(getNode(hook), loading('zone', waiting), true);
 }
 
 function askAjaxChunk(url, mode, params, hook, beforeSend, onGet, waiting) {
@@ -209,9 +220,9 @@ function askAjaxChunk(url, mode, params, hook, beforeSend, onGet, waiting) {
      HTML node element into which the result has been inserted.
 
      p_waiting is the name of the animated icon that will be shown while waiting
-     for the ajax result. If null, it will be loadingBig.gif. Other values can
-     be "loading", "loadingBtn" or "loadingPod" (the .gif must be omitted).
-     If "none", there will be no icon at all.
+     for the ajax result. If null, it will be "loading_big.gif". Other values
+     can be "loading_link.gif", "loading_button.gif" or "loading_icon.gif".
+     If string "none" is passed, there will be no icon at all.
   */
   // First, get a non-busy XMLHttpRequest object
   var sync = hook[0] == '*', pos = -1;
@@ -321,10 +332,16 @@ function askAjax(hook, form, params, waiting) {
   // If a p_form id is given, integrate the form submission in the ajax request
   if (form) {
     var f = document.getElementById(form),
-        mode = 'POST';
-    // Deduce the action from the form action
-    if (f.action != 'none'){
-      var parts = _rsplit(f.action, '/', 3).slice(1);
+        mode = 'POST',
+        action = f.action,
+        saction = f.getAttribute('data-sub');
+    /* Deduce the action from the form action, either via custom field
+       "action-sub", already containing the correct action sub-path, or by
+       extracting it from the "action" field. */
+    if (saction) d.params['action'] = saction;
+    else if (action != 'none'){
+      var i = (action.indexOf('@') == -1)? 3: 4,
+          parts = _rsplit(action, '/', i).slice(1);
       d.params['action'] = parts.join('*');
     }
     // Get the other params
@@ -407,6 +424,7 @@ function askBunchSwitchColset(hook, colset) {
 }
 
 function clickOn(node) {
+  if (!node) return;
   // If node is a form, disable all form buttons
   if (node.tagName == 'FORM') {
     var i = node.elements.length -1;
@@ -422,10 +440,10 @@ function clickOn(node) {
   /* For a button, show the preloader directly. For a link, show it only after
      a while, if the target page is still not there. */
   if (node.tagName == 'A') {
-    setTimeout(function(){injectChunk(node, loadingLink)}, 700);}
+    setTimeout(function(){injectChunk(node, loading('link'))}, 700);}
   else {
-    var loading = (cn.search('buttonIcon') == -1)? loadingButton : loadingIcon;
-    injectChunk(node, loading);
+    var name = (cn.search('buttonIcon') == -1)? 'button' : 'icon';
+    injectChunk(node, loading(name));
   }
 }
 
@@ -435,7 +453,7 @@ function gotoTied(objectUrl, field, numberWidget, total, popup) {
     var number = parseInt(numberWidget.value);
     if (!isNaN(number)) {
       if ((number >= 1) && (number <= total)) {
-        goto(objectUrl + '/gotoTied?field=' + field + '&number=' + number +
+        goto(objectUrl + '/' + field + '/gotoTied?number=' + number +
              '&popup=' + popup);
       }
       else numberWidget.style.background = wrongTextInput; }
@@ -731,9 +749,10 @@ function getMasterValues(master) {
     /* Get the selected value among all radio buttons of the group (p_master is
        the first one from this group) */
     var radios = document.getElementsByName(master.name);
+    res = [];
     for (var i=0; i < radios.length; i++) {
       if (radios[i].checked) {
-        res = [radios[i].value];
+        res.push(radios[i].value);
         break;
       }
     }
@@ -1037,11 +1056,9 @@ function updateFileNameStorer(field, storer) {
   storer.value = name;
 }
 
-function onUnlockPage(objectUid, pageName) {
-  var f = document.getElementById('unlockForm');
-  f.objectUid.value = objectUid;
-  f.pageName.value = pageName;
-  askConfirm('form', 'unlockForm', action_confirm);
+function onUnlockPage(objectUrl, page) {
+  var code = "post('" + objectUrl + "/unlock', {'page': '" + page + "'})";
+  askConfirm('script', code, action_confirm);
 }
 
 function createCookie(name, value, days) {
@@ -1131,15 +1148,15 @@ function openPopup(popupId, msg, width, height, back, commentLabel) {
       frame = popupId == 'iframePopup'; // Is it the "iframe" popup ?
   /* Define height and width. For non-iframe popups, do not set its height: it
      will depend on its content. */
-  if (!width)  { width =  (frame)? window.innerWidth -200: 350 }
+  if (!width)  { width =  (frame)? window.innerWidth -200: null }
   if (!height) { height = (frame)? window.innerHeight-200: null }
-  popup.style.width = width.toFixed() + 'px';
+  if (width) popup.style.width = width.toFixed() + 'px';
   if (height) popup.style.height = height.toFixed() + 'px';
   if (frame) {
     // Set the enclosed iframe dimensions
     var iframe = document.getElementById('appyIFrame');
     iframe.style.width = (width - 20).toFixed() + 'px';
-    iframe.style.height = height.toFixed() + 'px';
+    iframe.style.height = (height - 40).toFixed() + 'px';
     popup['back'] = back;
   }
   popup.style.display = 'block';
@@ -1165,7 +1182,7 @@ function closePopup(popupId, clean, tryCancel) {
       var iframe = popup.getElementsByTagName('iframe')[0],
           cancels = iframe.contentDocument.getElementsByName('cancel'),
           cancel = (cancels.length > 0)? cancels[0]: null;
-      if (cancel && (cancel.type == 'button')) {
+      if (cancel && (cancel.tagName == 'A')) {
         cancel.click();
         canceled = true;
       }
@@ -1358,8 +1375,9 @@ function onSelectObjects(popupId, initiatorId, objectUrl, mode, onav,
   closePopup('iframePopup');
   /* When refreshing the Ref field we will need to pass all those parameters,
      for replaying the popup query. */
-  var params = {'selected': uids, 'semantics': semantics, 'sortKey': sortKey,
-                'sortOrder': sortOrder, 'filters': filters};
+  var params = {'selected': uids, 'semantics': semantics,
+                'sortKey': sortKey || '', 'sortOrder': sortOrder,
+                'filters': filters || ''};
   if (onav) params['nav'] = onav;
   if (mode == 'repl') {
     /* Link the selected objects (and unlink the potentially already linked
@@ -1373,21 +1391,33 @@ function onSelectObjects(popupId, initiatorId, objectUrl, mode, onav,
   }
 }
 
-function onSelectObject(checkboxId, initiatorId, objectUrl) {
-  /* In a Ref field with link="popup", a single object has been clicked. If
-     multiple objects can be selected, simply update the corresponding checkbox
-     status. Else, close the popup and return the selected object. */
-  var checkbox = document.getElementById(checkboxId),
-      visible = checkbox.parentNode.className != 'hide';
-  // If the td is visible, simply click the checkbox
-  if (visible) checkbox.click();
+function onSelectObject(checkboxId, initiatorId, id, ckNum) {
+  // An object, whose ID is p_id, has been selected in a popup
+  var checkbox = document.getElementById(checkboxId);
+  if (ckNum) {
+    // The object has been selected from a popup opened by ckeditor
+    var imageUrl = siteUrl + '/' + checkbox.value + '/file/download';
+    window.opener.CKEDITOR.tools.callFunction(ckNum, imageUrl);
+    window.close();
+  }
   else {
-    /* Close the popup and directly refresh the initiator field with the
-       selected object. */
-    var uids = checkbox.value;
-    closePopup('iframePopup');
-    var params = {'selected': uids, 'semantics': 'checked'};
-    askField(':'+initiatorId, objectUrl, 'edit', params, false);
+    /* The object has been selected from a Ref with link="popup[Ref]", in a
+       popup displaying search results or objects from another Ref. A single
+       object has been clicked. If multiple objects can be selected, simply
+       update the corresponding checkbox status. Else, close the popup and
+       return the selected object. */
+    var checkbox = document.getElementById(checkboxId),
+        visible = checkbox.parentNode.className != 'hide';
+    // If the td is visible, simply click the checkbox
+    if (visible) checkbox.click();
+    else {
+      /* Close the popup and directly refresh the initiator field with the
+         selected object. */
+      var uids = checkbox.value;
+      closePopup('iframePopup');
+      var params = {'selected': uids, 'semantics': 'checked'};
+      askField(':'+initiatorId, siteUrl + '/' + id, 'edit', params, false);
+    }
   }
 }
 
@@ -1447,6 +1477,4 @@ function dragIt(event) {
   event.preventDefault();
 }
 
-function dragPropose(event) {
-  event.target.style.cursor = 'move';
-}
+function dragPropose(event) { event.target.style.cursor = 'move'; }
