@@ -18,7 +18,7 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from base64 import encodebytes
 from mimetypes import guess_type
-import xml.sax, http.client, urllib, urllib.parse
+import xml.sax, http.client, urllib, urllib.parse, ssl
 import os, sys, re, stat, time, socket, random, hashlib
 
 from appy.model.utils import Object
@@ -311,7 +311,7 @@ class HttpResponse:
             if not contentType.startswith(xmlHeader): continue
             # Return an unmarshalled version of the XML content, for easy use
             try:
-                parser = Unmarshaller(utf8=self.utf8)
+                parser = Unmarshaller()
                 res = parser.parse(self.body)
                 if parser.rootTag == 'exception':
                     # This is an exception: "res" contains the traceback
@@ -393,23 +393,28 @@ class Resource:
         if not (self.username and self.password): return
         if 'Authorization' in headers: return
         if self.authMethod == 'Basic':
-            credentials = '%s:%s' % (self.username, self.password)
-            credentials = credentials.replace('\012', '')
-            headers['Authorization'] = "%s %s" % (self.authMethod,
-                                                  encodebytes(credentials))
+            creds = ('%s:%s' % (self.username, self.password)).encode()
+            authorization = '%s %s' % (self.authMethod,
+                                       encodebytes(creds).strip().decode())
+            headers['Authorization'] = authorization
         headers['Connection'] = 'close'
         headers['Accept'] = '*/*'
 
     def send(self, method, uri, body=None, headers=None, bodyType=None,
              responseType=None):
-        '''Sends a HTTP request with p_method, for p_uri.'''
-        connectionClass = http.client.HTTPSConnection \
-          if self.protocol == 'https' else http.client.HTTPConnection
-        conn = connectionClass(self.host, self.port, timeout=self.timeout)
+        '''Sends a HTTP request with p_method, for p_uri'''
+        hc = http.client
+        if self.protocol == 'https':
+            context = ssl.SSLContext()
+            context.verify_mode = ssl.CERT_NONE
+            conn = hc.HTTPSConnection(self.host, self.port,
+                                      timeout=self.timeout, context=context)
+        else:
+            conn = hc.HTTPConnection(self.host, self.port, timeout=self.timeout)
         try:
             conn.connect()
         except socket.gaierror as sge:
-            raise self.Error('Check your Internet connection (%s)'% str(sge))
+            raise self.Error('Check your Internet connection (%s)' % str(sge))
         except socket.timeout as se:
             raise self.Error(self.TIMEOUT_ERROR % conn.timeout)
         except socket.error as se:
@@ -456,37 +461,19 @@ class Resource:
         toDeleteUri = self.uri + '/' + name
         return self.send('DELETE', toDeleteUri)
 
-    def add(self, content, type='fileName', name=''):
-        '''Adds a file in this resource. p_type can be:
-           - "fileName"  In this case, p_content is the path to a file on disk
-                         and p_name is ignored;
-           - "zope"      In this case, p_content is an instance of
-                         OFS.Image.File and the name of the file is given in
-                         p_name.
-        '''
-        if type == 'fileName':
-            # p_content is the name of a file on disk
-            size = os.stat(content)[stat.ST_SIZE]
-            body = open(content, 'rb')
-            name = os.path.basename(content)
-            fileType, encoding = guess_type(content)
-            bodyType = 'file'
-        elif type == 'zope':
-            # p_content is a "Zope" file, ie a OFS.Image.File instance
-            # p_name is given
-            fileType = content.content_type
-            encoding = None
-            size = content.size
-            body = content
-            bodyType = 'zope'
+    def add(self, path, name=''):
+        '''Adds a file in this resource, whose p_path is passed'''
+        size = os.stat(path)[stat.ST_SIZE]
+        body = open(path, 'rb')
+        name = os.path.basename(path)
+        fileType, encoding = guess_type(path)
         fileUri = self.uri + '/' + name
         headers = {'Content-Length': str(size)}
         if fileType: headers['Content-Type'] = fileType
         if encoding: headers['Content-Encoding'] = encoding
-        res = self.send('PUT', fileUri, body, headers, bodyType=bodyType)
-        # Close the file when relevant
-        if type =='fileName': body.close()
-        return res
+        r = self.send('PUT', fileUri, body, headers, bodyType='file')
+        body.close()
+        return r
 
     def get(self, uri=None, headers=None, params=None, followRedirect=True,
             responseType=None):
