@@ -49,12 +49,14 @@ class Config:
     testable = False
 
     def __init__(self, fromName=None, fromEmail='info@appyframework.org',
-                 server='localhost', port=25, secure=False,
+                 replyTo=None, server='localhost', port=25, secure=False,
                  login=None, password=None, enabled=True):
         # The name that will appear in the "from" part of the messages
         self.fromName = fromName
         # The mail address that will appear in the "from" part of the messages
         self.fromEmail = fromEmail
+        # The optional "reply-to" mail address
+        self.replyTo = replyTo
         # The SMTP server address and port
         if ':' in server:
             self.server, port = server.split(':')
@@ -111,6 +113,7 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
         return
     # Just log things if mail is disabled
     fromAddress = config.getFrom()
+    replyTo = replyTo or config.replyTo
     if not config.enabled or not config.server:
         msg = '' if config.server else ' (%s)' % NO_SERVER
         if log:
@@ -123,14 +126,14 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
         return
     if log: log(MSG_SENDING % (fromAddress, str(to), subject))
     # Create the base MIME message
-    body = MIMEText(body, 'plain', 'utf-8')
+    body = MIMEText(body, 'plain')
     if attachments:
         msg = MIMEMultipart()
         msg.attach(body)
     else:
         msg = body
     # Add the header values
-    msg['Subject'] = Header(subject, 'utf-8')
+    msg['Subject'] = Header(subject)
     msg['From'] = fromAddress
     msg['Date'] = formatdate(localtime=True)
     if replyTo: msg.add_header('reply-to', replyTo)
