@@ -20,7 +20,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re, cgi
+import re, html
 from appy.utils import flipDict, asDict
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -221,7 +221,7 @@ def formatText(text, format='html'):
     '''Produces a representation of p_text into the desired p_format, which
        is "html" by default.'''
     if 'html' in format:
-        if format == 'html_from_text': text = cgi.escape(text)
+        if format == 'html_from_text': text = html.escape(text)
         r = text.replace('\r\n', '<br/>').replace('\n', '<br/>')
     elif format == 'text':
         r = text.replace('<br/>', '\n')
@@ -375,26 +375,35 @@ def upper(s):
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class WhitespaceCruncher:
     '''Takes care of removing unnecessary whitespace in several contexts'''
-    whitechars = ' \r\t\n' # Chars considered as whitespace
-    allWhitechars = whitechars + ' ' # nbsp
 
-    @staticmethod
-    def crunch(s, previous=None):
-        '''Return a version of string p_s where all "whitechars" are:
-           * converted to real whitespace;
-           * reduced in such a way that there cannot be 2 consecutive
-             whitespace chars.
-           If p_previous is given, those rules must also apply globally to
-           previous+s.'''
+    # Base chars considered as whitespace
+    baseChars = ' \r\t\n'
+
+    # Whitechars, as a dict
+    whiteChars = asDict(baseChars)
+
+    # Whitechars, including the non-blocking space (nbsp)
+    allChars = asDict(baseChars + ' ')
+
+    @classmethod
+    def crunch(class_, s, previous=None):
+        '''Return a version of string p_s where "whitechars" are removed'''
+        # More precisely, whitechars, as listed in p_class_.whiteChars, are:
+        # * converted to real whitespace;
+        # * reduced in such a way that there cannot be 2 consecutive
+        #   whitespace chars.
+        # ~~~
+        # If p_previous is given, those rules must also apply globally to
+        # previous+s.
         r = ''
         # Initialise the previous char
         previousChar = previous[-1] if previous else ''
+        wchars = class_.whiteChars
         for char in s:
-            if char in WhitespaceCruncher.whitechars:
+            if char in wchars:
                 # Include the current whitechar in the result if the previous
                 # char is not a whitespace or nbsp.
-                if not previousChar or \
-                   (previousChar not in WhitespaceCruncher.allWhitechars):
+                if not previousChar or (previousChar not in class_.allChars):
                     r += ' '
             else: r += char
             previousChar = char
